/*
 * Copyright 2006-2007 Nuova Systems, Inc. All Rights Reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: bootarb.S 6 2016-12-02 03:26:55Z buildsa $"

#include <config.h>
#include <version.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>
#include <asm/addrspace.h>
#include <palo/fw/mcpu/include/pca9541.h>
#include "booti2c.h"
#include "palodtls.h"

#define is_buson(reg, resultreg)                                \
    andi    REGADDR_REG, reg, ARB_CON_BUSON;                    \
    sltu    REGADDR_REG, zero, REGADDR_REG;                     \
    andi    REGDATA_REG, reg, ARB_CON_NBUSON;                   \
    sltu    REGDATA_REG, zero, REGDATA_REG;                     \
    xor     resultreg, REGADDR_REG, REGDATA_REG

#define is_mybus(reg, resultreg)                                \
    andi    REGADDR_REG, reg, ARB_CON_MYBUS;                    \
    sltu    REGADDR_REG, zero, REGADDR_REG;                     \
    andi    REGDATA_REG, reg, ARB_CON_NMYBUS;                   \
    sltu    REGDATA_REG, zero, REGDATA_REG;                     \
    xor     resultreg, REGADDR_REG, REGDATA_REG;                \
    xori    resultreg, resultreg, 0x1

/*
 * Routine: bootarb_take_i2c_bus_control
 *
 * Input: None
 *
 * Output: None
 *
 * Purpose: acquire the downstream i2c bus on arbiter specified by 
 * address TADDR_REG
 *          
 */

# void
# bootarb_take_i2c_bus_control(void)
# {
    .globl  bootarb_take_i2c_bus_control
    .ent    bootarb_take_i2c_bus_control
bootarb_take_i2c_bus_control:

    move    BOOTARB_TAKE_I2C_BUS_CONTROL_RA, ra      /* save return address */

    BAL     (booti2c_init_master)
    nop

    li      EEP_OFFSET_REG, ARB_CON             /* read ARB_CON    */
    li      EEP_ADDRWIDTH_REG, ADDR_WIDTH_8     /* addr width 8-bit */
    BAL     (booti2c_master_read)
    nop

    bne     STATUS_REG, BOOTI2C_STATUS_SUCCESS, bootarb_take_i2c_bus_control_exit
    nop

    /*
     * check BUSON and NBUSON
     */

    is_buson(EEP_DATA_REG, TDATA_REG1)

    beqz    TDATA_REG1, loop1
    nop

    /* check MYBUS and NMYBUS */

    is_mybus(EEP_DATA_REG, TDATA_REG1)

    bnez    TDATA_REG1, loop1
    nop
    
    /*
     * sleep for 250 msec
     */

    li      DELAYTIME_REG, 250000
    BAL     (booti2c_delay_usec)
    nop

    /*
     * re-read the arb-control register
     */

    li      EEP_OFFSET_REG, ARB_CON
    li      EEP_ADDRWIDTH_REG, ADDR_WIDTH_8
    BAL     (booti2c_master_read)
    nop

    bne     STATUS_REG, BOOTI2C_STATUS_SUCCESS, bootarb_take_i2c_bus_control_exit
    nop
    
    /*
     * for (ARB_LOOP_CNT_REG = 3 ; ARB_LOOP_CNT_REG > 0 ; ARB_LOOP_CNT_REG--) {
     */

    li      ARB_LOOP_CNT_REG, ARB_MAX_ACQUIRE_ATTEMPTS

loop1:
    move    TDATA_REG2, EEP_DATA_REG        /* copy value of ARB_CON register */
    is_buson(EEP_DATA_REG, TDATA_REG1)

    bnez    TDATA_REG1, check_mybus
    nop
    xor     EEP_DATA_REG, EEP_DATA_REG, ARB_CON_BUSON

check_mybus:
    is_mybus(EEP_DATA_REG, TDATA_REG1)

    bnez    TDATA_REG1, 1f
    nop
    xor     EEP_DATA_REG, EEP_DATA_REG, ARB_CON_MYBUS

1:
    /*
     * compare the 2 control values
     */

    bne     TDATA_REG2, EEP_DATA_REG, 1f
    nop

    /* both are equal, return success */

    li      STATUS_REG, BOOTI2C_STATUS_SUCCESS
    b       bootarb_take_i2c_bus_control_exit  
    nop

1:
    /*
     * write control register to take the bus with BUSINIT
     */

    li      EEP_OFFSET_REG, ARB_CON
    li      EEP_ADDRWIDTH_REG, ADDR_WIDTH_8
    or      EEP_DATA_REG, EEP_DATA_REG, ARB_CON_BUSINIT
    BAL     (booti2c_master_write)
    nop

    bne    STATUS_REG, BOOTI2C_STATUS_SUCCESS,  bootarb_take_i2c_bus_control_i2c_access_failed
    nop

    /*
     * read the control register back to see if it got stuck (2 masters)
     */

    li      EEP_OFFSET_REG, ARB_CON
    li      EEP_ADDRWIDTH_REG, ADDR_WIDTH_8
    BAL     (booti2c_master_read)
    nop

    /*
     * if i2c access failed, return
     */

    bne    STATUS_REG, BOOTI2C_STATUS_SUCCESS, bootarb_take_i2c_bus_control_i2c_access_failed
    nop

    is_buson(EEP_DATA_REG, TDATA_REG1)
    beqz    TDATA_REG1, bootarb_take_i2c_bus_control_exit
    nop

    is_mybus(EEP_DATA_REG, TDATA_REG1)
    bnez    TDATA_REG1, bootarb_take_i2c_bus_control_exit
    nop


    li      DELAYTIME_REG, 250000
    BAL     (booti2c_delay_usec)
    nop

    li      EEP_OFFSET_REG, ARB_CON
    li      EEP_ADDRWIDTH_REG, ADDR_WIDTH_8
    BAL     (booti2c_master_read)
    nop

    /*
     * if i2c access failed, return
     */

    bne     STATUS_REG, BOOTI2C_STATUS_SUCCESS, bootarb_take_i2c_bus_control_i2c_access_failed
    nop

    /*
     * read success, check max tries
     */

    sub     ARB_LOOP_CNT_REG, ARB_LOOP_CNT_REG, 1   /* count-- */
    bnez    ARB_LOOP_CNT_REG, loop1
    nop

    /*
     * } <-- end of for loop
     */

    li      STATUS_REG, BOOTI2C_STATUS_MAXTRIES
    b       bootarb_take_i2c_bus_control_exit
    nop

bootarb_take_i2c_bus_control_i2c_access_failed:

bootarb_take_i2c_bus_control_exit:
    /*
     * restore the ra and return
     */

    move    ra, BOOTARB_TAKE_I2C_BUS_CONTROL_RA
    jr      ra

    nop
    .end    bootarb_take_i2c_bus_control
# }

/*
 * Routine: bootarb_release_i2c_bus_control
 *
 * Input: None
 *
 * Output: None
 *
 * Purpose: release the downstream i2c bus on arbiter specified by 
 * address TADDR_REG
 *          
 */

# void
# bootarb_release_i2c_bus_control(void)
# {
    .globl  bootarb_release_i2c_bus_control
    .ent    bootarb_release_i2c_bus_control
bootarb_release_i2c_bus_control:

    move    BOOTARB_TAKE_I2C_BUS_CONTROL_RA, ra      /* save return address */

    BAL     (booti2c_init_master)
    nop

    li      EEP_OFFSET_REG, ARB_CON             /* read ARB_CON    */
    li      EEP_ADDRWIDTH_REG, ADDR_WIDTH_8     /* addr width 8-bit */
    BAL     (booti2c_master_read)
    nop

    
    bne     STATUS_REG, BOOTI2C_STATUS_SUCCESS, bootarb_release_i2c_bus_control_exit
    nop

    is_buson(EEP_DATA_REG, TDATA_REG1)

    beqz    TDATA_REG1, bootarb_take_i2c_bus_control_exit
    nop

    is_mybus(EEP_DATA_REG, TDATA_REG1)

    beqz    TDATA_REG1, bootarb_take_i2c_bus_control_exit
    nop

    /*
     * Both bus-on and my-bus conditions are true
     * negate the BUS-ON bit (xor will do that) and 
     * write the CNTL register back.
     */

    xor     EEP_DATA_REG, EEP_DATA_REG, ARB_CON_BUSON
    li      EEP_OFFSET_REG, ARB_CON
    li      EEP_ADDRWIDTH_REG, ADDR_WIDTH_8
    BAL     (booti2c_master_write)
    nop

    bne     STATUS_REG, BOOTI2C_STATUS_SUCCESS, bootarb_release_i2c_bus_control_exit
    nop

    /*
     * Flush the interrupt status register.
     */

    li      EEP_OFFSET_REG, ARB_ISTAT
    li      EEP_ADDRWIDTH_REG, ADDR_WIDTH_8
    BAL     (booti2c_master_read)
    nop

    
bootarb_release_i2c_bus_control_exit:
    /*
     * restore the ra and return
     */

    move    ra, BOOTARB_TAKE_I2C_BUS_CONTROL_RA
    jr      ra

    nop
    .end    bootarb_release_i2c_bus_control
# }
    
/* vi:set filetype=c: */
