/*
 * Copyright 2006-2007 Nuova Systems, Inc. All Rights Reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: booti2c.S 21 2016-12-02 21:25:59Z buildsa $"

#include <config.h>
#include <version.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>
#include <asm/addrspace.h>
#include <palo/include/synopsis/dw_i2c.h>
#include <palo/include/synopsis/dw_gpio.h>
#include <palo/include/palo_impl.h>
#include <palo/include/palo_gpio.h>
#include <palo/include/misc_mregs.h>
#include <palo/include/palo_i2ctiming.h>
#include <palo/fw/mcpu/include/pca9541.h>
#include <common/include/mezzi2c.h>
#include "booti2c.h"
#include "palodtls.h"

/*
 * This file assumes PL_PA_GPIO_I2C0_SCL (GPIO 0) as the I2C pin.
 */

        .set    mips32r2

/*
 * Routine: booti2c_init
 *
 * Input: None
 *
 * Output: None
 *
 * Purpose: initialize the i2c port 0 interface with an address in 
 *          a given register a0.
 *
 * Register usage:
 *    TADDR_REG   : target addr to program in the TAR
 *    REGADDR_REG : register addresses
 *    REGDATA_REG : register values
 */

# void
# booti2c_init_master(void)
# {
    .globl  booti2c_init_master
booti2c_init_master:
    /*
     * disable i2c interface before configuring the interface
     */

    li      REGDATA_REG, 0
    REG_WRITE(IC_ENABLE(0), REGDATA_REG)

    /*
     * enable 7-bit addressing and standard speed
     */

    li      REGDATA_REG, IC_CON_SLAVE_DISABLE | IC_CON_RESTART_EN | \
                    IC_CON_SPEED_STANDARD | IC_CON_MASTER_MODE
    REG_WRITE(IC_CON(0), REGDATA_REG)

    /*
     * program standard speed scl high count
     */

    li      REGDATA_REG, SS_SCL_HCNT
    REG_WRITE(IC_SS_HCNT(0), REGDATA_REG)

    /*
     * program standard speed scl low count
     */

    li      REGDATA_REG, SS_SCL_LCNT
    REG_WRITE(IC_SS_LCNT(0), REGDATA_REG)

    /*
     * program target address
     */

    REG_WRITE(IC_TAR(0), TADDR_REG)

    /*
     * enable i2c interface
     */

    li      REGDATA_REG, 1
    REG_WRITE(IC_ENABLE(0), REGDATA_REG)

    jr      ra
    nop
# }


/*
 * Routine: booti2c_delay_usec
 *
 * Input: usecs
 *      usecs = number of microseconds to sleep
 *
 * Output: None
 *
 * Purpose: 
 *
 * Register usage:
 */

# void
# booti2c_delay_usec(DELAYTIME_REG=usecs)
# {
    .globl  booti2c_delay_usec
booti2c_delay_usec:
    
    /*
     * loop until DELAYTIME_REG becomes 0
     */

nusecloop:    
    mtc0    zero, CP0_COUNT             # initialize count to 0
    ehb                                 # hazard barrier

    /*
     * following loop is for 1 usec
     */

oneusecloop:
    mfc0    LEAFPROCTEMPREG1, CP0_COUNT  # t0 = CP0_COUNT
    slt     LEAFPROCTEMPREG1, LEAFPROCTEMPREG1, ONE_USEC_COUNT # t1 = (t0 < 175) ? 1 : 0
    beq     LEAFPROCTEMPREG1, zero, nextusec                   # if (t1 == 0) go to 1f
    nop
    b       oneusecloop                         # repeat
    nop

nextusec:
    subu    DELAYTIME_REG, DELAYTIME_REG, 1     # t2 = t2 - 1
    bne     DELAYTIME_REG, zero, nusecloop             # if (t2 != 0) repeat
    nop

    jr      ra
    nop
# }

/*
 * Routine: booti2c_master_read
 *
 * Input: offset, addr width
 *     EEP_OFFSET_REG      : offset
 *     EEP_ADDRWIDTH_REG   : addr width : 0 - 8-bit
 *                                        1 - 16-bit
 *
 * Output: status, data
 *     STATUS_REG : status
 *     EEP_DATA_REG : data
 *
 * Purpose: I2C master read
 *          
 *  This routine reads one byte of data from the device selected
 *  through the booti2c_master_initialize with the offset specified
 *  in a0. a1 specifies the addr width. 
 *  The status will be returned in a0. The data read will be 
 *  stored in a1 if success
 */

# void
# booti2c_master_read(void)
# {
    .globl  booti2c_master_read
booti2c_master_read:

    /*
     * save return address
     */

    move    BOOTI2C_RW_RA, ra

    /*
     * for (I2C_ACCESS_CNT_REG = I2C_MAX_ACCESS_ATTEMPTS ; i > 0 ; i--) {
     */

    li      I2C_ACCESS_CNT_REG, I2C_MAX_ACCESS_ATTEMPTS

read_attempt_loop:

    /*
     * for timeout
     */

    mtc0    zero, CP0_COUNT             # initialize count to 0
    ehb                                 # hazard barrier

    /*
     * enable i2c interface
     */

    li      REGDATA_REG, 1
    REG_WRITE(IC_ENABLE(0), REGDATA_REG)


    /*
     * write the address offset with write command
     */

    beqz    EEP_ADDRWIDTH_REG, read_lowaddr /* if data width is 8-bit skip writing the upper 8 bits */
    nop

    srl     REGDATA_REG, EEP_OFFSET_REG, 8       /* get upper byte of offset/addr and write it */
    and     REGDATA_REG, REGDATA_REG, 0xff
    or      REGDATA_REG, REGDATA_REG, IC_DATA_CMD_WRITE

    REG_WRITE(IC_DATA_CMD(0), REGDATA_REG)

read_lowaddr:
    and     REGDATA_REG, EEP_OFFSET_REG, 0xff    /* get lower byte of offset/addr and write it */
    or      REGDATA_REG, REGDATA_REG, IC_DATA_CMD_WRITE
    REG_WRITE(IC_DATA_CMD(0), REGDATA_REG)

    /*
     * write the read command
     */

    li      REGDATA_REG, IC_DATA_CMD_READ
    REG_WRITE(IC_DATA_CMD(0), REGDATA_REG)

    /*
     * wait until we read the data or timeout
     */

    /*
     * read the status
     */
read_status_reread:
    REG_READ(IC_STATUS(0), REGDATA_REG)

    and     REGDATA_REG, REGDATA_REG, IC_STATUS_RFNE
    beqz    REGDATA_REG, read_abortcheck    /* if RFNE is not set re-read the status register */
    nop
    
    /*
     * RFNE is set: we received the byte. Read the byte and return
     */

    REG_READ(IC_DATA_CMD(0), REGDATA_REG)

    and     EEP_DATA_REG, REGDATA_REG, 0xff  /* get the data byte */
    li      STATUS_REG, BOOTI2C_STATUS_SUCCESS   /* STATUS_REG = 0: success */
    b       read_wait_for_master_idle        /* go to wait for master_idle and return */
    nop
    
read_abortcheck:
    /*
     * check for abort status
     */

    REG_READ(IC_RAW_INTR_STAT(0), REGDATA_REG)

    and     REGDATA_REG, REGDATA_REG, IC_INTR_TX_ABRT
    beqz    REGDATA_REG, read_timeoutcheck /* no tx_abort, check timeout and re-read status register */
    nop

    /*
     * tx-abort is set. Clear the tx abort and 
     * return failure
     */

    REG_READ(IC_CLR_TX_ABRT(0), REGDATA_REG)

    li      STATUS_REG, BOOTI2C_STATUS_TX_ABORT      /* a0 = 1 : failure */
    b       read_wait_for_master_idle  /* wait for master to become idle */
    nop

read_timeoutcheck:

    /*
     * check for timeout
     */

    mfc0    REGDATA_REG, CP0_COUNT                           # regdata = CP0_COUNT

    /*  regdata = (regdata < I2C_XFER_TIMEOUT_COUNT) ? 1 : 0  */
    slt     REGDATA_REG, REGDATA_REG, I2C_XFER_TIMEOUT_COUNT 
    bnez    REGDATA_REG, read_status_reread   # if not timedout, check the status again
    nop

    /*
     * timedout
     */
    
    li      STATUS_REG, BOOTI2C_STATUS_TIMEOUT
    b       read_exit
    nop

read_wait_for_master_idle:

    /*
     * wait for master to become idle. 
     * booti2c_wait_for_master_idle can modify the status.
     */

    BAL     (booti2c_wait_for_master_idle)
    nop

    /*
     * check for the overall status
     */

    bne     STATUS_REG, BOOTI2C_STATUS_SUCCESS, 1f
    nop

    /*
     * SUCCESS
     */

    b       read_exit
    nop

1:
    /*
     * TX_ABORT or TIMEOUT
     */
    
    bne     STATUS_REG, BOOTI2C_STATUS_TIMEOUT, 1f
    nop

    /*
     * TIMEOUT
     */

    b       read_exit
    nop
1:
    /* 
     * TX_ABORT: retry until retry exhausted
     */

    sub     I2C_ACCESS_CNT_REG, I2C_ACCESS_CNT_REG, 1
    bnez    I2C_ACCESS_CNT_REG, read_attempt_loop
    nop

read_exit:
    /*
     * restore ra and return
     */

    move    ra, BOOTI2C_RW_RA

    jr      ra
    nop
# }


/*
 * Routine: booti2c_master_write
 *
 * Input: offset, addr width
 *     EEP_OFFSET_REG     : offset
 *     EEP_ADDRWIDTH_REG  : data width : 0 - 8-bit
 *                                       1 - 16-bit
 *     EEP_DATA_REG: data
 *
 * Output: status
 *     STATUS_REG : status
  *
 * Output: None
 *
 * Purpose: I2C master write
 *          
 */

# void
# booti2c_master_write(void)
# {
    .globl  booti2c_master_write
booti2c_master_write:

    /*
     * save return address
     */

    move    BOOTI2C_RW_RA, ra

    /*
     * for (I2C_ACCESS_CNT_REG = I2C_MAX_ACCESS_ATTEMPTS ; i > 0 ; i--) {
     */

    li      I2C_ACCESS_CNT_REG, I2C_MAX_ACCESS_ATTEMPTS

write_attempt_loop:

     /*
     * for timeout
     */

    mtc0    zero, CP0_COUNT             # initialize count to 0
    ehb                                 # hazard barrier

    /*
     * enable i2c interface
     */

    li      REGDATA_REG, 1
    REG_WRITE(IC_ENABLE(0), REGDATA_REG)


    /*
     * write the address offset with write command
     */


    beqz    EEP_ADDRWIDTH_REG, write_lowaddr /* if data width is 8-bit skip writing the upper 8 bits */
    nop

    srl     REGDATA_REG, EEP_OFFSET_REG, 8       /* get upper byte of offset/addr and write it */
    and     REGDATA_REG, REGDATA_REG, 0xff
    or      REGDATA_REG, REGDATA_REG, IC_DATA_CMD_WRITE
    REG_WRITE(IC_DATA_CMD(0), REGDATA_REG)

write_lowaddr:
    and     REGDATA_REG, EEP_OFFSET_REG, 0xff    /* get lower byte of offset/addr and write it */
    or      REGDATA_REG, REGDATA_REG, IC_DATA_CMD_WRITE
    REG_WRITE(IC_DATA_CMD(0), REGDATA_REG)


    /*
     * write the data in EEP_DATA_REG
     */

    and     REGDATA_REG, EEP_DATA_REG, 0xff
    or      REGDATA_REG, REGDATA_REG, IC_DATA_CMD_WRITE
    REG_WRITE(IC_DATA_CMD(0), REGDATA_REG)

write_status_reread:
    /*
     * read the i2c status
     */

    REG_READ(IC_STATUS(0), REGDATA_REG)

    /*
     * read the interrupt status and check for ABRT
     */

    REG_READ(IC_RAW_INTR_STAT(0), TDATA_REG1)

    and     TDATA_REG1, TDATA_REG1, IC_INTR_TX_ABRT
    beqz    TDATA_REG1, write_noabort      /* no abort */
    nop

    /*
     * TX_ABRT set. Clear TX_ABORT and return failure
     */

    REG_READ(IC_CLR_TX_ABRT(0), REGDATA_REG)

    li      STATUS_REG, BOOTI2C_STATUS_TX_ABORT
    b       write_wait_for_master_idle
    nop

write_noabort:
    and     REGDATA_REG, REGDATA_REG, IC_STATUS_TFE
    /* 
     * if transmit fifo not yet empty. 
     * check for timeout 
     */

    beqz    REGDATA_REG, write_timeoutcheck 
    nop

    /*
     * transmit fifo is empty. return good status
     */

    li      STATUS_REG, BOOTI2C_STATUS_SUCCESS
    BAL     (write_wait_for_master_idle)
    nop

write_timeoutcheck:
    /*
     * check for timeout
     */

    mfc0    REGDATA_REG, CP0_COUNT                           # regdata = CP0_COUNT

    /*  regdata = (regdata < I2C_XFER_TIMEOUT_COUNT) ? 1 : 0  */
    slt     REGDATA_REG, REGDATA_REG, I2C_XFER_TIMEOUT_COUNT 
    bnez    REGDATA_REG, write_status_reread   # if not timedout, check the status again
    nop

    /*
     * timedout
     */


    li      STATUS_REG, BOOTI2C_STATUS_TIMEOUT
    b       write_exit
    nop

write_wait_for_master_idle:
    
    /*
     * wait for master to become idle
     */

    BAL     (booti2c_wait_for_master_idle)
    nop

    /*
     * check for the overall status
     */

    bne     STATUS_REG, BOOTI2C_STATUS_SUCCESS, 1f
    nop

    /*
     * SUCCESS
     */

    b       write_exit
    nop

1:
    /*
     * TX_ABORT or TIMEOUT
     */
    
    bne     STATUS_REG, BOOTI2C_STATUS_TIMEOUT, 1f
    nop

    /*
     * TIMEOUT
     */

    b       write_exit
    nop
1:
     sub     I2C_ACCESS_CNT_REG, I2C_ACCESS_CNT_REG, 1
    bnez    I2C_ACCESS_CNT_REG, write_attempt_loop
    nop

    li      STATUS_REG, BOOTI2C_STATUS_MAXTRIES

write_exit:
    /*
     * restore return address and return
     */

    move    ra, BOOTI2C_RW_RA
    jr      ra
    nop
# }


/*
 * Routine: booti2c_wait_for_master_idle
 *
 * Input: None
 *
 * Output: None
 *
 * Purpose:
 *          
 */

# void
# booti2c_wait_for_master_idle(void)
# {
    .globl  booti2c_wait_for_master_idle
booti2c_wait_for_master_idle:

1:
    /*
     * read the status
     */

    REG_READ(IC_STATUS(0), REGDATA_REG)

    /*
     * if active, re-read the status
     */

    and     REGDATA_REG, REGDATA_REG, IC_STATUS_MST_ACTIVITY
    beqz    REGDATA_REG, master_not_active /* master not active any more : done */

    /*
     * do not modify the original status set by the caller
     */

    nop

    /*
     * check for timeout
     */
    
    mfc0    REGDATA_REG, CP0_COUNT          # regdata = CP0_COUNT
    slt     REGDATA_REG, REGDATA_REG, I2C_XFER_TIMEOUT_COUNT 
    bnez    REGDATA_REG, 1b                 # if not timedout, check the status again
    nop

    /*
     * operation timed out
     */

    li      STATUS_REG, BOOTI2C_STATUS_TIMEOUT
master_not_active:

    /*
     * master is now idle or operation timedout. Disable the i2c interface
     */

    li      REGDATA_REG, 0
    REG_WRITE(IC_ENABLE(0), REGDATA_REG)
    
    /*
     * return
     */

    jr      ra
    nop
# }

/*
 * Routine: booti2c_write_post_code
 *
 * Input: postcode, slotno
 *  a0 = postcode
 *  a1 = slotno
 *
 * Output: None
 *
 * Purpose:
 *          
 */

# void
# booti2c_write_post_code(void)
# {
    .globl  booti2c_write_post_code
    .ent    booti2c_write_post_code
booti2c_write_post_code:

    /*
     * save ra
     */

    move BOOTI2C_WRITE_POST_CODE_RA, ra

    /*******************************************************
     *
     *        issue bus-init through gpio clock-out
     *
     *******************************************************/

    /*
     * reset i2c block
     */

    REG_READ(MISC_MREGS_SW_HARD_RST, REGDATA_REG)
    or      REGDATA_REG, REGDATA_REG, (1 << MISC_MREGS_SW_HARD_RST_I2C_SHIFT)
    REG_WRITE(MISC_MREGS_SW_HARD_RST, REGDATA_REG)

    /*
     * 50 usec delay
     */

    li      DELAYTIME_REG, 50              # 50 usec delay
    BAL     (booti2c_delay_usec)
    nop

    /*
     * bring i2c block out of reset
     */

    REG_READ(MISC_MREGS_SW_HARD_RST, REGDATA_REG)
    and     REGDATA_REG, REGDATA_REG, ~(1 << MISC_MREGS_SW_HARD_RST_I2C_SHIFT)
    REG_WRITE(MISC_MREGS_SW_HARD_RST, REGDATA_REG)

    /*
     * configure GPIO 0 as output
     */

    REG_READ(PL_GPIO_DDR(PL_PA_GPIO_I2C0_SCL), REGDATA_REG)
    or      REGDATA_REG, REGDATA_REG, PL_GPIO_BIT(PL_PA_GPIO_I2C0_SCL)
    REG_WRITE(PL_GPIO_DDR(PL_PA_GPIO_I2C0_SCL), REGDATA_REG)

    /*
     * set GPIO 0 to 0 first
     */

    REG_READ(PL_GPIO_DR(PL_PA_GPIO_I2C0_SCL), REGDATA_REG)
    and     REGDATA_REG, REGDATA_REG, ~PL_GPIO_BIT(PL_PA_GPIO_I2C0_SCL)
    REG_WRITE(PL_GPIO_DR(PL_PA_GPIO_I2C0_SCL), REGDATA_REG)

    /*
     * 50 usec delay
     */

    li      DELAYTIME_REG, 50
    BAL     (booti2c_delay_usec)
    nop

    # for (i = 0 ; i < 9 ; i++)
    # {

    li      LOOPCNT_REG1, 9

    /*
     * set gpio 0 to 1
     */
nextclockout:
    REG_READ(PL_GPIO_DR(PL_PA_GPIO_I2C0_SCL), REGDATA_REG)
    or      REGDATA_REG, REGDATA_REG, PL_GPIO_BIT(PL_PA_GPIO_I2C0_SCL)
    REG_WRITE(PL_GPIO_DR(PL_PA_GPIO_I2C0_SCL), REGDATA_REG)

    /*
     * 50 usec delay
     */

    li      DELAYTIME_REG, 50
    BAL     (booti2c_delay_usec)
    nop
     
    /*
     * set gpio 0 to 0
     */

    REG_READ(PL_GPIO_DR(PL_PA_GPIO_I2C0_SCL), REGDATA_REG)
    and     REGDATA_REG, REGDATA_REG, ~PL_GPIO_BIT(PL_PA_GPIO_I2C0_SCL)
    REG_WRITE(PL_GPIO_DR(PL_PA_GPIO_I2C0_SCL), REGDATA_REG)

    /*
     * 50 usec delay
     */

    li      DELAYTIME_REG, 50
    BAL     (booti2c_delay_usec)
    nop
    
    sub     LOOPCNT_REG1, LOOPCNT_REG1, 1   
    beq     LOOPCNT_REG1, zero, clockoutdone
    nop
    b       nextclockout
    nop
    # }
    #

clockoutdone:
    /*
     * configure GPIO 0 as input
     */

    REG_READ(PL_GPIO_DDR(PL_PA_GPIO_I2C0_SCL), REGDATA_REG)
    and     REGDATA_REG, REGDATA_REG, ~PL_GPIO_BIT(PL_PA_GPIO_I2C0_SCL)
    REG_WRITE(PL_GPIO_DDR(PL_PA_GPIO_I2C0_SCL), REGDATA_REG)

    /*
     * 50 usec delay
     */

    li      DELAYTIME_REG, 50
    BAL     (booti2c_delay_usec)
    nop

    /*
     * reset i2c block after gpio clockout
     */

    REG_READ(MISC_MREGS_SW_HARD_RST, REGDATA_REG)
    or      REGDATA_REG, REGDATA_REG, (1 << MISC_MREGS_SW_HARD_RST_I2C_SHIFT)
    REG_WRITE(MISC_MREGS_SW_HARD_RST, REGDATA_REG)

    /*
     * 50 usec delay
     */

    li      DELAYTIME_REG, 50              # 50 usec delay
    BAL     (booti2c_delay_usec)
    nop

    /*
     * bring i2c block out of reset
     */

    REG_READ(MISC_MREGS_SW_HARD_RST, REGDATA_REG)
    and     REGDATA_REG, REGDATA_REG, ~(1 << MISC_MREGS_SW_HARD_RST_I2C_SHIFT)
    REG_WRITE(MISC_MREGS_SW_HARD_RST, REGDATA_REG)


    /*************************************************************
     *
     *  end of bus-init through gpio-clockout
     *
     *************************************************************/

    /*************************************************************
     * 
     * now let's acquire the bus and do protected transfer
     *
     ************************************************************/

    li      LOOPCNT_REG1, POST_WRITE_MAX_ATTEMPTS

arbloop:

    /*
     * create an address for the arbiter in a0
     */
    
    li      TADDR_REG, I2C_ARBITER_TARGET_ADDR(0)
    or      TADDR_REG, TADDR_REG, SLOTNUM_REG

    BAL     (bootarb_take_i2c_bus_control)
    nop

    bne     STATUS_REG, BOOTI2C_STATUS_SUCCESS, booti2c_write_post_code_exit
    nop

    /*
     * we acquired the bus. 
     * let's Try to read from the POST EEPROM
     */

    li      TADDR_REG, I2C_POST_TARGET_ADDR(0)
    or      TADDR_REG, TADDR_REG, SLOTNUM_REG
    BAL     (booti2c_init_master)
    nop

    /*
     * read the seq1.
     */

    li      EEP_OFFSET_REG, I2C_POST_BASE_ADDRESS
    li      EEP_ADDRWIDTH_REG, ADDR_WIDTH_16
    BAL     (booti2c_master_read)
    nop

    bne     STATUS_REG, BOOTI2C_STATUS_TX_ABORT, 2f
    nop

    /*
     * add delay before another eeprom access
     */

    li      DELAYTIME_REG, I2C_EEPROM_ACCESS_DELAY_USEC
    BAL     (booti2c_delay_usec)
    nop

    /*
     * we got TX_ABORT for POST EEPROM
     * Now, try the FRU EEPROM
     */

    li      TADDR_REG, I2C_ALTPOST_TARGET_ADDR(0)
    or      TADDR_REG, TADDR_REG, SLOTNUM_REG
    BAL     (booti2c_init_master)
    nop

    li      EEP_OFFSET_REG, I2C_POST_BASE_ADDRESS
    li      EEP_ADDRWIDTH_REG, ADDR_WIDTH_16
    BAL     (booti2c_master_read)
    nop

2:
    /*
     * check if the transaction was aborted
     */

    bne     STATUS_REG, BOOTI2C_STATUS_SUCCESS, buslostlogic
    nop

    /*
     * transaction was successful from  one of the EEPROMS
     */

    move    SEQNO_REG, EEP_DATA_REG

    /*
     * add delay before another eeprom access
     */

    li      DELAYTIME_REG, I2C_EEPROM_ACCESS_DELAY_USEC
    BAL     (booti2c_delay_usec)
    nop

    /*
     * seq1 was read successfully
     * now, Let's increment the seq1 and write the sequence no
     */

    add     SEQNO_REG, SEQNO_REG, 1

    move    EEP_DATA_REG, SEQNO_REG
    li      EEP_OFFSET_REG, I2C_POST_BASE_ADDRESS
    li      EEP_ADDRWIDTH_REG, ADDR_WIDTH_16

    BAL     (booti2c_master_write)
    nop
    
    bne     STATUS_REG, BOOTI2C_STATUS_SUCCESS, buslostlogic
    nop

    /*
     * wrote seq1 successfully. 
     * Let's write post code one byte at a time with MSB first
     */

    li      POSTCODE_REG_BYTE_COUNT, 4 /* write 4 bytes */

postcodeloop:
    /*
     * add delay before another eeprom access
     */

    li      DELAYTIME_REG, I2C_EEPROM_ACCESS_DELAY_USEC
    BAL     (booti2c_delay_usec)
    nop
 
    move    EEP_DATA_REG, POSTCODE_REG
    sub     TDATA_REG1, POSTCODE_REG_BYTE_COUNT, 1  /* x = count -1 */
    mul     TDATA_REG1, TDATA_REG1, 8               /* x = x * 8    */
    srl     EEP_DATA_REG, EEP_DATA_REG, TDATA_REG1  /* data = postcode >> (8 * (count - 1)) */
    and     EEP_DATA_REG, EEP_DATA_REG, 0xff        /* data &= 0xff : mask lower bits */
    
    add     EEP_OFFSET_REG, EEP_OFFSET_REG , 1      /* offset++ */
    li      EEP_ADDRWIDTH_REG, ADDR_WIDTH_16

    BAL     (booti2c_master_write)
    nop

    bne     STATUS_REG, BOOTI2C_STATUS_SUCCESS, buslostlogic
    nop
    
    sub     POSTCODE_REG_BYTE_COUNT, POSTCODE_REG_BYTE_COUNT, 1
    bnez    POSTCODE_REG_BYTE_COUNT, postcodeloop
    nop

    /*
     * add delay before another eeprom access
     */

    li      DELAYTIME_REG, I2C_EEPROM_ACCESS_DELAY_USEC
    BAL     (booti2c_delay_usec)
    nop
 
    /*
     * done writing post code.
     * now write seq2 (same as seq1)
     */

    add     EEP_OFFSET_REG, EEP_OFFSET_REG , 1      /* offset++ */
    move    EEP_DATA_REG, SEQNO_REG
    li      EEP_ADDRWIDTH_REG, ADDR_WIDTH_16

    BAL     (booti2c_master_write)
    nop

    /*
     * DONE : exit
     */
3:
    nop
buslostlogic:
    move    STATUS_SAVE_REG, STATUS_REG

    /*
     * check for bus lost
     */

    li      TADDR_REG, I2C_ARBITER_TARGET_ADDR(0)
    or      TADDR_REG, TADDR_REG, SLOTNUM_REG
    BAL     (booti2c_init_master)    /* initialize the i2c with arbiter address */
    nop

    li      EEP_OFFSET_REG, ARB_ISTAT
    li      EEP_ADDRWIDTH_REG, ADDR_WIDTH_8
    BAL     (booti2c_master_read)    /* read arbiter int_status register */
    nop

    /*
     * if arbiter read fails, exit
     */

    bne     STATUS_REG, BOOTI2C_STATUS_SUCCESS, booti2c_write_post_code_exit
    nop

    /*
     * arbiter read success, check the BUSLOST bit
     */

    and     EEP_DATA_REG, EEP_DATA_REG, ARB_ISTAT_BUSLOST
    bnez    EEP_DATA_REG, nextstep1     /* bus was lost, retry */
    nop

    /*
     * bus was not lost
     */

    /*
     * create an address for the arbiter in a0
     */

    li      TADDR_REG, I2C_ARBITER_TARGET_ADDR(0)
    or      TADDR_REG, TADDR_REG, SLOTNUM_REG

    /*
     * Release i2c bus and exit
     */

    BAL     (bootarb_release_i2c_bus_control)
    nop

    b       booti2c_write_post_code_exit
    nop

    /*
     * bus was lost in the middle of the transaction
     */

nextstep1:


    subu    LOOPCNT_REG1, LOOPCNT_REG1, 1
    bnez    LOOPCNT_REG1, arbloop
    nop
    
    /*
     * restore ra and return
     */

booti2c_write_post_code_exit:
    move    ra, BOOTI2C_WRITE_POST_CODE_RA

    jr      ra
    nop
    .end    booti2c_write_post_code
# }


/* vi:set filetype=c: */
