
/*
 * Copyright 2006-2007 Nuova Systems, Inc. All Rights Reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: booti2c.h 21 2016-12-02 21:25:59Z buildsa $"

#ifndef _BOOTI2C_H_
#define _BOOTI2C_H_

/*
 * Address widths
 */

#define ADDR_WIDTH_8   0
#define ADDR_WIDTH_16  1

/*
 * TX fifo water marks
 */

#define TX_FIFO_LOW_WATER_MARK   2
#define TX_FIFO_HIGH_WATER_MARK  4

/*
 * I2C status codes
 */

#define BOOTI2C_STATUS_SUCCESS              0
#define BOOTI2C_STATUS_TX_ABORT             1
#define BOOTI2C_STATUS_TIMEOUT              2
#define BOOTI2C_STATUS_MAXTRIES             3

/*
 * attempts-constants
 */

#define ARB_MAX_ACQUIRE_ATTEMPTS            3
#define I2C_MAX_ACCESS_ATTEMPTS             3
#define POST_WRITE_MAX_ATTEMPTS             3

/*
 * macros to read/write chip registers
 */

#define REG_READ(addr, reg)                 \
    li      REGADDR_REG, KSEG1ADDR(addr);   \
    lw      reg, 0(REGADDR_REG)

#define REG_WRITE(addr, reg)                \
    li      REGADDR_REG, KSEG1ADDR(addr);   \
    sw      reg, 0(REGADDR_REG)


/*
 * The CP0_COUNT register ticks at 1/2 the processor clock.
 */
#define ONE_USEC_COUNT  (PALO_ASIC_CPU_CLK / 2000000)

/*
 * 100 msec i2c timeout
 */

#define I2C_XFER_TIMEOUT_MSEC           100
#define I2C_XFER_TIMEOUT_USEC           ((I2C_XFER_TIMEOUT_MSEC) * 1000)
#define I2C_XFER_TIMEOUT_COUNT          (I2C_XFER_TIMEOUT_USEC * ONE_USEC_COUNT)

#define I2C_EEPROM_ACCESS_DELAY_MSEC    10
#define I2C_EEPROM_ACCESS_DELAY_USEC    (I2C_EEPROM_ACCESS_DELAY_MSEC * 1000)

/*
 * cpu register usage for post-code update
 */

#define CLKOUT_LOOP_CNT_REG                 v0      /* $2 */
#define STATUS_REG                          v1      /* $3 */

#define POSTCODE_REG                        a0      /* $4 */
#define SLOTNUM_REG                         a1      /* $5 */
#define DELAYTIME_REG                       a2      /* $6 */
#define I2C_ACCESS_CNT_REG                  a3      /* $7 */

#define REGADDR_REG                         t0      /* $8 */
#define REGDATA_REG                         t1      /* $9 */
#define LEAFPROCTEMPREG1                    t2      /* $10 */
#define LEAFPROCTEMPREG2                    t3      /* $11 */
#define TADDR_REG                           t4      /* $12 */
#define EEP_OFFSET_REG                      t5      /* $13 */
#define EEP_ADDRWIDTH_REG                   t6      /* $14 */
#define EEP_DATA_REG                        t7      /* $15 */

#define BOOT_WRITE_POST_CODE_RA             s0      /* $16 */
#define BOOTI2C_WRITE_POST_CODE_RA          s1      /* $17 */
#define LOOPCNT_REG1                        s2      /* $18 */
#define ARB_LOOP_CNT_REG                    s3      /* $19 */
#define BOOTI2C_RW_RA                       s4      /* $20 */
#define BOOTARB_TAKE_I2C_BUS_CONTROL_RA     s5      /* $21 */
#define POSTCODE_REG_BYTE_COUNT             s5      /* $21 */ /* reuse */
#define EEP_DATA_SAVE_REG                   s6      /* $22 */
#define STATUS_SAVE_REG                     s6      /* $22 */ /* reuse */
#define SEQNO_REG                           s7      /* $23 */

#define TDATA_REG2                          t8      /* $24 */
#define TDATA_REG1                          t9      /* $25 */

#define UNDEF_REG_S8                        s8      /* $26 */


#endif /* _BOOTI2C_H_ */
