
/*
 * Copyright 2008 Nuova Systems, Inc.  All rights reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: cache.S 21 2016-12-02 21:25:59Z buildsa $"

#include <config.h>
#include <version.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>
#include <asm/addrspace.h>
#include <asm/cacheops.h>
#include "palodtls.h"

        .text
        .set    noreorder
        .set    mips32r2

#define cacheloop(len, op) \
        li      t0, KSEG0;                              \
        li      t1, KSEG0 + len - CFG_CACHELINE_SIZE;   \
1:      op,     0(t0);                                  \
        bne     t0, t1, 1b;                             \
        addiu   t0, t0, CFG_CACHELINE_SIZE

#define icacheloop(op) \
        cacheloop(CFG_ICACHE_SIZE, op)

#define dcacheloop(op) \
        cacheloop(CFG_DCACHE_SIZE, op)

/*
 * Initialize the L1 caches.
 * Enable cache parity and perform an invalidate/fill loop.
 * Leaves KSEG0 cacheable.
 */
LEAF(mips_cache_reset)
        /*
         * Clear RAM from which the caches will be filled.
         * Fill via KSEG0 in uncached accelerated mode in order
         * to guarantee burst-writes to DDR in Cruz.
         */
        li      t0, CONF_CM_CACHABLE_ACCELERATED
        mtc0    t0, CP0_CONFIG
        ehb
	li	t0, KSEG0
        li      t1, KSEG0 + CFG_CACHEFILL_SIZE - 32
1:	
        sw	zero, 0(t0)
	sw	zero, 4(t0)
	sw	zero, 8(t0)
	sw	zero, 12(t0)
	sw	zero, 16(t0)
	sw	zero, 20(t0)
	sw	zero, 24(t0)
	sw	zero, 28(t0)
	bne	t0, t1, 1b
	addiu	t0, 32

        /*
         * Enable cache parity.
         */
        li      t0, 0x80000000
        mtc0    t0, CP0_ECC
        ehb

        /*
         * Zero the I-Cache and D-Cache Tags.
         */
        mtc0    zero, CP0_TAGLO
        mtc0    zero, CP0_DTAGLO
        ehb
        icacheloop(cache Index_Store_Tag_I)
        dcacheloop(cache Index_Store_Tag_D)

        /*
         * Turn on KSEG0 caching and fill the caches with good parity.
         */
        li      t0, CONF_CM_CACHABLE_NONCOHERENT
        mtc0    t0, CP0_CONFIG
        ehb
        icacheloop(cache Fill)
        dcacheloop(lw t2)

        /*
         * Clear the I-Cache as code will be loaded soon.
         */
        icacheloop(cache Index_Store_Tag_I)

	jr  ra
        nop
        END(mips_cache_reset)


/*
 * Flush the caches.
 * Used following a copy-in/load of code to RAM.
 * All caches are completely flushed, even though we're passed in
 * (start, len) arguments - for implementation simplicity.
 */
LEAF(flush_cache)
        dcacheloop(cache Index_Writeback_Inv_D)
        icacheloop(cache Index_Invalidate_I)
        sync
        jr.hb   ra
        nop
        END(flush_cache)

/*
 * Initialize the L1 data cache with some locked lines for a 'C' stack.
 *      a0 = base_va
 *      a1 = len
 */
LEAF(mips_dcache_init_locked)
        /*
         * Zero the D-Cache Tags.
         */
        mtc0    zero, CP0_DTAGLO
        ehb
        dcacheloop(cache Index_Store_Tag_D)

        /*
         * Write the locked entries.
         */
        li      t0, 0x1ffff800          // t0 = PTagLo mask
        addu    t1, a0, a1              // t1 = end_va
1:
        and     t2, t0, a0              // t2 = PTagLo
        ori     t2, 0xa0                // PTagLo,V=1,D=0,L=1,P=0
        mtc0    t2, CP0_DTAGLO
        ehb
        cache   Index_Store_Tag_D, 0(a0)
        addiu   a0, 32
        bne     a0, t1, 1b
        nop
        jr.hb   ra
        nop
        END(mips_dcache_init_locked)

