/* * Copyright 2007 Nuova Systems, Inc.  All rights reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: ecc.S 21 2016-12-02 21:25:59Z buildsa $"

#include <config.h>
#include <version.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>
#include <asm/addrspace.h>
#include <palo/include/misc_regs.h>
#include <palo/include/misc_mregs.h>
#include "palodtls.h"

	.text
	.set noreorder
	.set mips32r2

LEAF(ecc_init)
        /*
         * Save return address.
         */
        PUSH    (ra)

        /*
         * Copy the fill routine into KSEG1.
         */
        la      t0, KSEG1ADDR(0)
        la      t1, fill_start
        la      t2, fill_end
1:
        lw      t3, 0(t1)
        sw      t3, 0(t0)
        addiu   t1, t1, 4
        bne     t1, t2, 1b
        addiu   t0, t0, 4   /* delay slot */

        /*
         * Setup a TLB entry to map VA(N) => PA(N) - uncached accelerated
         * across the full 2GB of KUSEG.
         *
         *      VA 00000000-0fffffff => PA 00000000-0fffffff
         *      VA 10000000-1fffffff => PA 90000000-9fffffff (alias to 10000000)
         *      VA 20000000-2fffffff => PA 20000000-2fffffff
         *      VA 30000000-3fffffff => PA 30000000-3fffffff
         *      VA 40000000-4fffffff => PA 40000000-4fffffff
         *      VA 50000000-5fffffff => PA 50000000-5fffffff
         *      VA 60000000-6fffffff => PA 60000000-6fffffff
         *      VA 70000000-7fffffff => PA 70000000-7fffffff
         *
         * Using 0x90000000 as the PA for the 2nd 256MB region is required due
         * to NAXI directing 0x10000000 to the internal registers.
         * Cruz has 2GB RAM, and 0x90000000 will steer to the memory system
         * and will alias to DDR address 0x10000000.  A (benign) "Accesses
         * beyond physical memory" wll be reported from the memory controller.
         *
         * The fill routine only fills the configured RAM-size bytes, so the
         * mappings are good for Palo (128MB)/Sereno (256MB) and Cruz (2GB).
         */
        li      t0, 4                   // Wire 4 TLB entries
        mtc0    t0, CP0_WIRED
        li      t0, 0x1fffe000          // select a 256MB pagesize
        mtc0    t0, CP0_PAGEMASK

        /*
         * Build the 1st TLB entry, with the special alias for VA=10000000
         * VA=00000000, PA=00000000
         * VA=10000000, PA=90000000 (alias for 10000000)
         * Uncached accelerated, Dirty (writable), Valid, Global
         */
        mtc0    zero, CP0_INDEX         // TLB Index 0
        mtc0    zero, CP0_ENTRYHI       // VA=00000000-1fffffff
        li      t0, 0x0000003f          // VA=00000000, PA=00000000
        mtc0    t0, CP0_ENTRYLO0
        li      t0, 0x0240003f          // VA=10000000, PA=90000000 (alias)
        mtc0    t0, CP0_ENTRYLO1
        ehb
        tlbwi
        ehb

        /*
         * Build the remaining 3 entries, covering VA=20000000-7fffffff.
         * VA=PA, Uncached accelerated, Dirty (writable), Valid, Global
         */
        li      t0, 1                   // t0 = Index
        li      t1, 0x20000000          // t1 = EntryHi
        li      t2, 0x0080003f          // t2 = EntryLo0
        li      t3, 0x00c0003f          // t3 = EntryLo1
        li      t4, 0x20000000          // t4 = dEntryHi
        li      t5, 0x00800000          // t5 = dEntryLo
        li      t6, 3                   // t6 = Final Index
1:
        mtc0    t0, CP0_INDEX
        mtc0    t1, CP0_ENTRYHI
        mtc0    t2, CP0_ENTRYLO0
        mtc0    t3, CP0_ENTRYLO1
        ehb
        tlbwi
        ehb
        addu    t1, t4                  // EntryHi.VPN2 += 0x20000000
        addu    t2, t5                  // EntryLo0.PFN += 0x20000000
        addu    t3, t5                  // EntryLo1.PFN += 0x20000000
        bne     t0, t6, 1b
        addiu   t0, 1                   // Index++

        /*
         * Call the fill routine, cached.
         */
        li      t0, KSEG0ADDR(0)
        jalr    t0
        nop

        /*
         * Invalidate the TLB entries and reset the TLB.
         */
        mtc0    zero, CP0_ENTRYLO0
        mtc0    zero, CP0_ENTRYLO1
        move    t0, zero
        li      t1, 3
1:
        mtc0    t0, CP0_INDEX
        ehb
        tlbwi
        ehb
        bne     t0, t1, 1b
        addiu   t0, 1
        mtc0    zero, CP0_WIRED
        mtc0    zero, CP0_PAGEMASK

#ifdef CONFIG_BASE_SERENO
        /*
         * Enable BECC detection/correction
         */
        li      t0, 0x00000003
        sw      t0, KSEG1ADDR(MISC_MREGS_BE_ECC_CFG_CONTROL)
        lw      t0, KSEG1ADDR(MISC_MREGS_BE_ECC_CFG_CONTROL)
#endif /* CONFIG_BASE_SERENO */

        /*
         * And we're done.
         */
        POP     (ra)
        jr      ra
        nop
        END(ecc_init)

/*
 * Write zeroes into RAM from the end of the already-initialized cache
 * area to the end of RAM via memory mapped as uncached-accelerated.
 */
fill_start:
        li      t0, CFG_CACHEFILL_SIZE
        lw      t2, KSEG1ADDR(MISC_MREGS_EMULATION)
        li      t1, MISC_EMULATION_FPGA
        beq     t2, t1, fpga_mem
        nop
        li      t1, CFG_SDRAM_LEN_ASIC - 32
        b       1f
        nop
fpga_mem:
        li      t1, CFG_SDRAM_LEN_FPGA - 32
1:
        sw      zero, 0(t0)
        sw      zero, 4(t0)
        sw      zero, 8(t0)
        sw      zero, 12(t0)
        sw      zero, 16(t0)
        sw      zero, 20(t0)
        sw      zero, 24(t0)
        sw      zero, 28(t0)
        bne     t0, t1, 1b
        addiu   t0, t0, 32 /* delay slot */
        jr      ra
        nop
fill_end:

