
/*
 * Copyright 2010 Cisco Systems, Inc.  All rights reserved.
 *
 * (C) Copyright 2000-2002
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2001 Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Andreas Heppel <aheppel@sysgo.de>

 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <environment.h>
#include <asm/addrspace.h>
#include <palo/fw/mcpu/include/paloflash.h>
#include "palodtls.h"

DECLARE_GLOBAL_DATA_PTR;

extern uchar default_environment[];

#define env_ptr     gd->env_ptr

char *env_name_spec = "Flash";

uchar
env_get_char_spec(int index)
{
    return *(uchar *)(gd->env_addr + index);
}

void
env_relocate_spec(void)
{
    memcpy(env_ptr, (void *)palo_env_addr(), sizeof(*env_ptr));
}

int
saveenv(void)
{
    int len, rc;
    ulong end_addr;
    ulong flash_sect_addr, flash_sect_size;
    uchar *env_buffer = (uchar *)env_ptr;
    int rcode = 0;

    flash_sect_addr = palo_env_addr();
    flash_sect_size = palo_env_sect_size();
    len = sizeof(*env_ptr);

    end_addr = flash_sect_addr + flash_sect_size - 1;

    if (flash_sect_protect(0, flash_sect_addr, end_addr)) {
        return 1;
    }

    puts("Erasing Flash...");
    if (flash_sect_erase(flash_sect_addr, end_addr)) {
        return 1;
    }

    puts("Writing to Flash... ");
    rc = flash_write((char *)env_buffer, flash_sect_addr, len);
    if (rc != 0) {
        flash_perror(rc);
        rcode = 1;
    } else {
        puts("done\n");
    }

    /* try to re-protect */
    (void)flash_sect_protect(1, flash_sect_addr, end_addr);
    return rcode;
}

/*
 * Probe for the environment.
 * Consider it located if the first two words aren't erased and the CRC32
 * of the data passes.
 */
static int
env_probe(ulong addr)
{
    ulong *mem = (ulong *)addr;

    if (mem[0] != 0xffffffff && mem[1] != 0xffffffff) {
        env_ptr = (env_t *)mem;
        if (crc32(0, env_ptr->data, ENV_SIZE) == env_ptr->crc) {
            gd->env_addr = (ulong)&env_ptr->data;
            gd->env_valid = 1;
            return 0;
        }
    }
    return -1;
}

/*
 * Initialize the environment.
 * Running from flash, the chip type cannot be determined, so we must probe
 * all likely locations.
 */
int
env_init(void)
{
    int ret;

    ret = env_probe(KSEG1ADDR(PALOFLASH_PHYS + PALOFLASH_M29_ENV_OFFSET));
    if (ret < 0) {
        ret = env_probe(KSEG1ADDR(PALOFLASH_PHYS + PALOFLASH_P30_ENV_OFFSET));
    }
    if (ret < 0) {
        gd->env_addr = (ulong)&default_environment[0];
        gd->env_valid = 0;
    }
    return 0;
}

/*
 * Return the true location of the environment (used when saving..
 */
unsigned long
palo_env_addr(void)
{
    return palo_part_addr(PT_ENV);
}

unsigned long
palo_env_sect_size(void)
{
    return palo_part_size(PT_ENV);
}
