
/*
 * Copyright (c) 2008 Nuova Systems, Inc.  All rights reserved.
 */
#ident "$Id: gpio.c 6 2016-12-02 03:26:55Z buildsa $"

#ifdef __UBOOT__
#include <common.h>
#include <command.h>
#include <asm/addrspace.h>
#include <asm/io.h>
#include <palo/include/synopsis/dw_gpio.h>
#include "libgpio.h"
#include "palodtls.h"
#else
#ifdef __KERNEL__
#include <linux/kernel.h>
#include <palo/include/synopsis/dw_gpio.h>
#include <asm/io.h>
#include "libgpio.h"
#include "regio.h"
#else
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include "regio.h"
#include "synopsis/dw_gpio.h"
#include "liblock.h"
#include "libgpio.h"

/*
 * Define GPIO_LOCKING if multiple processes will be accessing the GPIOs
 * simultaneously.
 *
 */
#define GPIO_LOCKING
#endif
#endif

#ifdef GPIO_LOCKING
static lockobj_t *gpio_lp = NULL;
static int gpio_initdone = 0;
#endif

static u_int32_t
gp_read_port(int pin, u_int32_t base, u_int32_t stride)
{
    u_int32_t addr = base + stride * (pin / 4);

    return (readreg(addr));
}

static void
gp_write_port(int pin, u_int32_t base, u_int32_t stride, u_int32_t val)
{
    u_int32_t addr = base + stride * (pin / 4);

    writereg(addr, val);
}


static int
gp_read_bit(int pin, u_int32_t base, u_int32_t stride)
{
    u_int32_t val;
    int bit = pin % 4;

    val = gp_read_port(pin, base, stride);

    return ((val >> bit) & 0x1);
}

static void
gp_write_bit(int pin, u_int32_t base, u_int32_t stride, int val)
{
    u_int32_t rv;
    int bit = pin % 4;

    rv = gp_read_port(pin, base, stride);
    rv = (rv & ~(1 << bit)) | (val << bit);
    gp_write_port(pin, base, stride, rv);
}

int
gpio_input(int pin)
{
    int val;

    val = (gp_read_bit(pin, GPIO_EXT_PORTA, GPIO_EXT_PORTB - GPIO_EXT_PORTA));

    return (val);
}

void
gpio_output(int pin, int val)
{
    gpio_lock();
    gp_write_bit(pin, GPIO_SWPORTA_DR,
            GPIO_SWPORTB_DR - GPIO_SWPORTA_DR, val);
    gpio_unlock();
}

int
gpio_getoutput(int pin)
{
    int val;

    gpio_lock();
    val = gp_read_bit(pin, GPIO_SWPORTA_DR, GPIO_SWPORTB_DR - GPIO_SWPORTA_DR);
    gpio_unlock();

    return (val);
}

void
gpio_setdirection(int pin, int dir)
{
    gpio_lock();
    gp_write_bit(pin, GPIO_SWPORTA_DDR,
            GPIO_SWPORTB_DDR - GPIO_SWPORTA_DDR, dir);
    gpio_unlock();
}

int
gpio_getdirection(int pin)
{
    int val;

    gpio_lock();
    val = gp_read_bit(pin, GPIO_SWPORTA_DDR,
            GPIO_SWPORTB_DDR - GPIO_SWPORTA_DDR);
    gpio_unlock();

    return (val);
}

#ifdef GPIO_LOCKING
static void
gpio_init(void)
{
    gpio_lp = lock_get("gpio");
    gpio_initdone = 1;
}

int
gpio_lock(void)
{
    if (!gpio_initdone)
        gpio_init();
    return (lock_acquire(gpio_lp));
}

void
gpio_unlock(void)
{
    if (!gpio_initdone)
        gpio_init();
    else
        lock_release(gpio_lp);
}
#else
int
gpio_lock(void)
{
    return (0);
}

void
gpio_unlock(void)
{
}
#endif
