
/*
 * Copyright 2008 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id$"

#include <common.h>
#include <command.h>
#include <common/include/mezzi2c.h>
#include <palo/fw/mcpu/include/board_conf.h>
#include "palodtls.h"

#define TRANSFER_ATTEMPTS       3

/* ****************************************************************************
 * WRITE POST STATUS.
 */

struct postinfo_s {
    u_int32_t   postval;
    u_int32_t   cardid;
};

static int
read_post_bytes(mezzpost_t *p, u_int8_t target)
{
    i2c_master_initialize(0, target, TRANSFER_ATTEMPTS);
    return (i2c_master_access(0, I2C_OP_READ, I2C_POST_BASE_ADDRESS,
                    I2C_ADDR_WIDTH_16, (u_int8_t *)p, sizeof (*p), 0));
}

static int
do_read(u_int32_t cardid, mezzpost_t *p)
{
    int ret;

    ret = read_post_bytes(p, I2C_POST_TARGET_ADDR(cardid));

    return (ret);
}

static int
post_write(void *opaque)
{
    struct postinfo_s *xp = opaque;
    mezzpost_t post;
    int ret;

    ret = do_read(xp->cardid, &post);
    if (ret == 0) {
        ++post.seq1;
        post.seq2 = post.seq1;
        post.post = htonl(xp->postval);
        ret = i2c_master_access(0, I2C_OP_WRITE, I2C_POST_BASE_ADDRESS,
                    I2C_ADDR_WIDTH_16, (u_int8_t *)&post, sizeof (post), 0);
    }
    return (ret);
}

int
write_post(u_int32_t postval)
{
    struct postinfo_s xinfo;
    int ret;

    xinfo.cardid = board_conf_card_address();
    xinfo.postval = postval;

    ret = arb_protected_transfer(0, I2C_ARBITER_TARGET_ADDR(xinfo.cardid),
            TRANSFER_ATTEMPTS, post_write, &xinfo);

    return (ret);
}

static int
post_read(void *opaque)
{
    struct postinfo_s *xp = opaque;
    mezzpost_t post;
    int ret;

    ret = do_read(xp->cardid, &post);
    if (ret == 0) {
        printf("%u 0x%x %u\n", post.seq1, ntohl(post.post), post.seq2);
        if (post.seq1 == post.seq2)
            xp->postval = ntohl(post.post);
    }

    return (ret);
}

int
read_post(u_int32_t *valp)
{
    struct postinfo_s xinfo;
    int ret;

    xinfo.cardid = board_conf_card_address();
    xinfo.postval = 0;

    ret = arb_protected_transfer(0, I2C_ARBITER_TARGET_ADDR(xinfo.cardid),
            TRANSFER_ATTEMPTS, post_read, &xinfo);
    if (ret == 0)
        *valp = xinfo.postval;

    return (ret);
}


/* ****************************************************************************
 * READ/WRITE BOOT COMMAND
 */
struct bcmdinfo_s {
    u_int32_t   cardid;
    mezzbootctrl_t bcmd;
};

static int
read_bcmd_bytes(mezzbootctrl_t *p, u_int8_t target)
{
    i2c_master_initialize(0, target, TRANSFER_ATTEMPTS);
    return (i2c_master_access(0, I2C_OP_READ, I2C_BOOTCTRL_BASE_ADDRESS,
                    I2C_ADDR_WIDTH_16, (u_int8_t *)p, sizeof (*p), 0));
}

static int
bcmd_read(void *opaque)
{
    struct bcmdinfo_s *xp = opaque;
    mezzbootctrl_t *p = &xp->bcmd;
    int ret;

    ret = read_bcmd_bytes(p, I2C_POST_TARGET_ADDR(xp->cardid));

    return (ret);
}

int
read_bcmd(u_int32_t *bcmdp)
{
    struct bcmdinfo_s xinfo;
    int ret;

    xinfo.cardid = board_conf_card_address();

    ret = arb_protected_transfer(0, I2C_ARBITER_TARGET_ADDR(xinfo.cardid),
            TRANSFER_ATTEMPTS, bcmd_read, &xinfo);
    if (ret == 0) {
        if (ntohl(xinfo.bcmd.magic) == BOOTCMD_MAGIC)
            *bcmdp = ntohl(xinfo.bcmd.cmd);
        else
            *bcmdp = 0;
    }

    return (ret);
}

static int
write_bcmd_bytes(mezzbootctrl_t *p, u_int8_t target)
{
    i2c_master_initialize(0, target, TRANSFER_ATTEMPTS);
    return (i2c_master_access(0, I2C_OP_WRITE, I2C_BOOTCTRL_BASE_ADDRESS,
                    I2C_ADDR_WIDTH_16, (u_int8_t *)p, sizeof (*p), 0));
}

static int
bcmd_write(void *opaque)
{
    struct bcmdinfo_s *xp = opaque;
    mezzbootctrl_t *p = &xp->bcmd;
    int ret;

    ret = write_bcmd_bytes(p, I2C_POST_TARGET_ADDR(xp->cardid));

    return (ret);
}

int
write_bcmd(u_int32_t bcmdval)
{
    struct bcmdinfo_s xinfo;
    int ret;

    xinfo.cardid = board_conf_card_address();
    xinfo.bcmd.magic = htonl(BOOTCMD_MAGIC);
    xinfo.bcmd.cmd = htonl(bcmdval);

    ret = arb_protected_transfer(0, I2C_ARBITER_TARGET_ADDR(xinfo.cardid),
            TRANSFER_ATTEMPTS, bcmd_write, &xinfo);

    return (ret);
}
