
/*
 * Copyright 2012 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: lowlevel_init_macros.h 6 2016-12-02 03:26:55Z buildsa $"

#ifndef _LOWLEVEL_INIT_MACROS_H_
#define _LOWLEVEL_INIT_MACROS_H_

        /*
         * word macro.  Works like the fs2 command.
         * word addr, val   - write val to address addr (clobber t0, t1)
         * word addr        - read addr into t2 (clobber t0, t2)
         *
         * Tracks of the last addr (for reads/write) and val (for writes)
         * to optimize the loads/stores (remove redundant reloads of registers)
         *
         * Use it like this:
         *      word_init
         *      word
         *      word
         *      word
         *
         * Do not touch t0, and t1 through a 'word' list.  Other registers
         * are ok to clobber.
         */
        .macro  word_init
        oaddr   = 0             // t0
        oval    = 0             // t1
        move    t0, zero
        move    t1, zero
        .endm

        .macro  word a, v
            // load address into t0
            .ifne   oaddr-(\a >> 16)
                lui     t0, \a >> 16
                oaddr = \a >> 16
            .endif
            .ifnc   "\v", ""
                // write
                .ifeq   \v
                    // write zero
                    sw      zero, (\a & 0xffff)(t0)
                .else
                    // write non-zero
                    // load data into t1
                    .ifne   oval-\v
                        li      t1, \v
                        oval = \v
                    .endif
                    sw      t1, (\a & 0xffff)(t0)
                .endif
            .else
                // read into scratch (t2)
                lw      t2, (\a & 0xffff)(t0)
            .endif
        .endm

        /*
         * Poll an address until a bit is set ('and' = non-zero)
         * Clobbers t2 and t3.
         */
        .macro poll_until_set a, v
        li      t3, \v
1:
        word    \a
        and     t2, t3
        beqz    t2, 1b
        nop
        .endm

        /*
         * Poll an address until a bit is clear ('and' = zero)
         * Clobbers t2 and t3.
         */
        .macro poll_until_clr a, v
        li      t3, \v
1:
        word    \a
        and     t2, t3
        bnez    t2, 1b
        nop
        .endm

#endif
