
/*
 * Copyright 2008 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: palodtls.h 21 2016-12-02 21:25:59Z buildsa $"

#ifndef _PALODTLS_H_
#define _PALODTLS_H_

/*
 * Assembly language compatible macros.
 */
#define PL_GPIO_REG_ADDR(pin, base, stride) \
        ((base) + (stride) * ((pin) / 4))

#define PL_GPIO_DDR(pin) \
        PL_GPIO_REG_ADDR(pin, GPIO_SWPORTA_DDR, \
                GPIO_SWPORTB_DDR - GPIO_SWPORTA_DDR)

#define PL_GPIO_DR(pin) \
        PL_GPIO_REG_ADDR(pin, GPIO_SWPORTA_DR, \
                GPIO_SWPORTB_DR - GPIO_SWPORTA_DR)

#define PL_GPIO_EXT(pin) \
        PL_GPIO_REG_ADDR(pin, GPIO_EXT_PORTA, \
                GPIO_EXT_PORTB - GPIO_EXT_PORTA)

#define PL_GPIO_BIT(pin)        (1 << ((pin) % 4))

#ifdef __ASSEMBLY__

#define BAL(x)              \
        la      t9, x;      \
        jalr    t9

#define LEAF(name)          \
        .globl  name;       \
        .text;              \
        .ent    name;       \
name:

#define END(name)           \
        .size name,.-name;  \
        .end    name

/*
 * 4-entry "stack" for lowlevel functions that need to preserve registers
 * across subroutine calls.
 */
#define PUSHINIT            \
        move    s5, zero

#define PUSH(r)             \
        tne     s5, zero;   \
        move    s5, s6;     \
        move    s6, s7;     \
        move    s7, s8;     \
        move    s8, r

#define POP(r)              \
        move    r, s8;      \
        move    s8, s7;     \
        move    s7, s6;     \
        move    s6, s5;     \
        move    s5, zero

#else

/*
 * blockcopy.S
 */
void blockcopy(void *cpdest, void *data, u_int32_t cplen);

/*
 * part.c
 */
enum {
	PT_OBFL, PT_CONFIG, PT_FIRMWARE, PT_BOOT, PT_FWMAP,
        PT_ENV, PT_KDUMP, PT_DIAG, PT_NUM
};

unsigned long palo_part_size(int pt);
unsigned long palo_part_addr(int pt);

/*
 * i2cops.c
 */
int read_post(u_int32_t *postcode);
int write_post(u_int32_t postcode);
int read_bcmd(u_int32_t *bcmdp);
int write_bcmd(u_int32_t bcmd);

/*
 * arb.c
 */
int arb_protected_transfer(int port, u_int8_t arb_target_addr, int nattempts,
        int (*xfer_func)(void *opaque), void *opaque);

/*
 * i2c.c
 */
#define I2C_NPORTS      2

enum {
    I2C_ERR_NONE        =  0,
    I2C_ERR_TIMEOUT     = -1,
    I2C_ERR_MAXTRIES    = -2,
    I2C_ERR_TX_ABORT    = -3,
    I2C_ERR_INVALID     = -4,
};

enum {
    I2C_ADDR_WIDTH_0,
    I2C_ADDR_WIDTH_8,
    I2C_ADDR_WIDTH_16,
    I2C_ADDR_WIDTH_32,
};

enum {
    I2C_OP_READ,
    I2C_OP_WRITE,
};

void i2c_bus_reset(int port);
void i2c_master_initialize(int port, u_int8_t target, int nattempts);
int i2c_master_access(int port, int op, u_int32_t addr,
        int addr_width, u_int8_t *buffer, int len, u_int32_t flags);

/*
 * lowlevel_boardid.S
 */
#if defined CONFIG_BASE_PALO
int lowlevel_is_montereypark(void);
#elif defined CONFIG_BASE_CRUZ
int lowlevel_is_montclair(void);
int lowlevel_is_calistoga(void);
int lowlevel_is_mountainview(void);
#endif

/*
 * findvar.S
 */
const char *findvar(const char *s);

/*
 * palo.c
 */
long int initdram(int board_type);

#define readreg(a)          readl(KSEG1ADDR(a))
#define writereg(a, v)      writel((v), KSEG1ADDR(a))

static inline u_int64_t sa_timer_get(void)
{
    u_int64_t t = get_ticks();

    t = t * 1000000000ULL;
    t = t / (u_int64_t)get_tbclk();

    return t;
}

static inline void usleep(u_int32_t us)
{
    reset_timer();
    udelay(us);
}

/*
 * XXX: copied from /usr/include/linux/i2c.h.
 */
struct i2c_msg {
        u_int16_t addr;     /* slave address                        */
        u_int16_t flags;
        u_int16_t len;      /* msg length                           */
        u_int8_t  *buf;     /* pointer to msg data                  */
};
#define I2C_M_RD        0x01

struct i2c_rdwr_ioctl_data {
    struct i2c_msg *msgs;
    int nmsgs;
};

int palo_i2c_xfer(struct i2c_msg *msg, int num);

extern int errno;
enum {
    EAGAIN = 1,
    ENXIO,
    EIO,
    EINVAL,
    ENOMEM
};

#endif

#endif
