
/*
 * Copyright 2010 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: part.c 6 2016-12-02 03:26:55Z buildsa $"

#include <common.h>
#include <command.h>
#include <flash.h>
#include <asm/addrspace.h>
#include <asm/io.h>
#include <palo/fw/mcpu/include/paloflash.h>
#include "palodtls.h"

DECLARE_GLOBAL_DATA_PTR;

extern flash_info_t flash_info[];

struct partition {
    u_int32_t   offset;
    u_int32_t   size;
};

static struct partition palo_p30_partitions[] = {
    [PT_OBFL] = {
        .offset     = PALOFLASH_P30_OBFL_OFFSET,
        .size       = PALOFLASH_P30_OBFL_SIZE,
    },
    [PT_CONFIG] = {
        .offset     = PALOFLASH_P30_CONFIG_OFFSET,
        .size       = PALOFLASH_P30_CONFIG_SIZE,
    },
    [PT_FIRMWARE] = {
        .offset     = PALOFLASH_P30_FIRMWARE_OFFSET,
        .size       = PALOFLASH_P30_FIRMWARE_SIZE,
    },
    [PT_BOOT] = {
        .offset     = PALOFLASH_P30_BOOT_OFFSET,
        .size       = PALOFLASH_P30_BOOT_SIZE,
    },
    [PT_FWMAP] = {
        .offset     = PALOFLASH_P30_FWMAP_OFFSET,
        .size       = PALOFLASH_P30_FWMAP_SIZE,
    },
    [PT_ENV] = {
        .offset     = PALOFLASH_P30_ENV_OFFSET,
        .size       = PALOFLASH_P30_ENV_SIZE,
    },
    [PT_KDUMP] = {
        .offset     = PALOFLASH_P30_KDUMP_OFFSET,
        .size       = PALOFLASH_P30_KDUMP_SIZE,
    },
    [PT_DIAG] = {
        .offset     = PALOFLASH_P30_DIAG_OFFSET,
        .size       = PALOFLASH_P30_DIAG_SIZE,
    },
};

static struct partition palo_m29_partitions[] = {
    [PT_OBFL] = {
        .offset     = PALOFLASH_M29_OBFL_OFFSET,
        .size       = PALOFLASH_M29_OBFL_SIZE,
    },
    [PT_CONFIG] = {
        .offset     = PALOFLASH_M29_CONFIG_OFFSET,
        .size       = PALOFLASH_M29_CONFIG_SIZE,
    },
    [PT_FIRMWARE] = {
        .offset     = PALOFLASH_M29_FIRMWARE_OFFSET,
        .size       = PALOFLASH_M29_FIRMWARE_SIZE,
    },
    [PT_BOOT] = {
        .offset     = PALOFLASH_M29_BOOT_OFFSET,
        .size       = PALOFLASH_M29_BOOT_SIZE,
    },
    [PT_FWMAP] = {
        .offset     = PALOFLASH_M29_FWMAP_OFFSET,
        .size       = PALOFLASH_M29_FWMAP_SIZE,
    },
    [PT_ENV] = {
        .offset     = PALOFLASH_M29_ENV_OFFSET,
        .size       = PALOFLASH_M29_ENV_SIZE,
    },
    [PT_KDUMP] = {
        .offset     = PALOFLASH_M29_KDUMP_OFFSET,
        .size       = PALOFLASH_M29_KDUMP_SIZE,
    },
    [PT_DIAG] = {
        .offset     = PALOFLASH_M29_DIAG_OFFSET,
        .size       = PALOFLASH_M29_DIAG_SIZE,
    },
};

static struct partition parts[PT_NUM];
static int parts_init_done = 0;

#define CFI_CMDSET_INTEL_EXTENDED       1
#define CFI_CMDSET_AMD_STANDARD         2
#define CFI_CMDSET_INTEL_STANDARD       3
#define CFI_CMDSET_AMD_EXTENDED         4

static void
parts_init(void)
{
    flash_info_t *info = &flash_info[0];

    switch (info->vendor) {
    case CFI_CMDSET_INTEL_STANDARD:
    case CFI_CMDSET_INTEL_EXTENDED:
        memcpy(parts, palo_p30_partitions, sizeof (parts));
        break;

    case CFI_CMDSET_AMD_STANDARD:
    case CFI_CMDSET_AMD_EXTENDED:
    default:
        memcpy(parts, palo_m29_partitions, sizeof (parts));
        break;
    }

    parts_init_done = 1;
}

unsigned long
palo_part_size(int pt)
{
    if (!parts_init_done)
        parts_init();
    return parts[pt].size;
}

unsigned long
palo_part_addr(int pt)
{
    if (!parts_init_done)
        parts_init();
    return CFG_FLASH_BASE + parts[pt].offset;
}
