
/*
 * Copyright (c) 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: pcie_lowlevel.S 43 2016-12-03 22:59:19Z buildsa $"

#include <config.h>
#include <version.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>
#include <asm/addrspace.h>
#include <palo/include/misc_regs.h>
#include <palo/include/pcie_mregs.h>
#include <palo/include/palo_gpio.h>
#include <palo/include/synopsis/dw_gpio.h>
#include <palo/fw/mcpu/include/board_conf.h>
#include "palodtls.h"

        .set mips32r2
        .set noreorder
        .text

/*
 * Initialize PCIE
 */
LEAF(pcie_lowlevel_init)
        PUSH    (ra)                        // save ra
#if defined CONFIG_BASE_PALO || defined CONFIG_BASE_SERENO
        /*
         * Enable the PCIe lock-out to the host.
         */
        li      t0, 1
        sw      t0, KSEG1ADDR(PCIEC_MREGS_WRO_BARRIER_OPEN)
        lw      t0, KSEG1ADDR(PCIEC_MREGS_STA_BARRIER)  // read STA to flush

#if defined CONFIG_BASE_PALO
        /*
         * Tweak the timer modifier for the PCIe MAC Replay Timer from the
         * default value of 6 to the spec-compliant value of 1.
         */
        li      t0, 0x200043aa
        sw      t0, KSEG1ADDR(PCIEP_MREGS_DHS_PCIE_SW_CFG + 0x718)

        /*
         * If enabled, turn off pcie.
         */
        la      a0, pcieoff
        BAL     (findvar)
        nop
        beqz    v0, 1f
        nop
        lb      t0, 0(v0)
        xori    t0, t0, '1'
        beqz    t0, pcie_off
        nop
1:
        /*
         * If this is a Monterey Park card and we're running in low power
         * mode, disable the PCI-E serdes and skip PCIE link training.
         */
        BAL     (lowlevel_is_montereypark)
        nop
        beqz    v0, pcie_on             /* branch if not montereypark */
        nop

        /*
         * Card is Monterey Park.  Check main power bit.
         */
        lw      t0, KSEG1ADDR(PL_GPIO_EXT(PL_MP_GPIO_MAINPOWER))
        and     t0, t0, PL_GPIO_BIT(PL_MP_GPIO_MAINPOWER)
        beqz    t0, pcie_off            /* main power is off */
        nop

#endif /* CONFIG_BASE_PALO */

#if defined CONFIG_BASE_SERENO

        /*
         * Swap PCIe lanes on ASICs (i.e. Greenwood and Vasona as of 10/2010).
         */
        lw      t0, KSEG1ADDR(MISC_MREGS_EMULATION)
        li      t1, MISC_EMULATION_SERENO
        bne     t0, t1, pcie_on
        nop

        /*
         * Check main power bit for PCIe card.
         */
        BAL     (lowlevel_boardid)
        nop

        li      t0, BOARD_CONF_BRD_LEXINGTON
        beq     v0, t0, check_main_power
        nop
        li      t0, BOARD_CONF_BRD_KIRKWOODLAKE
        beq     v0, t0, check_main_power
        nop
        li      t0, BOARD_CONF_BRD_ICEHOUSE
        beq     v0, t0, check_main_power
        nop
        li      t0, BOARD_CONF_BRD_OAKLAND
        beq     v0, t0, pcie_on
        nop
        li      t0, BOARD_CONF_BRD_ICEHOUSE2
        beq     v0, t0, check_main_power
        nop
        li      t0, BOARD_CONF_BRD_WALNUTCREEK
        beq     v0, t0, pcie_on
        nop
        li      t0, BOARD_CONF_BRD_SUSANVILLE
        beq     v0, t0, check_main_power
        nop
        li      t0, BOARD_CONF_BRD_TORRANCE
        beq     v0, t0, check_main_power
        nop
        b       pcie_on_sereno
        nop

check_main_power:
        /*
         * Lexington and Kirkwood has the main power bit mapped to the
         * same gpio pin.
         */
        lw      t0, KSEG1ADDR(PL_GPIO_EXT(PL_LX_GPIO_MAINPOWER))
        and     t0, t0, PL_GPIO_BIT(PL_LX_GPIO_MAINPOWER)
        beqz    t0, pcie_off                   /* Main power is off */
        nop

pcie_on_sereno:
        li      t0, KSEG1ADDR(PCIEP_MREGS_CFG_MAC)
        lw      t1, 0(t0)
        or      t1, t1, 1 << PCIEP_MREGS_CFG_MAC_RX_LANE_FLIP_EN_SHIFT
        or      t1, t1, 1 << PCIEP_MREGS_CFG_MAC_TX_LANE_FLIP_EN_SHIFT
        sw      t1, 0(t0)
        lw      t1, 0(t0)   // flush

#endif /* CONFIG_BASE_SERENO */

        /*
         * Don't add code here -- not all execution paths get here.
         * pcie_on/off is branched to from multiple places above.
         */
pcie_on:
        /*
         * LTSSM Enable.
         */
        li      t0, KSEG1ADDR(PCIEP_MREGS_CFG_MAC)
        lw      t1, 0(t0)
        ori     t1, t1, 1
        sw      t1, 0(t0)
        b       done
        nop

pcie_off:
        /* no op */
#endif
done:
        POP     (ra)
        jr      ra
        nop
        END(pcie_lowlevel_init)
pcieoff:
        .ascii  "pcieoff=\000"
