/*
 * Copyright 2006-2007 Nuova Systems, Inc. All Rights Reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: post.S 6 2016-12-02 03:26:55Z buildsa $"

#include <config.h>
#include <version.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>
#include <asm/addrspace.h>
#include <asm/cacheops.h>
#include <palo/fw/mcpu/include/palopostcodes.h>

#define POST_SHORT      // Short DDR POST via CHUNK_SIZE + skip most MBIST except L2 for Palo 

#define MEM_START	(KSEG1 + 4)
#if defined CONFIG_BASE_PALO
#define MEM_END		(KSEG1 + 0x8000000)            // 128MB for Palo DDR 
#define MEM_SIZE	0x8000000
#define CHUNK_SIZE	0x400    /* 1K chunk - need for fast 128MB memory sweep for Palo ASIC */
#define SPRAM_BASE	(KSEG1 + 0x10000000)
#define SPRAM_START	(KSEG1 + 0x10000000 + 4) 
#define SPRAM_END	(KSEG1 + 0x10000000 + 0x4000)  // 16K SPRAM 
#elif defined CONFIG_BASE_SERENO
#define MEM_END         (KSEG1 + CFG_SDRAM_LEN_ASIC)   // 224MB for Sereno ECC-enabled DDR, no SPRAM for Sereno  
#define MEM_SIZE        CFG_SDRAM_LEN_ASIC
#define CHUNK_SIZE      0x800    /* 2K chunk - need for fast 256MB non-ECC memory sweep for Sereno ASIC */
#elif defined CONFIG_BASE_CRUZ
#define MEM_END         (KSEG1 + 0x100000)   // 1MB for Cruzita (BUG BUG)
#define MEM_SIZE        0x100000
#define CHUNK_SIZE      0x800    /* 2K chunk - need for fast 256MB non-ECC memory sweep for Sereno ASIC */
#endif /* CONFIG_BASE */

//U-boot mipsregs.h don't have these defines 
#define CP0_ERRCTL       $26
#define CP0_DATALO       $28,1
#define CP0_DATAHI       $29,1

#if defined CONFIG_BASE_PALO
#define PAL_L2_SW_MBIST                                     0x1204c260
#define PAL_IU_SW_MBIST                                     0x12431d28
#define PAL_MISC_SW_MBIST_CPU                               0x12c00074
#define PAL_NET_STA_SW_MBIST                                0x13058004
#define PAL_DCE_SW_MBIST                                    0x13d00814
#define PAL_IG_SW_MBIST                                     0x141780a0
#define PAL_EG_SW_MBIST                                     0x14488148
#define PAL_PCIE_P_SW_MBIST                                 0x1610fcf0
#define PAL_MISC_SW_HARD_RST                                0x12c00050
#endif /* CONFIG_BASE */


/* Global nested function. */
#define NESTED(name, framesz, rareg)    \
        .text;                  	\
        .globl  name;                   \
        .ent    name;                   \
        .frame  sp, framesz, rareg;     \
name:

/* Leaf function. */
#define LEAF(name)                      \
        .text;                          \
        .ent    name;                   \
name:

/* Function termination. */
#define END(name)                       \
        .size name,.-name;              \
        .end    name

       .set mips32r2           /* to make ehb opcode work */

/* t0: mask = (1 << mask_width) - 1, t2 = timeout */
#define mbist(reg_addr, mask_width, shift_width) \
        .set    noreorder;                       \
        li      t2, 0x8000;                      \
	li	t3, 1;				 \
        sll     t0, t3, mask_width;              \
        sub     t0, t0, 1;                       \
	li	t3, KSEG1ADDR(reg_addr - 4);     \
        sw      t0, 0(t3);     			 \
	ehb;					 \
        li      t3, KSEG1ADDR(reg_addr);	 \
22:     sub	t2, t2, 1;			 \
	beqz	t2, 23f;			 \
        nop;                                     \
	lw      t1, 0(t3);         		 \
	ehb;					 \
        and     t1, t1, t0;                      \
        bne     t0, t1, 22b;                     \
        nop;                                     \
        sll     t0, t0, shift_width;             \
        lw      t1, 0(t3);         		 \
        ehb;                                     \
        and     t1, t1, t0;                      \
        bne     t0, t1, 23f;                     \
        nop;                                     \
        li      v0, 0x0;                         \
        b       24f;                             \
        nop;                                     \
23:     li      v0, 0xdeadbeef;                  \
        move    a1, t1;                          \
24:     .set    reorder


NESTED(mcpu_post, 0, ra)
/* use k0 for ra preservation, v0 for each test P/F, a0 for each test
   identifier, a1 - a3 for fail test addr, data pattern, etc.
   P = 0, F = 0xdeadbeef. */

	li 	t8, MEM_END  
        li      t0, 0xfffffffc
        and     t8, t8, t0

        move    k0, ra

	bal	data_walk
	beq	v0, 0xdeadbeef, 1f
        bal     addr_walk
        beq   	v0, 0xdeadbeef, 1f
        bal     dev_test
        beq     v0, 0xdeadbeef, 1f

// Sereno has no SPRAM - dzwang 
#if defined CONFIG_BASE_PALO
        bal     spram_test
        beq     v0, 0xdeadbeef, 1f
#endif /* CONFIG_BASE */

        la      t9, flush_cache
        jalr    t9
        nop
	bal	cache_test
        beq     v0, 0xdeadbeef, 1f

// Sereno L2 cache MBIST is JTAG type takes very long test time, has to skip - dzwang
#if defined CONFIG_BASE_PALO            
	li	a0, PALO_POST_L2MBIST_FAILED	/* MBIST L2 identifier */
	mbist(PAL_L2_SW_MBIST, 7, 7)
        beq     v0, 0xdeadbeef, 1f
        li      t0, KSEG1ADDR(PAL_MISC_SW_HARD_RST)
        li      t1, 0x20
        sw      t1, 0(t0)
        ehb
        nop
        nop
        li      t1, 0
        sw      t1, 0(t0)
        ehb
        nop
        nop
#if 0
        li      a0, 0xbbbb2     /* MBIST MISC_SW identifier */
        mbist(PAL_MISC_SW_MBIST_CPU, 9, 9)
        beq     v0, 0xdeadbeef, 1f
        li      a0, 0xbbbb7     /* MBIST PCIE identifier */
        mbist(PAL_PCIE_P_SW_MBIST, 4, 4)
        beq     v0, 0xdeadbeef, 1f
#endif
#ifndef POST_SHORT    // This is run during Linux based mfg test, to save test time here - dzwang 
        li      a0, 0xbbbb1     /* MBIST IU identifier */
        mbist(PAL_IU_SW_MBIST, 6, 6)
        beq     v0, 0xdeadbeef, 1f
        li      a0, 0xbbbb3     /* MBIST NET_STA identifier */
        mbist(PAL_NET_STA_SW_MBIST, 13, 16)
        beq     v0, 0xdeadbeef, 1f 
        li      a0, 0xbbbb4     /* MBIST DCE identifier */
        mbist(PAL_DCE_SW_MBIST, 3, 3)
        beq     v0, 0xdeadbeef, 1f
        li      a0, 0xbbbb5     /* MBIST IG identifier */
        mbist(PAL_IG_SW_MBIST, 6, 6)
        beq     v0, 0xdeadbeef, 1f
        li      a0, 0xbbbb6     /* MBIST EG identifier */
        mbist(PAL_EG_SW_MBIST, 5, 5)
        beq     v0, 0xdeadbeef, 1f 
#endif
#endif /* CONFIG_BASE */


1:
	move	ra, k0 
        jr      ra
        nop
END(mcpu_post)

LEAF(data_walk)

	li	a0, PALO_POST_DATAWALK_FAILED	/* data_walk identifier */
        la      t0, KSEG1
        li      t2, 0x1
3:
        sw      t2, 0(t0)
        lw      t1, 0(t0)
        bne     t1, t2, 4f
        sll     t2, t2, 1
        bne     t2, zero, 3b

        li      v0, 0x0
        jr      ra
        nop

4:
        li      v0, 0xdeadbeef
        move    a1, t2
        move    a2, t1
        jr      ra
        nop

END(data_walk)

LEAF(addr_walk)

        li      a0, PALO_POST_ADDRWALK_FAILED      /* addr_walk identifier */
        li      t3, MEM_SIZE
        li      t2, 0xaaaaaaaa
        li      t4, 0x55555555
        li      t5, 0x4       /* Had to work on the word granuality */
        la      t0, KSEG1
        sw      t4, 0(t0)
3:
        addu    t0, t5, KSEG1
        sw      t2, 0(t0)
	nop
        lw      t1, 0(t0)
        bne     t1, t2, 4f
        sll     t5, t5, 1
        blt     t5, t3, 3b

        la      t0, KSEG1
        li      t5, 0x4
        sw      t2, 0(t0)
        la      t0, MEM_START
5:
        sw      t4, 0(t0)
        la      t6, KSEG1
        lw      t1, 0(t6)
        bne     t1, t2, 4f
        li      t7, 0x4
        la      t6, MEM_START
6:                            /* Addr Alias Check */
        lw      t1, 0(t6)
        bne     t1, t2, 7f
8:
        sll     t7, t7, 1
        addu    t6, t7, KSEG1
        blt     t7, t3, 6b
        sw      t2, 0(t0)

        sll     t5, t5, 1
        addu    t0, t5, KSEG1
        blt     t5, t3, 5b

        li      v0, 0x0
        jr      ra
        nop

7:
        beq     t0, t6, 8b
        nop

4:
        li      v0, 0xdeadbeef
        move    a1, t1
	move 	a2, t6
        move    a3, t0
        jr      ra
        nop

END(addr_walk)

LEAF(dev_test)

	li	a0, PALO_POST_DEVTEST_FAILED	/* dev_test identifier */
        move    t3, t8
        la      t0, KSEG1
	li	t1, MEM_START 
1:
	sw      t1, 0(t0)
#ifdef POST_SHORT
	addu    t0, t0, CHUNK_SIZE
        addu    t1, t1, CHUNK_SIZE
#else
	addu    t0, t0, 0x4
	addu    t1, t1, 0x4
#endif
	blt	t0, t3, 1b

	la      t0, KSEG1
        li      t1, MEM_START
2:
	lw	t2, 0(t0)
	bne     t1, t2, 4f
	not	t4, t1
	sw      t4, 0(t0)
#ifdef POST_SHORT
        addu    t0, t0, CHUNK_SIZE 
        addu    t1, t1, CHUNK_SIZE
#else 
        addu    t0, t0, 0x4
        addu    t1, t1, 0x4 
#endif
        blt     t0, t3, 2b

        la      t0, KSEG1
        li      t1, MEM_START
3:
        lw      t2, 0(t0)
        not     t4, t1
        bne     t4, t2, 4f
#ifdef POST_SHORT
        addu    t0, t0, CHUNK_SIZE
        addu    t1, t1, CHUNK_SIZE
#else
        addu    t0, t0, 0x4
        addu    t1, t1, 0x4
#endif
        blt     t0, t3, 3b

        li      v0, 0x0
        jr      ra
        nop

4:
        li      v0, 0xdeadbeef
        move    a1, t0
        move    a2, t2
        jr      ra
        nop
END(dev_test)

#if defined CONFIG_BASE_PALO
LEAF(spram_test)  

        li      a0, PALO_POST_SPRAMTEST_FAILED      /* spram_test identifier */
        li      t3, SPRAM_END
        la      t0, SPRAM_BASE
        li      t1, SPRAM_START
1:
        sw      t1, 0(t0)
        addu    t0, t0, 0x4
        addu    t1, t1, 0x4
        blt     t0, t3, 1b

        la      t0, SPRAM_BASE
        li      t1, SPRAM_START
2:
        lw      t2, 0(t0)
        bne     t1, t2, 4f
        not     t4, t1
        sw      t4, 0(t0)
        addu    t0, t0, 0x4
        addu    t1, t1, 0x4
        blt     t0, t3, 2b

        la      t0, SPRAM_BASE
        li      t1, SPRAM_START
3:
        lw      t2, 0(t0)
        not     t4, t1
        bne     t4, t2, 4f
        addu    t0, t0, 0x4
        addu    t1, t1, 0x4
        blt     t0, t3, 3b

        li      v0, 0x0
        jr      ra
        nop     

4:
        li      v0, 0xdeadbeef
        move    a1, t0
        move    a2, t2
        jr      ra
        nop
END(spram_test)
#endif /* CONFIG_BASE */


#define cacheop(kva, n, linesize, op) \
	.set 	noreorder;	\
	blez	n, 11f; 	\
	addu	t1, kva, n;	\
	subu	t2, linesize, 1; \
	not	t2;		\
	and	t0, kva, t2;	\
	addu	t1, -1;		\
	and	t1, t2;		\
10:	cache	op, 0(t0);	\
	bne	t0, t1, 10b;	\
	addu	t0, linesize;	\
11:	.set	reorder


LEAF(cache_test)
        li      a0, PALO_POST_ICACHETEST_FAILED	/* cache_test Icache identifier */
/* L1 Icache tag & data line test */
        la      t6, CFG_ICACHE_SIZE     /* Set upper to 16KB L1 size */
        la      t0, 0
1:
	sll	t1, t0, 11
#	or	t1, 0x80       /* Set the Valid bit, No Dirty set for L1I */ 
	mtc0    t1, CP0_TAGLO, 0
        ehb
	cache   Index_Store_Tag_I, 0(t0)
        addu    t0, t0, 0x20
        bne     t0, t6, 1b

#if 0   // Disable Icache data line since Palo mips compiler don't like Index_Store_Data_I diags ops, will visit later to see how Menlo work  
        li      t1, 0xaaaaaaaa

        li      t4, 0x20000000   /* Set the WST for Store_Data */ 
        mtc0    t4, CP0_ERRCTL
        ehb
        la      t0, 0
        mtc0    t1, CP0_DATALO
        ehb
        mtc0    t1, CP0_DATAHI
        ehb
2:
	cache   Index_Store_Data_I, 0(t0)
        addu    t0, t0, 0x20
        bne     t0, t6, 2b

        li      t4, 0		/* Clear the WST */ 
        mtc0    t4, CP0_ERRCTL
        ehb
#endif
        la      t0, 0
3:
        cache   Index_Load_Tag_I, 0(t0)
	mfc0    t2, CP0_TAGLO, 0
        ehb
	srl	t2, t2, 11
	bne     t2, t0, 4f
	nop
#if 0   // Disable Icache data line due to compiler issue above  
	mfc0	t2, CP0_DATALO
	ehb
	bne	t2, t1, 4f
	nop
	mfc0    t2, CP0_DATAHI
        ehb
        bne     t2, t1, 4f
      	nop 
#endif
	addu    t0, t0, 0x20
        bne     t0, t6, 3b


/* Comprehensive L1 Dcache Test, on both CPU and memory side, in both rd & wr direction. */
        li      a0, PALO_POST_DCACHETEST_FAILED /* cache_test Dcache identifier */

/* Write directly to memory 1st to be rd via L1C */
        li      t4, 0xaaaaaaaa
        la      t0, KSEG1
        addu    t2, t0, CFG_DCACHE_SIZE
15:
        sw      t4, 0(t0)
        nop
        addu    t0, t0, 0x4
        bne     t0, t2, 15b

/* Enable KSEG0 cacheable, default is uncached - dzwang */
	mfc0	t0, CP0_CONFIG
	ehb
	srl	t0, t0, 3
	sll     t0, t0, 3
	or	t0, 0x3     /* Noncoherent WB */
	mtc0    t0, CP0_CONFIG
	ehb

/* Now check the entire rd direction of Dcache */
        la      t0, KSEG0
        addu    t3, t0, CFG_DCACHE_SIZE
16:
	lw      t2, 0(t0)
	nop
	bne     t4, t2, 4f
        addu    t0, t0, 0x4
        bne     t0, t3, 16b

/* Now fill all 16KB Dcache to be pushed out to memory */
#       la      t0, 0x9fffe000
#       la      t0, 0x8007e000
	la      t0, KSEG0
	addu    t3, t0, CFG_DCACHE_SIZE

	li	t1, 0x55555555
9:
	sw      t1, 0(t0)
	addu    t0, t0, 0x4
        bne     t0, t3, 9b

/* Check each cache line via cache instruction */
	la      t0, 0x0
	la	t6, CFG_DCACHE_SIZE     /* Set upper to 16KB L1 size */

12:				   /* Walk all cache line, not care way 0, 0x800, 0x1000, 0x1800 */ 
        addu    t5, t0, 0x20   	   /* Cache line size is 32B */
11:                                /* Check all data within 32B data line */ 
	cache   Index_Load_Tag_D, 0(t0)
	ehb
	mfc0    t4, CP0_TAGLO, 3    /* Rd DataLo of Dcache */
	ehb
	bne     t4, t1, 4f
	addu    t0, t0, 0x4
	bne	t0, t5, 11b

#       addu    t0, t0, 0x800
	bne     t0, t6, 12b

        move    t4, t1      /* Save the t1 value for later check */
/* Flush the cache by write miss */
	li      t1, 0xffffffff
	move	t0, t3
	addu    t2, t0, CFG_DCACHE_SIZE
	nop
17:
	sw      t1, 0(t0)
/* For some reason, need to put dummy instructions around, otherwise the pipeline harzard/delay */
	nop
	sync
	nop
        addu    t0, t0, 0x20  /* Only need 1 line at a time */ 
        bne     t0, t2, 17b
#if 0 
/* Flush the cache with instruction - side effect */
	la	a0, KSEG0 
	li	a1, CFG_DCACHE_SIZE  /* 16KB cache size */
	li	a2, 0x20    /* Cache line size 32B */
#	cacheop(a0, a1, a2, Index_Writeback_Inv_D)
	cacheop(a0, a1, a2, Hit_Writeback_D)
#endif

/* Now rd back directly from memory to check the WB */
        la      t0, KSEG1
        addu    t2, t0, CFG_DCACHE_SIZE
8:
        lw      t1, 0(t0)
        bne     t4, t1, 4f
        addu    t0, t0, 0x4
        bne     t0, t2, 8b


	li      v0, 0
        jr      ra
        nop

4:
        li      v0, 0xdeadbeef
        move    a1, t0
        jr      ra
        nop
END(cache_test)

