
/*
 * Copyright 2007 Nuova Systems, Inc.  All rights reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: palo.h 21 2016-12-02 21:25:59Z buildsa $"

#ifndef __CONFIG_H
#define __CONFIG_H

#ifdef __mips__
#include "palo/fw/mcpu/include/paloflash.h"
#endif

// #define DEBUG

#define CONFIG_MIPS32		1	/* MIPS 24Kc CPU core	*/
#define CONFIG_PALO		1	/* on a PALO Board	*/

#define CONFIG_BAUDRATE		38400

/* valid baudrates */
#define CFG_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }

#define	CONFIG_TIMESTAMP		/* Print image info with timestamp */

#define CONFIG_MISC_INIT_R

#define CFG_CONSOLE_INFO_QUIET

//-----------------------------------------------------------------------------
//  Auto boot.
//-----------------------------------------------------------------------------
#define CONFIG_AUTOBOOT_PROMPT      "Press CTRL-B to stop boot"
#define CONFIG_BOOTDELAY            0
#define	CONFIG_BOOTCOMMAND          "fwboot"
#define CONFIG_AUTOBOOT_KEYED
#define CONFIG_AUTOBOOT_STOP_STR    "\002"
#define CONFIG_ZERO_BOOTDELAY_CHECK

//-----------------------------------------------------------------------------
//  Command Line.
//-----------------------------------------------------------------------------
#define CONFIG_CMD_NOABBREVIATIONS
#define CONFIG_CMD_BDI
#define CONFIG_CMD_CONSOLE
#define CONFIG_CMD_DIAG
#define CONFIG_CMD_FLASH
#define CONFIG_CMD_ENV
#define CONFIG_CMD_MEMORY

//-----------------------------------------------------------------------------
//  Memory Layout.
//-----------------------------------------------------------------------------
#define CFG_SDRAM_BASE		0x80000000      /* start address of RAM */
#if defined CONFIG_BASE_PALO
#define CFG_SDRAM_LEN_ASIC      0x08000000      /* 128MB */
#define CFG_SDRAM_LEN_FPGA      0x08000000      /* 128MB */
#elif defined CONFIG_BASE_SERENO
#define CFG_SDRAM_LEN_ASIC      0x0e000000      /* 224MB */
#define CFG_SDRAM_LEN_FPGA      0x07000000      /* 112MB */
#elif defined CONFIG_BASE_CRUZ
#define CFG_SDRAM_LEN_ASIC      0x80000000      /* 2GB */
#define CFG_SDRAM_LEN_FPGA      0x80000000      /* 2GB */
#endif /* CONFIG_BASE */
#define CFG_INIT_SP_OFFSET      0x400000        /* for temporary stack */
#define CFG_MALLOC_LEN		128*1024
#define CFG_BOOTPARAMS_LEN	32*1024

//-----------------------------------------------------------------------------
//  PALO Specific Memory Layout.
//-----------------------------------------------------------------------------
#define CFG_UBOOT_RAMTOP        0x80100000      /* 1MB */
#ifdef CONFIG_BASE_CRUZ
#define CFG_GZIPIMG_BASE        0x84000000      /* where to load a gzip img */
#else
#define CFG_GZIPIMG_BASE        0x82000000      /* where to load a gzip img */
#endif /* CONFIG_BASE_CRUZ */


#define CONFIG_CMDLINE_EDITING

/*
 * Miscellaneous configurable options
 */
#ifndef __ASSEMBLY__
unsigned long palo_get_cfghz(void);
unsigned long palo_get_uartclk(void);
#endif

#define	CFG_LONGHELP				/* undef to save memory      */
#define	CFG_PROMPT		"boot> "	/* Monitor Command Prompt    */
#define	CFG_CBSIZE		256		/* Console I/O Buffer Size   */
#define	CFG_PBSIZE (CFG_CBSIZE+sizeof(CFG_PROMPT)+16)  /* Print Buffer Size */
#define CFG_HZ			palo_get_cfghz()
#define	CFG_MAXARGS		16		/* max number of command args*/

#define	CFG_LOAD_ADDR		0x80500000	/* default load address	*/

#define CFG_MEMTEST_START	0x80200000
#define CFG_MEMTEST_END		0x80800000

#define CONFIG_CONS_INDEX       1

//-----------------------------------------------------------------------------
//  Serial Configuration.
//-----------------------------------------------------------------------------
#define CFG_UART_BASE           0xb7f40000
#define CFG_NS16550
#define CFG_NS16550_SERIAL
#define CFG_NS16550_REG_SIZE    4
#define CFG_NS16550_CLK         palo_get_uartclk()
#define CFG_NS16550_COM1        (CFG_UART_BASE + 0)

/* The following #defines are needed to get flash environment right */
#define	CFG_MONITOR_BASE	TEXT_BASE
#define	CFG_MONITOR_LEN		(192 << 10)

#ifndef __ASSEMBLY__
//-----------------------------------------------------------------------------
//  Palo Environment API.
//-----------------------------------------------------------------------------
unsigned long palo_env_addr(void);
unsigned long palo_env_sect_size(void);
#endif

//-----------------------------------------------------------------------------
//  FLASH Configuration.
//-----------------------------------------------------------------------------
#define CFG_FLASH_BASE          (0xa0000000 + PALOFLASH_PHYS)
#define CFG_FLASH_CFI
#define CFG_FLASH_CFI_WIDTH     FLASH_CFI_16BIT
#define CFG_FLASH_CFI_DRIVER
#define CFG_FLASH_PROTECTION
#define CFG_MAX_FLASH_BANKS     1       /* max number of memory banks */
#define CFG_MAX_FLASH_SECT      1024    /* max number of sectors on one chip */

//-----------------------------------------------------------------------------
//  Environment.
//-----------------------------------------------------------------------------
#define CFG_ENV_IS_IN_FLASH

/*
 * The address and sector size of the flash are determined at runtime.
 * This supports building a Palo version of U-Boot that can run on both
 * M29 and P30 flashes with their different partitions.
 */
#define CFG_ENV_ADDR            palo_env_addr()
#define CFG_ENV_SECT_SIZE       palo_env_sect_size()

/*
 * The size of the environment is compile-time defined in line with
 * U-Boot's expectations.  For consistency with existing cards, Palo's
 * environment is fixed at 32K whilst Sereno's is fixed at 128K.
 */
#if defined CONFIG_BASE_PALO
#define CFG_ENV_SIZE            0x8000   /* palo/fw/mcpu/include/ubootenv.h */
#elif defined CONFIG_BASE_SERENO || defined CONFIG_BASE_CRUZ
#define CFG_ENV_SIZE            0x20000  /* palo/fw/mcpu/include/ubootenv.h */
#endif /* CONFIG_BASE */

//-----------------------------------------------------------------------------
//  MIPS Cache Geometry.
//-----------------------------------------------------------------------------
#if defined CONFIG_BASE_PALO
#define CFG_ICACHE_SIZE         16384       /* I-cache total size */
#define CFG_DCACHE_SIZE         16384       /* D-cache total size */
#define CFG_CACHELINE_SIZE      32          /* line size */
#define CFG_CACHEFILL_SIZE      16384       /* bytes cleared to fill cache */
#elif defined CONFIG_BASE_SERENO
#define CFG_ICACHE_SIZE         16384       /* I-cache total size */
#define CFG_DCACHE_SIZE         32768       /* D-cache total size */
#define CFG_CACHELINE_SIZE      32          /* line size */
#define CFG_CACHEFILL_SIZE      32768       /* bytes cleared to fill cache */
#elif defined CONFIG_BASE_CRUZ
#define CFG_ICACHE_SIZE         32768       /* I-cache total size */
#define CFG_DCACHE_SIZE         32768       /* D-cache total size */
#define CFG_CACHELINE_SIZE      32          /* line size */
#define CFG_CACHEFILL_SIZE      32768       /* bytes cleared to fill cache */
#endif /* CONFIG_BASE */

#endif	/* __CONFIG_H */
