
/*
 * Copyright (c) 2008 Nuova Systems, Inc.  All rights reserved.
 */
#ident "$Id: nictool_adapter_defs.h 59 2017-01-21 21:37:10Z buildsa $"

#ifndef _NICTOOL_ADAPTER_DEFS_H_
#define _NICTOOL_ADAPTER_DEFS_H_

#ifndef PACKED
#define PACKED  __attribute__((packed))
#endif

#ifdef __WINDOWS__
#pragma pack(1)
#endif
/******************************************************************************
 * Generic adapter sub-commands.
 */
#define NT_ADCMD(x)     NTCLSCMD(NTCLS_ADAPTER, x)

#define ADAPTER_ALERT           NT_ADCMD(0) // asynchronous alert from adapter
#define ADAPTER_PURGEEVENTS     NT_ADCMD(1) // purge event data
#define ADAPTER_PING            NT_ADCMD(2) // ping adapter
#define ADAPTER_IDENTIFY        NT_ADCMD(3) // obtain adapter identification
#define ADAPTER_SETTIME         NT_ADCMD(4) // set adapter time of day
#define ADAPTER_FILE_OPEN       NT_ADCMD(5) // open a file
#define ADAPTER_FILE_READ       NT_ADCMD(6) // read file data
#define ADAPTER_FILE_WRITE      NT_ADCMD(7) // write file data
#define ADAPTER_FILE_CLOSE      NT_ADCMD(8) // close a file
#define ADAPTER_FWUPDATE        NT_ADCMD(9) // perform firmware update
#define ADAPTER_GETFWUPDSTATUS  NT_ADCMD(10)// retrieve update status
#define ADAPTER_GETFWINVENTORY  NT_ADCMD(11)// retrieve firmware inventory
#define ADAPTER_SETIPPARAMS     NT_ADCMD(12)// set IP parameters
#define ADAPTER_GETIPPARAMS     NT_ADCMD(13)// get IP parameters
#define ADAPTER_SETEXTDATA      NT_ADCMD(14)// set extra function data
#define ADAPTER_GETEXTDATA      NT_ADCMD(15)// get extra function data
#define ADAPTER_GETEVENTS       NT_ADCMD(16)// retrieve event data
#define ADAPTER_SETALERTPERIOD  NT_ADCMD(17)// set async alert period
#define ADAPTER_GETALERTPERIOD  NT_ADCMD(18)// get async alert period
#define ADAPTER_FWACTIVATE      NT_ADCMD(19)// activate a firmware image
#define ADAPTER_SETIDENTITYINFO NT_ADCMD(20)// set adapter identity info
#define ADAPTER_GETIDENTITYINFO NT_ADCMD(21)// get adapter identity info
#define ADAPTER_GETSYSTEMSTATUS NT_ADCMD(22)// get system cpu and mem info
#define ADAPTER_GETTECHSUPPORT  NT_ADCMD(23)// get tech support info
#define ADAPTER_GETRESTARTCAUSE NT_ADCMD(24)// get firmware restart cause
#define ADAPTER_RCMD            NT_ADCMD(25)// run a remote command
#define ADAPTER_RESERVED_26     NT_ADCMD(26)// reserved(26)
#define ADAPTER_PROBEREQ        NT_ADCMD(27)// asynchronous probe request
#define ADAPTER_PROBERESP       NT_ADCMD(28)// asynchronous probe response
#define ADAPTER_GETVIFSTATE     NT_ADCMD(29)// get the state of a vif
#define ADAPTER_GETPHYUPDINFO   NT_ADCMD(30)// get phy ucode update information
#define ADAPTER_PHYUPDATE       NT_ADCMD(31)// start phy ucode update
#define ADAPTER_GETLOADEDFW     NT_ADCMD(32)// get loaded firmware info
#define ADAPTER_SETPARAM        NT_ADCMD(33)// set adapter parameter
#define ADAPTER_GETPARAM        NT_ADCMD(34)// get adapter parameter
#define ADAPTER_SETTIMEZONE     NT_ADCMD(35)// set adapter timezone
#define ADAPTER_RESTORE_DEFCFG  NT_ADCMD(36)// restore default configuration
#define ADAPTER_FACTORY_RESET   NT_ADCMD(37)// complete wipe incl. logs

#define ADAPTER_PATHMAX     256
#define ADAPTER_PARAMSMAX   512

/*
 * ADAPTER_IDENTIFY
 */
typedef struct {
    u_int16_t   type;
    char        description[32];
    char        hw_version[32];
    char        sw_version[32];
} PACKED adapter_identify_reply_t;
enum {
    /*
     * Pre-Balboa UCSM ASSERTs adapter_identify.type is
     * either _MENLO or _PALO.
     */
    ADAPTERTYPE_MENLO,
    ADAPTERTYPE_PALO,
    /*
     * Proberesp.type may specify one of these more specific types.
     */
    ADAPTERTYPE_MENLOQ,
    ADAPTERTYPE_MENLOE,
    ADAPTERTYPE_BAYLANDS,
    ADAPTERTYPE_PALOALTO,
    ADAPTERTYPE_MONTEREYPARK,
    ADAPTERTYPE_GREENWOOD,
    ADAPTERTYPE_VASONA,
    ADAPTERTYPE_COTATI,
    ADAPTERTYPE_LEXINGTON,
    ADAPTERTYPE_CALIMESA,
    ADAPTERTYPE_KIRKWOODLAKE,
    ADAPTERTYPE_ICEHOUSE,
    ADAPTERTYPE_OAKLAND,
    ADAPTERTYPE_CALISTOGA,
    ADAPTERTYPE_MONTCLAIR,
    ADAPTERTYPE_SUSANVILLE,
    ADAPTERTYPE_MOUNTAINVIEW,
    ADAPTERTYPE_WALNUTCREEK,
    ADAPTERTYPE_CLEARLAKE,
    ADAPTERTYPE_MOUNTTIAN,
    ADAPTERTYPE_TORRANCE,
    ADAPTERTYPE_MOUNTTIAN2,
    ADAPTERTYPE_CLAREMONT,
};

/*
 * ADAPTER_PROBERESP
 */
typedef adapter_identify_reply_t adapter_proberesp_t;

/*
 * ADAPTER_SETTIME
 */
typedef struct {
    u_int32_t   sec;
    u_int32_t   usec;
} PACKED adapter_settime_msg_t;

/*
 * ADAPTER_SETTIMEZONE
 */
#define TZ_NAMELEN_MAX   256

typedef struct {
    u_int32_t   extkey;                 // ext data key (zero for none)
    char tz_name[TZ_NAMELEN_MAX];       // Timezone name
    u_int8_t    data[0];                // configuration data
} PACKED adapter_settimezone_msg_t;

/*
 * ADAPTER_FWUPDATE
 */
// version 1
typedef struct {
    u_int32_t   imgsize;                // size in bytes of new firmware image
    u_int8_t    fwtype;                 // image type to update
    u_int8_t    pathlen;                // path name length (incl nul)
    char        path[0];                // nul-terminated filename on server
} PACKED adapter_fwupdate_v1_msg_t;

// version 2
typedef struct {
    u_int32_t   imgsize;                // size in bytes of new firmware image
    u_int8_t    fwtype;                 // image type to update
    u_int16_t   pathlen;                // path name length (incl nul)
    u_int16_t   paramslen;              // parameter string length (incl nul)
    char        data[0];                // path followed by params
} PACKED adapter_fwupdate_v2_msg_t;

enum {
    ADAPTER_FWTYPE_APP,                 // main application firmware
    ADAPTER_FWTYPE_DIAG,                // diagnostic firmware
    ADAPTER_FWTYPE_BOOT,                // boot block firmware
    ADAPTER_FWTYPE_EXT,                 // v2-only: non-adapter firmware
};

/*
 * ADAPTER_GETFWUPDSTATUS
 */
// version 1
typedef struct {
    u_int8_t    status;                 // overall status ADAPTER_FWUST_xxx
    u_int8_t    failcode;               // failure code ADAPTER_FWERR_xxx
} PACKED adapter_getfwupdstatus_v1_reply_t;

// version 2
typedef struct {
    u_int8_t    status;                 // overall status ADAPTER_FWUST_xxx
    u_int8_t    failcode;               // failure code ADAPTER_FWERR_xxx
    u_int8_t    phase;                  // update phase ADAPTER_FWPHASE_xxx
    u_int8_t    phase_progress;         // phase progress percent (0-100)
    u_int8_t    overall_progress;       // overall progress percent (0-100)
} PACKED adapter_getfwupdstatus_v2_reply_t;
enum {
    ADAPTER_FWUST_NONE,                 // fwupdate never issued
    ADAPTER_FWUST_INPROGRESS,           // update in progress
    ADAPTER_FWUST_COMPLETED,            // firmware update complete
    ADAPTER_FWUST_FAILED,               // firmware update failed
};

enum {
    ADAPTER_FWERR_NONE,                 // no error
    ADAPTER_FWERR_XFERREJECTED,         // file transfer request rejected
    ADAPTER_FWERR_XFERTIMEDOUT,         // file transfer timed out
    ADAPTER_FWERR_XFERERR,              // misc file transfer error
    ADAPTER_FWERR_BADIMAGE,             // file data invalid
    ADAPTER_FWERR_FLASHERR,             // flash write or verify failed
    ADAPTER_FWERR_PROTOERR,             // transfer protocol error
    ADAPTER_FWERR_INTERNAL,             // misc internal error
    ADAPTER_FWERR_NOMEM,                // out of memory to stage image
};

enum {
    ADAPTER_FWPHASE_NONE,               // no operation
    ADAPTER_FWPHASE_TRANSFERRING,       // firmware transfer in progress
    ADAPTER_FWPHASE_ERASING,            // erasing old firmware
    ADAPTER_FWPHASE_WRITING,            // writing new firmware
    ADAPTER_FWPHASE_VERIFYING,          // verifying write
};

/*
 * ADAPTER_FILE_OPEN
 */
typedef struct {
    u_int8_t    mode;                   // open mode
    u_int8_t    pathlen;                // path name length (incl nul)
    char        path[0];                // path name
} PACKED adapter_file_open_msg_t;
enum {
    ADAPTER_FILE_OPEN_READ,             // open for read
    ADAPTER_FILE_OPEN_WRITE             // open for write (not supported)
};

typedef struct {
    u_int32_t   cookie;                 // file cookie
} PACKED adapter_file_open_reply_t;

/*
 * ADAPTER_FILE_READ
 */
typedef struct {
    u_int32_t   offset;                 // file offset
    u_int16_t   nbytes;                 // number of bytes requested
    u_int32_t   cookie;                 // file cookie
} PACKED adapter_file_read_msg_t;

/*
 * ADAPTER_FILE_CLOSE
 */
typedef struct {
    u_int32_t   cookie;                 // file cookie
} PACKED adapter_file_close_msg_t;

/*
 * ADAPTER_GETFWINVENTORY
 */
typedef struct {
    u_int8_t    nentries;               // number of list entries
    struct {
        u_int8_t fwtype;                // firmware type (as fwupdate_msg_t)
        u_int8_t flags;                 // firmware flags
        char    version[32];            // firmware version
    } PACKED entry[0];
} PACKED adapter_getfwinventory_reply_t;
#define ADAPTER_FW_STARTUP      0x1     // firmware will be loaded at startup
#define ADAPTER_FW_RUNNING      0x2     // firmware currently running
#define ADAPTER_FW_ERROR        0x4     // firmware image is corrupt
#define ADAPTER_FWNUM_MAX       16      // 16 entries maximum

/*
 * ADAPTER_FWACTIVATE
 */
// version 1
typedef struct {
    u_int8_t    idx;                    // index of entry to activate
} PACKED adapter_fwactivate_v1_msg_t;

// version 2
typedef struct {
    u_int8_t    fwtype;                 // image type to activate
    u_int16_t   paramslen;              // parameter string length (incl nul)
    char        params[0];              // parameter string
} PACKED adapter_fwactivate_v2_msg_t;

/*
 * ADAPTER_SETIPPARAMS
 * ADAPTER_GETIPPARAMS
 */
typedef struct {
    u_int8_t    portid;                 // uplink port
    u_int32_t   address;                // IP address
    u_int32_t   netmask;                // Network mask
    u_int32_t   gateway;                // Gateway address (or zero)
} PACKED adapter_setipparams_msg_t;
typedef adapter_setipparams_msg_t adapter_getipparams_msg_t;
#define ADAPTER_PORTID_BMC      0xf0    // port id for the BMC
#define ADAPTER_PORTID_MGMT     0xf1    // port id for the MGMT interface

/*
 * ADAPTER_SETIDENTITYINFO
 * ADAPTER_GETIDENTITYINFO
 */
#define ADAPTER_IDF_VHOSTNAME_LEN       64
#define ADAPTER_IDF_CSN_LEN             64
#define ADAPTER_IDF_TFTP_DIR_NAME_LEN   256
typedef struct {
    char    virtual_hostname[ADAPTER_IDF_VHOSTNAME_LEN];
    char    csn_string[ADAPTER_IDF_CSN_LEN];
    char    tftp_core_dir[ADAPTER_IDF_TFTP_DIR_NAME_LEN];
} PACKED adapter_setidentityinfo_msg_t;
typedef adapter_setidentityinfo_msg_t adapter_getidentityinfo_reply_t;

/*
 * ADAPTER_GETSYSTEMSTATUS
 */
typedef struct {
    u_int32_t   extkey;                 // external data key
    u_int32_t   nbytes;                 // ext data length
} PACKED adapter_getsystemstatus_repl_t;
#define ADAPTER_SYSTEMSTATUS_MAXBYTES   (128 * 1024)  // 128k max data

/*
 * ADAPTER_GETTECHSUPPORT
 */
#define ADAPTER_GTS_TFTP_FILE_MAX_LEN   256
typedef struct {
    u_int32_t   tftphost;
    char        filename[ADAPTER_GTS_TFTP_FILE_MAX_LEN];
} PACKED adapter_gettechsupport_msg_t;

/*
 * ADAPTER_SETEXTDATA
 */
typedef struct {
    u_int32_t   key;                    // key to refer to data
    u_int32_t   len;                    // total data length
    u_int32_t   offset;                 // buffer offset
    u_int8_t    data[0];                // data
} PACKED adapter_setextdata_msg_t;

/*
 * ADAPTER_GETEXTDATA
 */
typedef struct {
    u_int32_t   key;                    // key to refer to data
    u_int32_t   offset;                 // buffer offset
    u_int16_t   nbytes;                 // data length
} PACKED adapter_getextdata_msg_t;

/*
 * ADAPTER_GETEVENTS
 */
typedef struct {
    u_int8_t    num;                    // max number of events to retrieve
    u_int8_t    purge;                  // auto-purge events
} PACKED adapter_getevents_msg_t;

#define ADAPTER_EVDATAMAX       512     // 512 bytes of data maximum

typedef struct {
    u_int32_t   timestamp;              // first occurance timestamp
    u_int32_t   evcode;                 // event code
    u_int16_t   count;                  // number of event occurances
    u_int16_t   len;                    // event data length
    u_int8_t    objtype;                // event object type
    u_int8_t    data[0];                // event data
} PACKED adapter_event_msg_t;
enum {
    ADAPTER_EVOBJTYPE_NONE,             // no data
    ADAPTER_EVOBJTYPE_FLAG,             // single byte flag
    ADAPTER_EVOBJTYPE_UINT32,           // 4-byte integer
    ADAPTER_EVOBJTYPE_STRING,           // null-terminated string
    ADAPTER_EVOBJTYPE_OPAQUE,           // event-specific object
};

typedef struct {
    u_int32_t   eventid;                // first event id
    u_int8_t    timemode;               // flags
#ifdef __WINDOWS__
    UCHAR               event[0];       //Handle MS compiler error C2233
#else    
    adapter_event_msg_t event[0];       // events
#endif    
} PACKED adapter_getevents_reply_t;
enum {
    ADAPTER_EVTIME_NONE,                // events not timestamped
    ADAPTER_EVTIME_UNSYNCD,             // unsynchronized timestamps
    ADAPTER_EVTIME_SYNCD                // synchronized timestamps
};

/*
 * ADAPTER_PURGEEVENTS
 */
typedef struct {
    u_int32_t   eventid;                // highest event id to purge
} PACKED adapter_purgeevents_msg_t;

/*
 * ADAPTER_SETALERTPERIOD
 * ADAPTER_GETALERTPERIOD
 */
typedef struct {
    u_int32_t   alertperiod;            // alert period in seconds
} PACKED adapter_setalertperiod_msg_t;

/*
 * ADAPTER_GETRESTARTCAUSE
 */
typedef struct {
    u_int32_t   cause;                  // restart cause
} PACKED adapter_getrestartcause_reply_t;
enum {
    ADAPTER_RSTCAUSE_UNKNOWN,           // restart for unknown reason
    ADAPTER_RSTCAUSE_POWERON,           // power on
    ADAPTER_RSTCAUSE_RESET,             // reset
    ADAPTER_RSTCAUSE_WATCHDOG,          // watchdog timeout
    ADAPTER_RSTCAUSE_PANIC,             // firmware panic
    ADAPTER_RSTCAUSE_COMMAND,           // commanded restart
};

/*
 * ADAPTER_RCMD
 */
#define ADAPTER_RCMD_LEN_MAX            255
typedef struct {
    u_int8_t    cmdlen;                 // command length (incl. nul)
    char        cmd[0];                 // command string
} PACKED adapter_rcmd_msg_t;

typedef struct {
    u_int32_t   extkey;                 // external data key
    u_int32_t   nbytes;                 // ext data length
} PACKED adapter_rcmd_repl_t;
#define ADAPTER_RCMD_MAXBYTES   (64 * 1024)   // 64k max data

typedef struct {
    char            magic[6];           // magic format prefix
    u_int32_t       session;            // session id
    char            echo;               // echo on challenge
    char            data[0];            // command/response data
} PACKED adapter_rcmd_fmt_t;
#define ADAPTER_RCMD_MAGIC      "<rcmd:"

/*
 * ADAPTER_GETVIFSTATE
 */
enum {
    ADAPTER_VIFSTATE_UP,                // VIF is up
    ADAPTER_VIFSTATE_DOWN,              // VIF is down
    ADAPTER_VIFSTATE_STANDBY            // VIF is on standby
};

// vif_state_info error flags
#define ADAPTER_VIF_ERR_MTU_MISCONFIG   0x01  // network MTU < provisioned MTU

typedef struct {
    u_int32_t vifid;
    u_int8_t uifid;
} PACKED adapter_getvifstate_msg_t;

typedef struct {
    u_int8_t state;
    u_int32_t err_flags;
} PACKED adapter_getvifstate_reply_t;

/*
 * ADAPTER_GETLOADEDFW
 */
typedef struct {
    u_int8_t    fwtype;                 // loaded firmware type
} PACKED adapter_getloadedfw_reply_t;
enum {
    ADAPTER_LOADEDFW_STARTUP,           // loaded startup image
    ADAPTER_LOADEDFW_BACKUP,            // loaded backup image
    ADAPTER_LOADEDFW_UNKNOWN,           // loaded unknown image
};


/*
 * ADAPTER_GETPHYUPDINFO
 */
typedef struct {
    u_int32_t   portmask;               // bitmap of ports to update
} PACKED adapter_getphyupdinfo_msg_t;

typedef struct {
    u_int8_t    portid;                 // port id
    u_int8_t    updstatus;              // update status
    char        info[32];               // phy info string
} PACKED adapter_phyupdinfo_t;

typedef struct {
    u_int8_t    nports;                 // port count
    adapter_phyupdinfo_t info[0];       // phy records
} PACKED adapter_getphyupdinfo_reply_t;
enum {
    ADAPTER_PHYUPDST_NOTAPPLICABLE,     // not applicable (e.g. 8072)
    ADAPTER_PHYUPDST_UNAVAILABLE,       // not available (e.g. bad firmware)
    ADAPTER_PHYUPDST_NOTREQUIRED,       // not required (e.g. already updated)
    ADAPTER_PHYUPDST_REQUIRED,          // required and available (ready to go)
    ADAPTER_PHYUPDST_INPROGRESS,        // in progress
    ADAPTER_PHYUPDST_COMPLETE,          // completed successfully
    ADAPTER_PHYUPDST_FAILED             // unsuccessful
};

/*
 * ADAPTER_PHYUPDATE
 */
typedef struct {
    u_int32_t   portmask;               // bitmap of ports to update
} PACKED adapter_phyupdate_msg_t;

/*
 * ADAPTER_SET_PARAM
 */
typedef struct {
    char    x[3];                       // format is name\0val\0 - min 3 bytes
} PACKED adapter_setparam_msg_t;

/*
 * ADAPTER_GET_PARAM
 */
typedef struct {
    char    x[2];                       // format is name\0 - min 2 bytes
} PACKED adapter_getparam_msg_t;

typedef struct {
    char    x;                          // format is val\0 - min 1 byte
} PACKED adapter_getparam_reply_t;

/*
 * ADAPTER_PING V2
 */
typedef struct {
    u_int32_t reqlen;                   // request length (incl nul)
    u_int32_t replylen;                 // reply length (incl nul)
    char      data[0];                  // request buffer
} PACKED adapter_ping_v2_msg_t;

typedef struct {
    u_int32_t replylen;                 // reply length (incl nul)
    char      data[0];                  // reply buffer
} PACKED adapter_ping_v2_reply_t;

/******************************************************************************
 * Generic adapter events.
 */
#define NT_ADEV(x, t)    NTCLSEV(NTCLS_ADAPTER, ADAPTER_EVOBJTYPE_##t, x)

/*
 * System Software Events
 */

// SYS_RESTART (data = restart cause)
#define ADAPTER_EV_SYS_RESTART              NT_ADEV(0, UINT32)

// SYS_FWUPDATE_COMPLETE (data = 0 for success, or ADAPTER_FWERR_xxx code)
#define ADAPTER_EV_SYS_FWUPDATE_COMPLETE    NT_ADEV(1, UINT32)

// SYS_NONSTARTUPFW_LOADED
#define ADAPTER_EV_SYS_NONSTARTUPFW_LOADED  NT_ADEV(2, NONE)

/*
 * vniccfgd failure events
 */
#define ADAPTER_EV_SYS_VNICCFG_LOAD_FAILURE NT_ADEV(3, NONE)

/*
 * Physical link events (data = portid)
 */
#define ADAPTER_EV_LINK_UP                  NT_ADEV(100, UINT32)
#define ADAPTER_EV_LINK_DOWN                NT_ADEV(101, UINT32)
#define ADAPTER_EV_LINK_ERROR_THRESHOLD     NT_ADEV(102, UINT32)

#define ADAPTER_EV_LINK_INIT_ERROR          NT_ADEV(103, UINT32)
#define ADAPTER_EV_LINK_PHY_UNSUPPORTED     NT_ADEV(104, UINT32)
#define ADAPTER_EV_LINK_PHY_FW_UNSUPPORTED  NT_ADEV(105, UINT32)

/*
 * VIF Events (data = adapter_vifev_t)
 */
typedef struct {
    u_int32_t   vifid;                  // vif id (network byte order)
    u_int8_t    portid;                 // uplink port
} PACKED adapter_vifev_t;
#define ADAPTER_EV_VIF_UP                   NT_ADEV(200, OPAQUE)
#define ADAPTER_EV_VIF_DOWN                 NT_ADEV(201, OPAQUE)
#define ADAPTER_EV_VIF_STANDBY              NT_ADEV(202, OPAQUE)
#define ADAPTER_EV_VIF_ERR                  NT_ADEV(203, OPAQUE)
#define ADAPTER_EV_VIF_MTU_MISCONFIG        NT_ADEV(204, OPAQUE)


#endif
