/*
 * Copyright 2006-2007 Nuova Systems, Inc.  All rights reserved.
 * Copyright 2014 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: ecpu_res.h 46 2016-12-03 23:17:24Z buildsa $"

#ifndef __ECPU_RES_H__
#define __ECPU_RES_H__

//
// This file documents the global resources in palo shared among the three
// ecpus.  This includes interrupt resources on the mcpu and edma channels
// across all three.
//

//
// *** If modifying interrupt positions, update (at least):
//      drivers/enic/ecpu_intlvl.h
//      drivers/palo/rq2wq.h/ecpu_intlvl.h
//

//
// mnic objects.
// There are four mnics:
//      0 & 1       - uif 0 and uif 1.
//      2           - BMC mnic
// For Sereno and Cruz, there are 8 more mnics:
//      3 - 10      - mnics for each uif subport
enum {
    MNIC_PORT_0,
    MNIC_PORT_1,
    MNIC_BMC,
    MNIC_PORT_0_0,
    MNIC_PORT_0_1,
    MNIC_PORT_0_2,
    MNIC_PORT_0_3,
    MNIC_PORT_1_0,
    MNIC_PORT_1_1,
    MNIC_PORT_1_2,
    MNIC_PORT_1_3,
};

/*
 * palotool interface indices for the mnics
 */
enum {
    PTIFIDX_PORT_0,
    PTIFIDX_PORT_1,
    PTIFIDX_BMC,
    PTIFIDX_MMNIC0, /* Index for mgmt mnic on host0 */
};

#define MNIC_EXT_COUNT          2       // external (uif) mnics
#ifdef CONFIG_BASE_PALO
#define MNIC_INT_COUNT          1       // internal (loopback) mnics
#endif // CONFIG_BASE_PALO
#if defined CONFIG_BASE_SERENO || defined CONFIG_BASE_CRUZ
#define MNIC_INT_COUNT          9       // internal mnics
#endif // CONFIG_BASE_SERENO || CONFIG_BASE_CRUZ
#define MNIC_COUNT              (MNIC_INT_COUNT + MNIC_EXT_COUNT)

#define MNIC_IS_EXT(idx)        ((idx) < MNIC_BMC)
#define MNIC_IS_INT_LOOPBACK(idx)   ((idx) == MNIC_BMC)
#define MNIC_IS_INT_RAW(idx)    ((idx) >= MNIC_PORT_0_0 && \
                                 (idx) <= MNIC_PORT_1_3)
#define MNIC_UIF(idx)           (MNIC_IS_EXT(idx) ? idx : \
                                 ((idx) < MNIC_PORT_1_0 ? 0 : 1))

/* For mgmt mnics, uif-0 is the default uif */
#define MNIC_MGMT_UIF           0 

#define MNIC_UIFPORT(idx)       (!MNIC_IS_INT_RAW(idx) ? (-1) : \
                                 ((idx) <= MNIC_PORT_0_3 ? \
                                  (idx) - MNIC_PORT_0_0 : \
                                  (idx) - MNIC_PORT_1_0))
#define MNIC_UIF2IDX(uif)       (uif)
#ifdef CONFIG_BASE_PALO
#define MNIC_UIFPORT2IDX(uif, uifport) MNIC_UIF2IDX((uif))
#endif /* CONFIG_BASE_PALO */
#if defined CONFIG_BASE_SERENO || defined CONFIG_BASE_CRUZ
#define MNIC_UIFPORT2IDX(uif, uifport) \
        ((uifport) + ((uif) == 0 ? MNIC_PORT_0_0 : MNIC_PORT_1_0))
#endif /* CONFIG_BASE_SERENO */

//
// This section documents the internal ecpu interrupt controller sources.
// Some of the sources are hardwired to the interrupt controller at
// the specific interrupt source and those are marked with (hw).
// The others are software selected and can be moved.
//
// The *_INT_LVL and *_INT_SRC parameters are often used as arguments
// to the intrc_set_int_level_addr_enc() macro in "intrc_regs_defs.h"
// to create the correct "set" address to generate (or have hardware
// generate) the associated interrupt.
//

// level 0 sources

#define ECPU1_PANIC_LVL         0
#define ECPU1_PANIC_SRC         0       // ecpu1 panic signal to mcpu

#define ECOM_INT_LVL            0
#if defined(CONFIG_BASE_PALO) || defined(CONFIG_BASE_SERENO)
#define ECOM_FCPU1_INT_SRC      1       // (sw) ecom intr from fcpu1
#define ECOM_FCPU2_INT_SRC      2       // (sw) ecom intr from fcpu2
#define ECOM_ICPU1_INT_SRC      3       // (sw) ecom intr from icpu1
#define ECOM_ICPU2_INT_SRC      4       // (sw) ecom intr from icpu2
#elif defined(CONFIG_BASE_CRUZ)
#define ECOM_FCPU1_INT_SRC      1       // (sw) ecom intr from fcpu1
#define ECOM_FCPU2_INT_SRC      2       // (sw) ecom intr from fcpu2
#define ECOM_FCPU3_INT_SRC      3       // (sw) ecom intr from fcpu3
#define ECOM_FCPU4_INT_SRC      4       // (sw) ecom intr from fcpu4
#define ECOM_ICPU1_INT_SRC      5       // (sw) ecom intr from icpu1
#define ECOM_ICPU2_INT_SRC      6       // (sw) ecom intr from icpu2
#define ECOM_ICPU3_INT_SRC      7       // (sw) ecom intr from icpu3
#define ECOM_ICPU4_INT_SRC      8       // (sw) ecom intr from icpu4
#else
#error "CONFIG_BASE not defined"
#endif /* CONFIG_BASE */

#define FLS_INT_LVL             0
#if defined(CONFIG_BASE_PALO) || defined(CONFIG_BASE_SERENO)
#define FLS_CQ_INT_SRC          5        // (sw) fls intr
#elif defined(CONFIG_BASE_CRUZ)
#define FLS_CQ_INT_SRC          9
#else
#error "CONFIG_BASE not defined"
#endif /* CONFIG_BASE */

#define VCONS_INT_LVL           0	// mcpu side of the ecpu vconsole
#define VCONS1_INT_SRC          12
#define VCONS2_INT_SRC          13
#define VCONS3_INT_SRC          14
#define VCONS4_INT_SRC          15
#define GPIO_INT_LVL            0
#define GPIO0_INT_SRC           22      // (hw) GPIO0
#define GPIO1_INT_SRC           23      // (hw) GPIO1
#define GPIO2_INT_SRC           24      // (hw) GPIO2
#define GPIO3_INT_SRC           25      // (hw) GPIO3

#define SPI_INT_LVL             0
#define SPI_INT_SRC             26      // (hw) SPI interface

#define I2C_INT_LVL             0
#define I2C0_INT_SRC            27      // (hw) i2c interface 0
#define I2C1_INT_SRC            28      // (hw) i2c interface 1

#define UART_INT_LVL            0
#define UART0_INT_SRC           29      // (hw) uart0
#define UART1_INT_SRC           30      // (hw) uart1

#define MDIO_INT_LVL            0
#define MDIO_INT_SRC            31      // (hw) MDIO interface

// level 1 sources

#define MCP_EDMA_INT_LVL        1
#define MCP_EDMA_INT_SRC        0       // (sw) mcp edma completion (0 - 15)

#define DEVCMD2_INT_LVL         1
#define DEVCMD2_INT_SRC         16      // (sw) devcmd2 arrival

#define MCP_LEARN_INT_LVL       1
#define MCP_LEARN_INT_SRC       17      // (sw) mcp nb_learn completion

#define FIPD_INT_LVL            1
#define FIPD_WCQ_INT_SRC        18      // (sw) fipd intr on wqs
#define FIPD_RCQ_INT_SRC        19      // (sw) fipd intr on rqs
#define FIPD_QERR_INT_SRC       20      // (sw) fipd intr on queue errors

#define RDMA_WQERR_INT_LVL      1
#define RDMA_WQERR_INT_SRC      21      // (hw) RDMA fw wq shared error intr
#define RDMA_RSTWQ_INT_LVL      1
#define RDMA_RSTWQ_INT_SRC      22      // (hw) RDMA reset exch wq cmpl intr

#define RNAXI0_INT_LVL          1
#define RNAXI0_INT_SRC          28      // (hw) other rnaxi slaves
#define RNAXI1_INT_LVL          1
#define RNAXI1_INT_SRC          29      // (hw) rnaxi1: ingress,egress
#define RNAXI2_INT_LVL          1
#define RNAXI2_INT_SRC          30      // (hw) rnaxi2: intr ctlr, no rnaxi
#define RNAXI3_INT_LVL          1
#define RNAXI3_INT_SRC          31      // (hw) rnaxi3: pcie INT
#if defined CONFIG_BASE_PALO || defined CONFIG_BASE_SERENO
#define RNAXI_PCIEC_INT_LVL   RNAXI0_INT_LVL // alias for pciec
#define RNAXI_PCIEC_INT_SRC   RNAXI0_INT_SRC // alias for pciec
#define RNAXI_PCIEP_INT_LVL   RNAXI3_INT_LVL // alias for pciep
#define RNAXI_PCIEP_INT_SRC   RNAXI3_INT_SRC // alias for pciep
#endif // PALO || SERENO
#define RNAXI_IG_INT_LVL      RNAXI1_INT_LVL // alias for ig
#define RNAXI_IG_INT_SRC      RNAXI1_INT_SRC // alias for ig
#define RNAXI_EG_INT_LVL      RNAXI1_INT_LVL // alias for eg
#define RNAXI_EG_INT_SRC      RNAXI1_INT_SRC // alias for eg
#if defined CONFIG_BASE_CRUZ
#define RNAXI_PCIE_INT_LVL    RNAXI3_INT_LVL // alias for pcie
#define RNAXI_PCIE_INT_SRC    RNAXI3_INT_SRC // alias for pcie
#define RNAXI_RC_INT_LVL       RNAXI0_INT_LVL
#define RNAXI_RC_INT_SRC       RNAXI0_INT_SRC
#endif // CRUZ

// level 2 sources

#define MNIC_INT_LVL(idx)       2
#define MNIC_INT_SRC_BASE       (32 - MNIC_COUNT)
#define MNIC_INT_SRC(idx)       (MNIC_INT_SRC_BASE + (idx))

#define BMC_SLOW_INT_LVL        2       // slow-path ncsi forwarding
#define BMC_SLOW_INT_SRC        (MNIC_INT_SRC_BASE - 1)

#define HWIO_ECPU_INT_LVL       2       // HWIO tests can use this lvl
#define HWIO_ECPU_INT_SRC_BASE  16      //   starting at this src
#define HWIO_ECPU_INT_SRC_COUNT 4

// level 3 sources
#if defined CONFIG_BASE_CRUZ
// Reserving 10 interrupts for MSI/MSI-X
#define RC_DEVICE_INT_LVL           3
#define RC_DEVICE_INT_SRC_BASE      0
#define RC_DEVICE_INT_SRC_COUNT     12
// RC Indirect handling interrupt
#define RC_INDIRECT_INT_LVL         3
#define RC_INDIRECT_INT_SRC         12
// RC device legacy interrupt
#define RC_DEVICE_LEGACY_INT_LVL    3
#define RC_DEVICE_LEGACY_INT_BASE   13
// snic int13
#define SNIC_INT13_INT_LVL          3
#define SNIC_INT13_INT_SRC          14
#endif
// Palonic shared interrupt
#define PALO_ENIC_INT_LVL_SHARED    3
#define PALO_ENIC_INT_SRC_SHARED    15

// level 4 sources

#define PCIE_IND_REQ_INT_LVL    4
#define PCIE_IND_REQ_INT_SRC    0


//
// This section documents the edma channel allocations
// Palo contains 8 edma channels
//

#define FCPU1_FCP_EDMA_CHNL     0
#define FCPU2_FCP_EDMA_CHNL     1
#if defined CONFIG_BASE_CRUZ
#define FCPU3_FCP_EDMA_CHNL     2
#define FCPU4_FCP_EDMA_CHNL     3
#endif /* CONFIG_BASE_CRUZ */
#define MCPU_MCP_EDMA_CHNL      4
#define DEVCMD2_EDMA_CHNL       5

#define HWIO_EDMA_CHNL          7       // For HWIO edma tests


//
// This section documents the uber vnic id
//
#define UBER_VNICID             0

//
// This section documents the ACL allocations
// Palo contains 8 ACLs
//
#define FIP_ACL_MULTI           0       // M/cast FIP ETYPE frames
#define FIP_ACL_UIF0            1       // FIP ETYPE frames to uif 0
#define FIP_ACL_UIF1            2       // FIP ETYPE frames to uif 1
#define BMC_MGMT_ACL            3       // BMC VLAN4043 frames to MCPU (eg)
#define BMC_NCSI_PROTO_ACL      4       // NC-SI ETYPE frames to MCPU (eg)
#define BMC_MGMT_DROP_ACL       5       // Drop BMC frames if Tx disabled (eg)
#define BMC_NCSI_SLOW_ACL       6       // NC-SI slow-path forwarding (eg)


// Lif resources
#define LIFID_INVALID           0xffff /* 65535 = -1 */

#define MGMTVNIC_LIFID_BASE     (PALO_MAX_LIF - PALO_MAX_HOSTS)
#define MGMTVNIC_LIFID(idx)     (MGMTVNIC_LIFID_BASE + (idx))
#define MGMTMNIC_LIFID_BASE     (MGMTVNIC_LIFID_BASE - PALO_MAX_HOSTS)

#ifdef CONFIG_BASE_CRUZ
#define ECPU4_LIFID             (MGMTMNIC_LIFID_BASE - 1)
#define ECPU3_LIFID             (ECPU4_LIFID - 1)
#define ECPU2_LIFID             (ECPU3_LIFID - 1)
#else
#define ECPU2_LIFID             (MGMTMNIC_LIFID_BASE - 1)
#endif // CONFIG_BASE

#define ECPU1_LIFID             (ECPU2_LIFID - 1)

#define MNIC_INT_LIFID_BASE     (ECPU1_LIFID - MNIC_INT_COUNT)
#define BMC_PATCH_LIFID         (MNIC_INT_LIFID_BASE - 1)
#define MNIC_EXT_LIFID_BASE     0

// For card without Cisco Card NIV mode, use a higher number LIFID for BMC
// Leave enough lower number LIFID for host
#define BMC_LIFID               (BMC_PATCH_LIFID - 1)

// For card with Cisco Card NIV mode, use a lower number LIFID for BMC
// because the switch can't handle higher number VNTAG (LIFID)
#define BMC_LIFID_NIV           (MNIC_EXT_LIFID_BASE + MNIC_EXT_COUNT)

//
// mnic resources
//
#define MNIC_NWQ                1
#define MNIC_NRQ                1
#define MNIC_NCQ                2
#define MNIC_NINTR              1

#define MNIC_VNICID_BASE        1       // skip uber-vnic & bmc slow-path id

#define MNIC_VNICID(idx)        (MNIC_VNICID_BASE + (idx))

#define MNIC_LIFID(idx)     \
        (MNIC_IS_EXT(idx) ? (MNIC_EXT_LIFID_BASE + (idx)) : \
        (MNIC_INT_LIFID_BASE + (idx) - MNIC_EXT_COUNT))

#define MNIC_WQ_BASE(nwqs, idx) ((nwqs) - (MNIC_NWQ * ((idx) + 1)))
// XXX rq: nrqs-1 to avoid rq 1023
#define MNIC_RQ_BASE(nrqs, idx) (((nrqs) - 1) - (MNIC_NRQ * ((idx) + 1)))
#define MNIC_CQ_BASE(ncqs, idx) ((ncqs) - (MNIC_NCQ * ((idx) + 1)))
#define MNIC_INTR_BASE(nintrs, idx) ((nintrs) - (MNIC_NINTR * ((idx) + 1)))

//
// BMC NC-SI Slow-Path resources
//
#define BMC_SLOW_NWQ            1
#define BMC_SLOW_NRQ            1
#define BMC_SLOW_NCQ            1
#define BMC_SLOW_NINTR          1

#define BMC_SLOW_VNICID         (MNIC_VNICID_BASE + MNIC_COUNT)

#define BMC_SLOW_WQ_BASE(nwqs) \
        (MNIC_WQ_BASE(nwqs, (MNIC_COUNT - 1)) - BMC_SLOW_NWQ)
#define BMC_SLOW_RQ_BASE(nrqs) \
        (MNIC_RQ_BASE(nrqs, (MNIC_COUNT - 1)) - BMC_SLOW_NRQ)
#define BMC_SLOW_CQ_BASE(ncqs) \
        (MNIC_CQ_BASE(ncqs, (MNIC_COUNT - 1)) - BMC_SLOW_NCQ)
#define BMC_SLOW_INTR_BASE(nintrs) \
        (MNIC_INTR_BASE(nintrs, (MNIC_COUNT - 1)) - BMC_SLOW_NINTR)

// The devcmd2 CQ

#define DEVCMD2_CQ              (BMC_SLOW_CQ_BASE(palo_conf_cq_count()) - 1)

// PALONIC - nic's created for iSCSI Boot.
#define IS_MNIC_PLATFORM_DEVICE_ID(platform_device_id) \
        ((platform_device_id) < MNIC_COUNT)

// level 3 sources
#define PALONIC_DEVICE_ID_BASE          MNIC_COUNT
#define PALONIC_INT_LVL(palo_nic_id)    3
#define PALONIC_INT_SRC_BASE            16
#define PALONIC_INT_SRC(palo_nic_id)    (PALONIC_INT_SRC_BASE + (palo_nic_id))
#define PALONIC_MAX_COUNT               16
#define PALONIC_PLATFORM_DEVICE_ID(palo_nic_id) \
        (PALONIC_DEVICE_ID_BASE  + (palo_nic_id))

#define IS_PALONIC_PLATFORM_DEVICE_ID(platform_device_id) \
   (((platform_device_id) >= PALONIC_DEVICE_ID_BASE) && \
     (((platform_device_id) - PALONIC_DEVICE_ID_BASE) < PALONIC_INT_MAX_COUNT))

// PALO_ENIC_INT_LVL and PALO_ENIC_INT_SRC macros are used by the enic driver
// kcompat mips.h for irq allocation
#define PALO_ENIC_INT_LVL(device_id) \
        (IS_MNIC_PLATFORM_DEVICE_ID(device_id) ? MNIC_INT_LVL(device_id) : \
                PALONIC_INT_LVL(device_id))
#define PALO_ENIC_INT_SRC(device_id) \
        (IS_MNIC_PLATFORM_DEVICE_ID(device_id) ? MNIC_INT_SRC(device_id) : \
                PALONIC_INT_SRC((device_id) - PALONIC_DEVICE_ID_BASE))
/*
 * This region is to reserve Cruz RW engine related registers
 * user_constX and usr_bufX
 * There are 2 user_const registers user_const0 = 32 bytes
 *                                  user_const1 =  8 bytes
 * There are 4 user_buf registers, user_buf0 through user_buf3
 * per UIF per direction giving a total of 16 registers
 * Shared user_buf registers can be used by all rw rules to store temporary
 * variables for the duration of the rule execution.
 * Exclusive user_bufs should not be used by any other module except the one
 * reserving it as the module will rely on the stored value persisting between
 * subsequent loads of its RW rules
 */
#ifdef CONFIG_BASE_CRUZ
#define RW_USER_CONST_FREE                  0
#define RW_USER_CONST_ROCE_EXCLUSIVE        1

#define RW_USR_BUF_SHARED_RANGE_BEGIN       0
#define RW_USR_BUF_SHARED_RANGE_END         2
#define RW_USR_BUF_ROCE_EXCLUSIVE           3

#define RW_GET_USR_CONST_ADDR(dir, num) \
            (IGCFT_MREGS_CFT_USER_CONST0 + \
            ((dir) * (EGCFT_MREGS_CFT_USER_CONST0 - \
            IGCFT_MREGS_CFT_USER_CONST0)) + ((num) * \
            (IGCFT_MREGS_CFT_USER_CONST1 - IGCFT_MREGS_CFT_USER_CONST0)))
#define RW_GET_USR_BUF_ADDR(dir, uif, num) \
            (IGCFT_MREGS_REWRITE_USER_BUF0_UIF0 + \
            ((dir) * (EGCFT_MREGS_REWRITE_USER_BUF0_UIF0 - \
            IGCFT_MREGS_REWRITE_USER_BUF0_UIF0)) + ((uif) * \
            (IGCFT_MREGS_REWRITE_USER_BUF0_UIF1 - \
            IGCFT_MREGS_REWRITE_USER_BUF0_UIF0)) + ((num) * \
            (IGCFT_MREGS_REWRITE_USER_BUF1_UIF0 - \
            IGCFT_MREGS_REWRITE_USER_BUF0_UIF0)))
#endif /* CONFIG_BASE_CRUZ */
#endif // __ECPU_RES_H__
