
/*
 * Copyright 2013 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: boardidrec.h 21 2016-12-02 21:25:59Z buildsa $"

#ifndef _BOARDIDREC_H_
#define _BOARDIDREC_H_

/*
 * The boardidrec_t structure is a flash-resident record that provides
 * the board id function previously implemented using GPIOs.
 *
 * This structure is used in newer Sereno designs as well as all Cruz and
 * future boards.
 *
 * The structure carries no information beyond a board type enumeration -
 * extending the 3-bit GPIO function in Sereno boards and replacing the use
 * of GPIOs in Cruz boards and beyond.
 *
 * The ID itself comes from the BOARD_CONF_BRD_ enumeration in board_conf.h.
 *
 * This structure is read-only and comes pre-programmed as part of the first
 * article flash image on new boards.
 */

#define BOARDIDREC_MAGIC    0x23647242      // "Brd#"

#ifndef __ASSEMBLY__

typedef struct {
    u_int32_t   magic;
    u_int32_t   brdid;                      // enum taken from board_conf.h
    u_int32_t   sum;                        // checksum
} boardidrec_t;

static inline void
boardidrec_init(boardidrec_t *p, u_int32_t brdid)
{
    p->magic = BOARDIDREC_MAGIC;
    p->brdid = brdid;
    p->sum = 0 - p->magic - brdid;
}

static inline int
boardidrec_isvalid(const boardidrec_t *p)
{
    return ((p->magic == BOARDIDREC_MAGIC) &&
            (p->magic + p->brdid + p->sum) == 0);
}

#else

#define BOARDIDREC_MAGIC_OFFSET     0
#define BOARDIDREC_ID_OFFSET        4
#define BOARDIDREC_SUM_OFFSET       8

#endif /* __ASSEMBLY__ */

#endif
