
/*
 * Copyright (c) 2007 Nuova Systems, Inc.  All rights reserved.
 */
#ident "$Id: paloflash.h 30 2016-12-02 23:05:00Z buildsa $"

#ifndef _PALOFLASH_H_
#define _PALOFLASH_H_

/*
 * Flash named partitions.
 */
#define PALOFLASH_BOARDID_NAME      "BoardID"
#define PALOFLASH_ENV_NAME          "Env"
#define PALOFLASH_FWMAP_NAME        "FwMap"
#define PALOFLASH_DIAG_NAME         "Diag"
#define PALOFLASH_BOOT_NAME         "Boot"
#define PALOFLASH_FIRMWARE_NAME     "Firmware"
#define PALOFLASH_CONFIG_NAME       "Config"
#define PALOFLASH_OBFL_NAME         "OBFL"
#define PALOFLASH_KDUMP_NAME        "KDump"



/*
 * Only U-Boot and the kernel know about the physical mapping and partitions.
 * All userland code uses /dev/mtd (except for very specialized apps).
 */
#if defined(__KERNEL__) || defined(PALO_LOWLEVEL_FLASH_APP)

/*
 * Flash layout for Palo cards using the P30 Flash.
 *
 * 01ff8000     32K  U-Boot Environment (32K)
 * 01ff0000     32K  Board ID
 * 01fe0000 2 x 32K  Firmware Map
 * 01c40000  3.625M  Manufacturing Diags
 * 01c00000    256K  U-Boot
 * 00800000     20M  Firmware
 * 00400000      4M  Config
 * 00100000      3M  OBFL
 * 00000000      1M  KDump
 */
#define PALOFLASH_PALO_P30_ENV_OFFSET       0x01ff8000
#define PALOFLASH_PALO_P30_ENV_SIZE         0x00008000

#define PALOFLASH_PALO_P30_BOARDID_OFFSET   0x01ff0000
#define PALOFLASH_PALO_P30_BOARDID_SIZE     0x00008000

#define PALOFLASH_PALO_P30_FWMAP_OFFSET     0x01fe0000
#define PALOFLASH_PALO_P30_FWMAP_SIZE       0x00010000

#define PALOFLASH_PALO_P30_DIAG_OFFSET      0x01c40000
#define PALOFLASH_PALO_P30_DIAG_SIZE        0x003a0000

#define PALOFLASH_PALO_P30_BOOT_OFFSET      0x01c00000
#define PALOFLASH_PALO_P30_BOOT_SIZE        0x00040000

#define PALOFLASH_PALO_P30_FIRMWARE_OFFSET  0x00800000
#define PALOFLASH_PALO_P30_FIRMWARE_SIZE    0x01400000

#define PALOFLASH_PALO_P30_CONFIG_OFFSET    0x00400000
#define PALOFLASH_PALO_P30_CONFIG_SIZE      0x00400000

#define PALOFLASH_PALO_P30_OBFL_OFFSET      0x00100000
#define PALOFLASH_PALO_P30_OBFL_SIZE        0x00300000

#define PALOFLASH_PALO_P30_KDUMP_OFFSET     0x00000000
#define PALOFLASH_PALO_P30_KDUMP_SIZE       0x00100000

/*
 * Flash layout for Palo cards using the M29 Flash.
 *
 * 01fe0000     128K - Board ID
 * 01fc0000     128K - U-Boot Environment (32K used)
 * 01f80000 2 x 128K - Firmware Map
 * 01c80000       3M - Config
 * 01c40000     256K - KDump
 * 01c00000     256K - U-Boot
 * 01400000       8M - Manufacturing Diags
 * 00400000      16M - Firmware
 * 00000000       4M - OBFL
 */
#define PALOFLASH_PALO_PHYS                 0x1e000000
#define PALOFLASH_PALO_SIZE                 0x02000000  /* 32MB */

#define PALOFLASH_PALO_M29_BOARDID_OFFSET   0x01fe0000
#define PALOFLASH_PALO_M29_BOARDID_SIZE     0x00020000

#define PALOFLASH_PALO_M29_ENV_OFFSET       0x01fc0000
#define PALOFLASH_PALO_M29_ENV_SIZE         0x00020000

#define PALOFLASH_PALO_M29_FWMAP_OFFSET     0x01f80000
#define PALOFLASH_PALO_M29_FWMAP_SIZE       0x00040000

#define PALOFLASH_PALO_M29_CONFIG_OFFSET    0x01c80000
#define PALOFLASH_PALO_M29_CONFIG_SIZE      0x00300000

#define PALOFLASH_PALO_M29_KDUMP_OFFSET     0x01c40000
#define PALOFLASH_PALO_M29_KDUMP_SIZE       0x00040000

#define PALOFLASH_PALO_M29_BOOT_OFFSET      0x01c00000
#define PALOFLASH_PALO_M29_BOOT_SIZE        0x00040000

#define PALOFLASH_PALO_M29_DIAG_OFFSET      0x01400000
#define PALOFLASH_PALO_M29_DIAG_SIZE        0x00800000

#define PALOFLASH_PALO_M29_FIRMWARE_OFFSET  0x00400000
#define PALOFLASH_PALO_M29_FIRMWARE_SIZE    0x01000000

#define PALOFLASH_PALO_M29_OBFL_OFFSET      0x00000000
#define PALOFLASH_PALO_M29_OBFL_SIZE        0x00400000




/*
 * Flash layout for Sereno cards using either P30 or M29 Flash.
 *
 * 01fe0000     128K - Board ID
 * 01fc0000     128K - U-Boot Environment
 * 01f80000 2 x 128K - Firmware Map
 * 01c80000       3M - Config
 * 01c40000     256K - KDump
 * 01c00000     256K - U-Boot
 * 01400000       8M - Manufacturing Diags
 * 00400000      16M - Firmware
 * 00000000       4M - OBFL
 */
#define PALOFLASH_SERENO_PHYS               0x1e000000
#define PALOFLASH_SERENO_SIZE               0x02000000  /* 32MB */

#define PALOFLASH_SERENO_BOARDID_OFFSET     0x01fe0000
#define PALOFLASH_SERENO_BOARDID_SIZE       0x00020000

#define PALOFLASH_SERENO_ENV_OFFSET         0x01fc0000
#define PALOFLASH_SERENO_ENV_SIZE           0x00020000

#define PALOFLASH_SERENO_FWMAP_OFFSET       0x01f80000
#define PALOFLASH_SERENO_FWMAP_SIZE         0x00040000

#define PALOFLASH_SERENO_CONFIG_OFFSET      0x01c80000
#define PALOFLASH_SERENO_CONFIG_SIZE        0x00300000

#define PALOFLASH_SERENO_KDUMP_OFFSET       0x01c40000
#define PALOFLASH_SERENO_KDUMP_SIZE         0x00040000

#define PALOFLASH_SERENO_BOOT_OFFSET        0x01c00000
#define PALOFLASH_SERENO_BOOT_SIZE          0x00040000

#define PALOFLASH_SERENO_DIAG_OFFSET        0x01400000
#define PALOFLASH_SERENO_DIAG_SIZE          0x00800000

#define PALOFLASH_SERENO_FIRMWARE_OFFSET    0x00400000
#define PALOFLASH_SERENO_FIRMWARE_SIZE      0x01000000

#define PALOFLASH_SERENO_OBFL_OFFSET        0x00000000
#define PALOFLASH_SERENO_OBFL_SIZE          0x00400000



/*
 * Flash layout for Cruz cards using either P30 or M29 Flash.
 *
 * 07fe0000     128K - Board ID
 * 07fc0000     128K - U-Boot Environment
 * 07f80000 2 x 128K - Firmware Map
 * 07c80000       3M - (unused)
 * 07c40000     256K - KDump
 * 07c00000     256K - U-Boot
 * 06e00000      14M - Config
 * 06000000      14M - OBFL
 * 04000000      32M - Manufacturing Diags
 * 00000000      64M - Firmware
 */
#define PALOFLASH_CRUZ_PHYS                 0x18000000
#define PALOFLASH_CRUZ_SIZE                 0x08000000  /* 128MB */

#define PALOFLASH_CRUZ_BOARDID_OFFSET       0x07fe0000
#define PALOFLASH_CRUZ_BOARDID_SIZE         0x00020000

#define PALOFLASH_CRUZ_ENV_OFFSET           0x07fc0000
#define PALOFLASH_CRUZ_ENV_SIZE             0x00020000

#define PALOFLASH_CRUZ_FWMAP_OFFSET         0x07f80000
#define PALOFLASH_CRUZ_FWMAP_SIZE           0x00040000

#define PALOFLASH_CRUZ_KDUMP_OFFSET         0x07c40000
#define PALOFLASH_CRUZ_KDUMP_SIZE           0x00040000

#define PALOFLASH_CRUZ_BOOT_OFFSET          0x07c00000
#define PALOFLASH_CRUZ_BOOT_SIZE            0x00040000

#define PALOFLASH_CRUZ_CONFIG_OFFSET        0x06e00000
#define PALOFLASH_CRUZ_CONFIG_SIZE          0x00e00000

#define PALOFLASH_CRUZ_OBFL_OFFSET          0x06000000
#define PALOFLASH_CRUZ_OBFL_SIZE            0x00e00000

#define PALOFLASH_CRUZ_DIAG_OFFSET          0x04000000
#define PALOFLASH_CRUZ_DIAG_SIZE            0x02000000

#define PALOFLASH_CRUZ_FIRMWARE_OFFSET      0x00000000
#define PALOFLASH_CRUZ_FIRMWARE_SIZE        0x04000000



/*
 * M29 vs. M30 convenience defines.
 */
#if defined(CONFIG_BASE_PALO)

#define PALOFLASH_PHYS                  PALOFLASH_PALO_PHYS
#define PALOFLASH_SIZE                  PALOFLASH_PALO_SIZE

#define PALOFLASH_P30_BOARDID_OFFSET    PALOFLASH_PALO_P30_BOARDID_OFFSET
#define PALOFLASH_P30_BOARDID_SIZE      PALOFLASH_PALO_P30_BOARDID_SIZE

#define PALOFLASH_P30_ENV_OFFSET        PALOFLASH_PALO_P30_ENV_OFFSET
#define PALOFLASH_P30_ENV_SIZE          PALOFLASH_PALO_P30_ENV_SIZE

#define PALOFLASH_P30_FWMAP_OFFSET      PALOFLASH_PALO_P30_FWMAP_OFFSET
#define PALOFLASH_P30_FWMAP_SIZE        PALOFLASH_PALO_P30_FWMAP_SIZE

#define PALOFLASH_P30_CONFIG_OFFSET     PALOFLASH_PALO_P30_CONFIG_OFFSET
#define PALOFLASH_P30_CONFIG_SIZE       PALOFLASH_PALO_P30_CONFIG_SIZE

#define PALOFLASH_P30_KDUMP_OFFSET      PALOFLASH_PALO_P30_KDUMP_OFFSET
#define PALOFLASH_P30_KDUMP_SIZE        PALOFLASH_PALO_P30_KDUMP_SIZE

#define PALOFLASH_P30_BOOT_OFFSET       PALOFLASH_PALO_P30_BOOT_OFFSET
#define PALOFLASH_P30_BOOT_SIZE         PALOFLASH_PALO_P30_BOOT_SIZE

#define PALOFLASH_P30_DIAG_OFFSET       PALOFLASH_PALO_P30_DIAG_OFFSET
#define PALOFLASH_P30_DIAG_SIZE         PALOFLASH_PALO_P30_DIAG_SIZE

#define PALOFLASH_P30_FIRMWARE_OFFSET   PALOFLASH_PALO_P30_FIRMWARE_OFFSET
#define PALOFLASH_P30_FIRMWARE_SIZE     PALOFLASH_PALO_P30_FIRMWARE_SIZE

#define PALOFLASH_P30_OBFL_OFFSET       PALOFLASH_PALO_P30_OBFL_OFFSET
#define PALOFLASH_P30_OBFL_SIZE         PALOFLASH_PALO_P30_OBFL_SIZE

#define PALOFLASH_M29_BOARDID_OFFSET    PALOFLASH_PALO_M29_BOARDID_OFFSET
#define PALOFLASH_M29_BOARDID_SIZE      PALOFLASH_PALO_M29_BOARDID_SIZE

#define PALOFLASH_M29_ENV_OFFSET        PALOFLASH_PALO_M29_ENV_OFFSET
#define PALOFLASH_M29_ENV_SIZE          PALOFLASH_PALO_M29_ENV_SIZE

#define PALOFLASH_M29_FWMAP_OFFSET      PALOFLASH_PALO_M29_FWMAP_OFFSET
#define PALOFLASH_M29_FWMAP_SIZE        PALOFLASH_PALO_M29_FWMAP_SIZE

#define PALOFLASH_M29_CONFIG_OFFSET     PALOFLASH_PALO_M29_CONFIG_OFFSET
#define PALOFLASH_M29_CONFIG_SIZE       PALOFLASH_PALO_M29_CONFIG_SIZE

#define PALOFLASH_M29_KDUMP_OFFSET      PALOFLASH_PALO_M29_KDUMP_OFFSET
#define PALOFLASH_M29_KDUMP_SIZE        PALOFLASH_PALO_M29_KDUMP_SIZE

#define PALOFLASH_M29_BOOT_OFFSET       PALOFLASH_PALO_M29_BOOT_OFFSET
#define PALOFLASH_M29_BOOT_SIZE         PALOFLASH_PALO_M29_BOOT_SIZE

#define PALOFLASH_M29_DIAG_OFFSET       PALOFLASH_PALO_M29_DIAG_OFFSET
#define PALOFLASH_M29_DIAG_SIZE         PALOFLASH_PALO_M29_DIAG_SIZE

#define PALOFLASH_M29_FIRMWARE_OFFSET   PALOFLASH_PALO_M29_FIRMWARE_OFFSET
#define PALOFLASH_M29_FIRMWARE_SIZE     PALOFLASH_PALO_M29_FIRMWARE_SIZE

#define PALOFLASH_M29_OBFL_OFFSET       PALOFLASH_PALO_M29_OBFL_OFFSET
#define PALOFLASH_M29_OBFL_SIZE         PALOFLASH_PALO_M29_OBFL_SIZE

#elif defined(CONFIG_BASE_SERENO)

#define PALOFLASH_PHYS                  PALOFLASH_SERENO_PHYS
#define PALOFLASH_SIZE                  PALOFLASH_SERENO_SIZE

#define PALOFLASH_P30_BOARDID_OFFSET    PALOFLASH_SERENO_BOARDID_OFFSET
#define PALOFLASH_P30_BOARDID_SIZE      PALOFLASH_SERENO_BOARDID_SIZE

#define PALOFLASH_P30_ENV_OFFSET        PALOFLASH_SERENO_ENV_OFFSET
#define PALOFLASH_P30_ENV_SIZE          PALOFLASH_SERENO_ENV_SIZE

#define PALOFLASH_P30_FWMAP_OFFSET      PALOFLASH_SERENO_FWMAP_OFFSET
#define PALOFLASH_P30_FWMAP_SIZE        PALOFLASH_SERENO_FWMAP_SIZE

#define PALOFLASH_P30_CONFIG_OFFSET     PALOFLASH_SERENO_CONFIG_OFFSET
#define PALOFLASH_P30_CONFIG_SIZE       PALOFLASH_SERENO_CONFIG_SIZE

#define PALOFLASH_P30_KDUMP_OFFSET      PALOFLASH_SERENO_KDUMP_OFFSET
#define PALOFLASH_P30_KDUMP_SIZE        PALOFLASH_SERENO_KDUMP_SIZE

#define PALOFLASH_P30_BOOT_OFFSET       PALOFLASH_SERENO_BOOT_OFFSET
#define PALOFLASH_P30_BOOT_SIZE         PALOFLASH_SERENO_BOOT_SIZE

#define PALOFLASH_P30_DIAG_OFFSET       PALOFLASH_SERENO_DIAG_OFFSET
#define PALOFLASH_P30_DIAG_SIZE         PALOFLASH_SERENO_DIAG_SIZE

#define PALOFLASH_P30_FIRMWARE_OFFSET   PALOFLASH_SERENO_FIRMWARE_OFFSET
#define PALOFLASH_P30_FIRMWARE_SIZE     PALOFLASH_SERENO_FIRMWARE_SIZE

#define PALOFLASH_P30_OBFL_OFFSET       PALOFLASH_SERENO_OBFL_OFFSET
#define PALOFLASH_P30_OBFL_SIZE         PALOFLASH_SERENO_OBFL_SIZE

#define PALOFLASH_M29_BOARDID_OFFSET    PALOFLASH_SERENO_BOARDID_OFFSET
#define PALOFLASH_M29_BOARDID_SIZE      PALOFLASH_SERENO_BOARDID_SIZE

#define PALOFLASH_M29_ENV_OFFSET        PALOFLASH_SERENO_ENV_OFFSET
#define PALOFLASH_M29_ENV_SIZE          PALOFLASH_SERENO_ENV_SIZE

#define PALOFLASH_M29_FWMAP_OFFSET      PALOFLASH_SERENO_FWMAP_OFFSET
#define PALOFLASH_M29_FWMAP_SIZE        PALOFLASH_SERENO_FWMAP_SIZE

#define PALOFLASH_M29_CONFIG_OFFSET     PALOFLASH_SERENO_CONFIG_OFFSET
#define PALOFLASH_M29_CONFIG_SIZE       PALOFLASH_SERENO_CONFIG_SIZE

#define PALOFLASH_M29_KDUMP_OFFSET      PALOFLASH_SERENO_KDUMP_OFFSET
#define PALOFLASH_M29_KDUMP_SIZE        PALOFLASH_SERENO_KDUMP_SIZE

#define PALOFLASH_M29_BOOT_OFFSET       PALOFLASH_SERENO_BOOT_OFFSET
#define PALOFLASH_M29_BOOT_SIZE         PALOFLASH_SERENO_BOOT_SIZE

#define PALOFLASH_M29_DIAG_OFFSET       PALOFLASH_SERENO_DIAG_OFFSET
#define PALOFLASH_M29_DIAG_SIZE         PALOFLASH_SERENO_DIAG_SIZE

#define PALOFLASH_M29_FIRMWARE_OFFSET   PALOFLASH_SERENO_FIRMWARE_OFFSET
#define PALOFLASH_M29_FIRMWARE_SIZE     PALOFLASH_SERENO_FIRMWARE_SIZE

#define PALOFLASH_M29_OBFL_OFFSET       PALOFLASH_SERENO_OBFL_OFFSET
#define PALOFLASH_M29_OBFL_SIZE         PALOFLASH_SERENO_OBFL_SIZE

#elif defined(CONFIG_BASE_CRUZ)

#define PALOFLASH_PHYS                  PALOFLASH_CRUZ_PHYS
#define PALOFLASH_SIZE                  PALOFLASH_CRUZ_SIZE

#define PALOFLASH_P30_BOARDID_OFFSET    PALOFLASH_CRUZ_BOARDID_OFFSET
#define PALOFLASH_P30_BOARDID_SIZE      PALOFLASH_CRUZ_BOARDID_SIZE

#define PALOFLASH_P30_ENV_OFFSET        PALOFLASH_CRUZ_ENV_OFFSET
#define PALOFLASH_P30_ENV_SIZE          PALOFLASH_CRUZ_ENV_SIZE

#define PALOFLASH_P30_FWMAP_OFFSET      PALOFLASH_CRUZ_FWMAP_OFFSET
#define PALOFLASH_P30_FWMAP_SIZE        PALOFLASH_CRUZ_FWMAP_SIZE

#define PALOFLASH_P30_CONFIG_OFFSET     PALOFLASH_CRUZ_CONFIG_OFFSET
#define PALOFLASH_P30_CONFIG_SIZE       PALOFLASH_CRUZ_CONFIG_SIZE

#define PALOFLASH_P30_KDUMP_OFFSET      PALOFLASH_CRUZ_KDUMP_OFFSET
#define PALOFLASH_P30_KDUMP_SIZE        PALOFLASH_CRUZ_KDUMP_SIZE

#define PALOFLASH_P30_BOOT_OFFSET       PALOFLASH_CRUZ_BOOT_OFFSET
#define PALOFLASH_P30_BOOT_SIZE         PALOFLASH_CRUZ_BOOT_SIZE

#define PALOFLASH_P30_DIAG_OFFSET       PALOFLASH_CRUZ_DIAG_OFFSET
#define PALOFLASH_P30_DIAG_SIZE         PALOFLASH_CRUZ_DIAG_SIZE

#define PALOFLASH_P30_FIRMWARE_OFFSET   PALOFLASH_CRUZ_FIRMWARE_OFFSET
#define PALOFLASH_P30_FIRMWARE_SIZE     PALOFLASH_CRUZ_FIRMWARE_SIZE

#define PALOFLASH_P30_OBFL_OFFSET       PALOFLASH_CRUZ_OBFL_OFFSET
#define PALOFLASH_P30_OBFL_SIZE         PALOFLASH_CRUZ_OBFL_SIZE

#define PALOFLASH_M29_BOARDID_OFFSET    PALOFLASH_CRUZ_BOARDID_OFFSET
#define PALOFLASH_M29_BOARDID_SIZE      PALOFLASH_CRUZ_BOARDID_SIZE

#define PALOFLASH_M29_ENV_OFFSET        PALOFLASH_CRUZ_ENV_OFFSET
#define PALOFLASH_M29_ENV_SIZE          PALOFLASH_CRUZ_ENV_SIZE

#define PALOFLASH_M29_FWMAP_OFFSET      PALOFLASH_CRUZ_FWMAP_OFFSET
#define PALOFLASH_M29_FWMAP_SIZE        PALOFLASH_CRUZ_FWMAP_SIZE

#define PALOFLASH_M29_CONFIG_OFFSET     PALOFLASH_CRUZ_CONFIG_OFFSET
#define PALOFLASH_M29_CONFIG_SIZE       PALOFLASH_CRUZ_CONFIG_SIZE

#define PALOFLASH_M29_KDUMP_OFFSET      PALOFLASH_CRUZ_KDUMP_OFFSET
#define PALOFLASH_M29_KDUMP_SIZE        PALOFLASH_CRUZ_KDUMP_SIZE

#define PALOFLASH_M29_BOOT_OFFSET       PALOFLASH_CRUZ_BOOT_OFFSET
#define PALOFLASH_M29_BOOT_SIZE         PALOFLASH_CRUZ_BOOT_SIZE

#define PALOFLASH_M29_DIAG_OFFSET       PALOFLASH_CRUZ_DIAG_OFFSET
#define PALOFLASH_M29_DIAG_SIZE         PALOFLASH_CRUZ_DIAG_SIZE

#define PALOFLASH_M29_FIRMWARE_OFFSET   PALOFLASH_CRUZ_FIRMWARE_OFFSET
#define PALOFLASH_M29_FIRMWARE_SIZE     PALOFLASH_CRUZ_FIRMWARE_SIZE

#define PALOFLASH_M29_OBFL_OFFSET       PALOFLASH_CRUZ_OBFL_OFFSET
#define PALOFLASH_M29_OBFL_SIZE         PALOFLASH_CRUZ_OBFL_SIZE

#endif /* CONFIG_BASE */

#endif /* __KERNEL__ */

#endif
