
/*
 * Copyright (c) 2007 Nuova Systems, Inc.  All rights reserved.
 */
#ident "$Id:$"

#ifndef _PALOFWMAP_H_
#define _PALOFWMAP_H_

/*
 * The PALO firwmare update process can be interrupted at any time by a
 * host reset so we maintain a log structured directory across two
 * flash sectors to record the available firmware, and order update operations
 * such that an unexpected reset never results in losing active firmware.
 *
 * Three data types are defined here:
 *  fwmap_t
 *      The firmware map holds the directory log.  It exists on two sectors -
 *      allowing one to be erased without losing the directory.  The firmware
 *      map with the highest generation id is the active one.
 *
 *  fwdir_t
 *      The firmware map holds versions of a directory.  The highest index
 *      valid directory is the latest one.
 *
 *  fwdirent_t
 *      A directory contains a number of firmware records.  The first record
 *      is the desired active firmware.  The records following it are backups.
 *      PALO is expected to have one active and one backup firmware.
 */

#define PALOFW_DIRENTMAGIC      0x43ef2311
typedef struct {
    u_int32_t   magic;                      // magic
    u_int32_t   addr;                       // firmware address
    u_int32_t   naddr;                      // inverse of firmware address
} fwdirent_t;

#define PALOFW_DIRMAGIC         0xfe912acc
#define PALOFW_NDIRENTS         8
typedef struct {
    u_int32_t   magic;                      // magic
    fwdirent_t  dirent[PALOFW_NDIRENTS];    // directory entries
} fwdir_t;

#define PALOFW_MAPMAGIC         0x54dac411
#define PALOFW_NDIRS            32
typedef struct {
    u_int32_t   magic;                      // magic
    u_int32_t   generation;                 // map generation id
    fwdir_t     dir[PALOFW_NDIRS];          // directories
} fwmap_t;

#endif

