/*
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: board_conf.c 46 2016-12-03 23:17:24Z buildsa $"

#ifdef __UBOOT__
#include <common.h>
#include <command.h>
#include <asm/addrspace.h>
#include <asm/io.h>
#include <palo/include/misc_mregs.h>
#include <palo/include/misc_regs_defs.h>
#include <palo/include/palo_gpio.h>
#include <palo/include/pci_ids.h>
#include <palo/include/palomem.h>
#include <palo/fw/mcpu/include/palofwimg.h>
#include <palo/fw/mcpu/include/paloflash.h>
#include <palo/fw/mcpu/include/board_conf.h>
#include <palo/fw/mcpu/include/boardidrec.h>
#include <palo/include/synopsis/dw_gpio.h>
#include <common/libfru/include/ipmi_sprom.h>
#include <common/nictool/include/nictool_adapter_defs.h>
#include <palo/include/palo_defs.h>
#include <palo/fw/common/include/ecpu_res.h>
#include "palodtls.h"
#include "libgpio.h"
#else
#ifdef __KERNEL__
#include <linux/kernel.h>
#include <asm/io.h>
#include <palo.h>
#include <paloint.h>
#include <palo/include/palo_impl.h>
#include <palo/include/misc_mregs.h>
#include <palo/include/misc_regs_defs.h>
#include <palo/include/palo_gpio.h>
#include <palo/include/pci_ids.h>
#include <palo/fw/mcpu/include/palofwimg.h>
#include <palo/fw/mcpu/include/paloflash.h>
#include <palo/fw/mcpu/include/board_conf.h>
#include <palo/fw/mcpu/include/boardidrec.h>
#include <palo/include/synopsis/dw_gpio.h>
#include <common/libfru/include/ipmi_sprom.h>
#include <common/nictool/include/nictool_adapter_defs.h>
#include <palo/include/palo_defs.h>
#include <palo/include/palomem.h>
#include <palo/fw/common/include/ecpu_res.h>
#include "libgpio.h"
#include "regio.h"
#else // __KERNEL__
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <fcntl.h>
#include "pci_ids.h"
#include "misc_mregs.h"
#include "misc_regs_defs.h"
#include "regio.h"
#include "palo_gpio.h"
#include "palofwimg.h"
#include "palomem.h"
#include "boardidrec.h"
#include "ipmi_sprom.h"
#include "ipmi_sprom_ops.h"
#include "libgpio.h"
#include "nictool_adapter_defs.h"
#include "nictool_palo_defs.h"
#include "board_conf.h"
#include "palo_conf.h"
#include "maclib.h"
#include "fruid.h"
#include "palo_defs.h"
#include "ecpu_res.h"
#include "palomtd.h"
#endif // __KERNEL__
#endif // __UBOOT__

#ifndef NMEL
#define NMEL(x)         (sizeof (x) / sizeof ((x)[0]))
#endif

struct __board_conf_info __board_conf_info;

#if !defined(__KERNEL__) && !defined(__UBOOT__)
// Only those who care about these fru functions must link with libfru
// to override these weak versions
int ipmi_sprom_mezz_fill(sprom_ipmi_mezz_t *sprom, int card_type)
    __attribute__((weak));
int ipmi_sprom_mezz_fill(sprom_ipmi_mezz_t *sprom, int card_type)
{ return (0); }
void ipmi_sprom_mezz_dump(sprom_ipmi_mezz_t *sprom, int card_type)
    __attribute__((weak));
void ipmi_sprom_mezz_dump (sprom_ipmi_mezz_t *sprom, int card_type) {}
int ipmi_sprom_mezz_dflt(sprom_ipmi_mezz_t *sprom, uint8_t mezz_type)
    __attribute__((weak));
int ipmi_sprom_mezz_dflt(sprom_ipmi_mezz_t *sprom, uint8_t mezz_type)
{ return (0); }

int ipmi_sprom_mpark_fill(sprom_ipmi_mpark_t *sprom, int card_type)
    __attribute__((weak));
int ipmi_sprom_mpark_fill(sprom_ipmi_mpark_t *sprom, int card_type)
{ return (0); }
void ipmi_sprom_mpark_dump(sprom_ipmi_mpark_t *sprom, int card_type)
    __attribute__((weak));
void ipmi_sprom_mpark_dump(sprom_ipmi_mpark_t *sprom, int card_type) {}
int ipmi_sprom_mpark_dflt(sprom_ipmi_mpark_t *sprom, uint8_t mezz_type)
    __attribute__((weak));
int ipmi_sprom_mpark_dflt(sprom_ipmi_mpark_t *sprom, uint8_t mezz_type)
{ return (0); }

int ipmi_header_checksum_verify(uint8_t *buf) __attribute__((weak));
int ipmi_header_checksum_verify(uint8_t *buf) { return (0); }
int ipmi_zero_checksum_verify(uint8_t *buf, int buflen) __attribute__((weak));
int ipmi_zero_checksum_verify(uint8_t *buf, int buflen) { return (0); }
int ipmi_multi_record_checksum_verify(ipmi_sprom_multi_record_header_t *mr)
    __attribute__((weak));
int ipmi_multi_record_checksum_verify(ipmi_sprom_multi_record_header_t *mr)
{ return (0); }
int fruid_get_hw_part_number(char *buf, const size_t bufsz)
    __attribute__((weak));
int fruid_get_hw_part_number(char *buf, const size_t bufsz)
{ return (-1); }
#endif // !__KERNEL__ && !__UBOOT__

#ifdef __KERNEL__
#if defined(CONFIG_BASE_SERENO) || defined(CONFIG_BASE_CRUZ)
static void
set_gpio_reg(u_int32_t reg_addr, int val, int pin)
{
     int x;
     if (val) {
         x = readreg(reg_addr) | (1 << pin);
     } else {
         x = readreg(reg_addr) & ~((u_int32_t)(1 << pin));
     }
     writereg(reg_addr, x);
}
#endif // SERENO
#endif // KERNEL

/*
 * Number of retimers per adapter based on type
 */
#define VASONA_MAX_RETIMER      2
#define COTATI_MAX_RETIMER      1
#define LEXINGTON_MAX_RETIMER   1
#define ICEHOUSE_MAX_RETIMER    2
#define SUSANVILLE_MAX_RETIMER  1
#define OAKLAND_MAX_RETIMER     1
#define WALNUTCREEK_MAX_RETIMER 1

#if !defined(__KERNEL__) && !defined(__UBOOT__)
u_int8_t
is_retimer_present(void)
{
#define VASONA_R_HW_PN_PREFIX  "73-14628"
#define COTATI_R_HW_PN_PREFIX  "73-14641"

    u_int8_t retimer = 0;
    int      r;
    char     hw_pn[32];

    /*
     * Could not just check "has_retimer" flag because for Vasona/Cotati
     * we need to decide it based on the part-number
     */
    if (board_conf_is_lexington() || board_conf_is_icehouse() ||
        board_conf_is_oakland() || board_conf_is_walnutcreek() ||
        board_conf_is_susanville()) {
        return (1);
    }

    r = fruid_get_hw_part_number(hw_pn, sizeof (hw_pn));
    if (r < 0) {
        return (0);
    }

    if (strstr(hw_pn, VASONA_R_HW_PN_PREFIX) != NULL) {
        retimer = 1;
    } else if (strstr(hw_pn, COTATI_R_HW_PN_PREFIX) != NULL) {
        retimer = 1;
    } else {
        retimer = 0;
    }

    return (retimer);

}
#endif // !__KERNEL__ && !__UBOOT__

#if !defined CONFIG_BASE_PALO
static int
read_boardidrec(boardidrec_t *p)
{
#if defined(__UBOOT__) || defined(__KERNEL__)
    /*
     * Direct mapped through KSEG1
     */
    memcpy(p,
            (void *)KSEG1ADDR(PALOFLASH_PHYS + PALOFLASH_M29_BOARDID_OFFSET),
            sizeof (*p));
    return (0);
#else
    /*
     * Read via mtd
     */
    palo_mtd_ctx_t *ctx;
    int ret;

    ctx = palo_mtd_open(PALO_MTD_BOARDID, O_RDONLY);
    if (ctx == NULL) {
        return (-1);
    }
    ret = palo_mtd_read(ctx, 0, p, sizeof (*p));
    palo_mtd_close(ctx);
    return (ret);
#endif
}

static int
get_flash_boardid(void)
{
    boardidrec_t rec;

    if (read_boardidrec(&rec) < 0) {
        return (-1);
    }
    if (!boardidrec_isvalid(&rec)) {
        return (-1);
    }
    return ((int)rec.brdid);
}
#endif /* !CONFIG_BASE_PALO */

static int
palo_common_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    u_int32_t val;
    int ret = 0;
#if !defined(__KERNEL__) && !defined(__UBOOT__)
    static int has_retimer_valid;
#endif

    switch (cmd) {
#ifdef __KERNEL__
    case BOARDCMD_INIT:
      {
        u_int32_t init_ddr, init_val;
        int pin, val;

#ifdef __UBOOT__
        init_ddr = bi->gpio_boot_ddr;
        init_val = bi->gpio_boot_val;
#else
        init_ddr = bi->gpio_init_ddr;
        init_val = bi->gpio_init_val;
#endif
        for (pin = 0; pin < 16; pin++) {
            if (init_ddr & (1 << pin)) {
                val = (init_val >> pin) & 0x1;
                gpio_output(pin, val);
                gpio_setdirection(pin, GPIO_DIR_OUTPUT);
            } else {
                gpio_setdirection(pin, GPIO_DIR_INPUT);
            }
        }
        break;
      }
#endif
    case BOARDCMD_IS_LOWPOWER: {
        *(int *)arg = bi->is_lowpower;
        break;
    }

    case BOARDCMD_CISCO_CARD_MODE: {
        *(int *)arg = bi->cisco_card_mode;
        break;
    }

    case BOARDCMD_GET_CARD_ADDRESS:
        if (!bi->card_address_valid) {
            bi->card_address = gpio_input(bi->card_address_pin);
            bi->card_address_valid = 1;
        }
        *(int *)arg = bi->card_address;
        break;

    case BOARDCMD_SET_STATUSLED:
        gpio_output(bi->statusled_pin, *(int *)arg);
        break;

    case BOARDCMD_SET_FAULTLED:
        gpio_output(bi->faultled_pin, *(int *)arg);
        break;

    case BOARDCMD_SET_PHYRESET:
        gpio_output(bi->phyreset_pin, *(int *)arg);
        break;

    case BOARDCMD_BEGIN_I2C0RESET:
        gpio_output(bi->i2c0reset_pin, 1);
        gpio_setdirection(bi->i2c0reset_pin, GPIO_DIR_OUTPUT);
        break;

    case BOARDCMD_SET_I2C0RESET:
        gpio_output(bi->i2c0reset_pin, *(int *)arg);
        break;

    case BOARDCMD_END_I2C0RESET:
        gpio_output(bi->i2c0reset_pin, 1);
        gpio_setdirection(bi->i2c0reset_pin, GPIO_DIR_INPUT);
        break;

    case BOARDCMD_GET_MFGDIAG:
        *(int *)arg = gpio_input(bi->mfgdiag_pin);
        break;

    case BOARDCMD_ARM_FLASHRESET:
        val = readreg(MISC_MREGS_CFG_PIN_SPARE);
        val &= ~bi->resetarm_mask;
        val |= bi->resetarm_val;
        writereg(MISC_MREGS_CFG_PIN_SPARE, val);
        break;

#if !defined(__KERNEL__) && !defined(__UBOOT__)
    case BOARDCMD_FRU_DATASIZE:
        *(int *)arg = bi->fru_datasize;
        break;

    case BOARDCMD_FRU_INIT: {
        printf("%s FRUID INIT\n", bi->brd_type_name);
        printf("-----------------\n");
        if (board_conf_is_pcie_card()) {
            sprom_ipmi_mpark_t *sprom = *(sprom_ipmi_mpark_t **)arg;

            ipmi_sprom_mpark_dflt(sprom, bi->fru_cardtype);
            ipmi_sprom_mpark_fill(sprom, bi->fru_cardtype);
            ipmi_sprom_mpark_dump(sprom, bi->fru_cardtype);
        } else {
            sprom_ipmi_mezz_t *sprom = *(sprom_ipmi_mezz_t **)arg;

            ipmi_sprom_mezz_dflt(sprom, bi->fru_cardtype);
            ipmi_sprom_mezz_fill(sprom, bi->fru_cardtype);
            ipmi_sprom_mezz_dump(sprom, bi->fru_cardtype);
        }
        break;
    }
    case BOARDCMD_FRU_SHOW: {
        if (board_conf_is_pcie_card()) {
            sprom_ipmi_mpark_t *sprom = *(sprom_ipmi_mpark_t **)arg;

            ipmi_sprom_mpark_dump(sprom, bi->fru_cardtype);
        } else {
            sprom_ipmi_mezz_t *sprom = *(sprom_ipmi_mezz_t **)arg;

            ipmi_sprom_mezz_dump(sprom, bi->fru_cardtype);
        }
        break;
    }
    case BOARDCMD_FRU_VERIFY: {
        if (board_conf_is_pcie_card()) {
            sprom_ipmi_mpark_t *sprom = *(sprom_ipmi_mpark_t **)arg;
            ipmi_mezz_internal_use_t *iu =  &sprom->internal_use;
            ipmi_ncsi_multi_record_t *mr =  &sprom->ncsi_multi_record;

            if (ipmi_header_checksum_verify((u_int8_t *)sprom) != 0) {
                ret = -1;
                break;
            }
            if (ipmi_zero_checksum_verify((u_int8_t *)iu, sizeof (*iu)) != 0) {
                ret = -2;
                break;
            }
            if (ipmi_multi_record_checksum_verify(&mr->header)) {
                ret = -3;
                break;
            }
        } else {
            sprom_ipmi_mezz_t *sprom = *(sprom_ipmi_mezz_t **)arg;
            ipmi_mezz_internal_use_t *iu =  &sprom->internal_use;

            if (ipmi_header_checksum_verify((u_int8_t *)sprom) != 0) {
                ret = -1;
                break;
            }
            if (ipmi_zero_checksum_verify((u_int8_t *)iu, sizeof (*iu)) != 0) {
                ret = -2;
                break;
            }
        }
        break;
    }
    case BOARDCMD_CHECK_RETIMER_PRESENCE:
        if (!has_retimer_valid) {
            bi->has_retimer = is_retimer_present();
            has_retimer_valid = 1;
        }
        *(int *)arg = bi->has_retimer;
        break;

#endif // !__KERNEL__ && !__UBOOT__

    default:
        ret = -1;
        break;
    }
    return (ret);
}

#if defined CONFIG_BASE_PALO

static int
baylands_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = -1;

    switch (cmd) {
    case BOARDCMD_BEGIN_I2C0RESET:
    case BOARDCMD_SET_I2C0RESET:
    case BOARDCMD_END_I2C0RESET:
    case BOARDCMD_GET_MFGDIAG:
        break;

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static int
paloalto_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = -1;

    switch (cmd) {
    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static int
montereypark_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    u_int32_t val;
    int ret = 0;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
        ret = -1;
        break;

    case BOARDCMD_SET_MDIOMUXBMC:
        val = (*(int *)arg) ? 0 : 1;
        gpio_output(PL_MP_GPIO_MDIO_MUX_SEL, val);
        break;

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static void
palo_config_baylands(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_BAYLANDS,
        .brd_type_name      = "Baylands",
        .brd_type_path      = "baylands",
        .adapter_name       = "Baylands",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_BAYLANDS,
        .memtop             = PALOMEM_MEMTOP,
        .ioctl              = baylands_ioctl,
        .gpio_init_ddr      = PL_BL_GPIO_DDR_INIT,
        .gpio_init_val      = PL_BL_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_BL_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_BL_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .has_i2c0reset      = 0,
        .has_statusled      = 1,
        .do_9541_workaround = 1,
        .use_alt_post_target = 1,
        .statusled_pin      = PL_BL_GPIO_LED_STATUS,
        .has_faultled       = 1,
        .faultled_pin       = PL_BL_GPIO_LED_FAULT,
        .phyreset_pin       = PL_BL_GPIO_PHY_RESET,
        .has_mfgdiag        = 0,
        .has_armflashreset  = 1,
        .resetarm_val       = 0xf,
        .resetarm_mask      = 0xf,
        .card_address_pin   = PL_BL_GPIO_CARD_ADDRESS,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_M81KR,
        .fwcompat           = PALOFW_COMPAT_BAYLANDS,
        .is_lowpower        = 0,
        .has_bmcphy         = 0,
        .has_mdiomux        = 0,
        .has_therm          = 1,
        .therm_type         = BOARD_CONF_THERM_MAX6698,
        .therm_i2c_port     = 1,
        .is_therm_protected = 0,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_PALO,
        .fru_datasize       = sizeof (sprom_ipmi_mezz_t),
        .cisco_card_mode    = 0,
        .bmc_lifid          = BMC_LIFID,
    };

    *bi = info;
}

static void
palo_config_paloalto(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_PALOALTO,
        .brd_type_name      = "Palo Alto",
        .brd_type_path      = "paloalto",
        .adapter_name       = "UCS VIC M81KR",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_PALOALTO,
        .memtop             = PALOMEM_MEMTOP,
        .ioctl              = paloalto_ioctl,
        .gpio_init_ddr      = PL_PA_GPIO_DDR_INIT,
        .gpio_init_val      = PL_PA_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_PA_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_PA_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .has_i2c0reset      = 1,
        .i2c0reset_pin      = PL_PA_GPIO_I2C0_SCL,
        .do_9541_workaround = 1,
        .has_statusled      = 1,
        .statusled_pin      = PL_PA_GPIO_LED_STATUS,
        .has_faultled       = 1,
        .faultled_pin       = PL_PA_GPIO_LED_FAULT,
        .phyreset_pin       = PL_PA_GPIO_PHY_RESET,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_PA_GPIO_MFGDIAG_JUMPER,
        .has_armflashreset  = 1,
        .resetarm_val       = 0xf,
        .resetarm_mask      = 0xf,
        .card_address_pin   = PL_PA_GPIO_CARD_ADDRESS,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_M81KR,
        .fwcompat           = PALOFW_COMPAT_PALOALTO,
        .is_lowpower        = 0,
        .has_bmcphy         = 0,
        .has_mdiomux        = 0,
        .has_therm          = 1,
        .therm_type         = BOARD_CONF_THERM_MAX6698,
        .therm_i2c_port     = 1,
        .is_therm_protected = 0,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_PALO,
        .fru_datasize       = sizeof (sprom_ipmi_mezz_t),
        .cisco_card_mode    = 0,
        .bmc_lifid          = BMC_LIFID,
    };

    *bi = info;
}

static void
palo_config_montereypark(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_MONTEREYPARK,
        .brd_type_name      = "Monterey Park",
        .brd_type_path      = "montereypark",
        .adapter_name       = "UCS VIC P81E",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_MONTEREYPARK,
        .memtop             = PALOMEM_MEMTOP,
        .ioctl              = montereypark_ioctl,
        .gpio_init_ddr      = PL_MP_GPIO_DDR_INIT,
        .gpio_init_val      = PL_MP_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_MP_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_MP_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .has_i2c0reset      = 1,
        .i2c0reset_pin      = PL_MP_GPIO_I2C0_SCL,
        .has_statusled      = 1,
        .statusled_pin      = PL_MP_GPIO_LED_STATUS,
        .phyreset_pin       = PL_MP_GPIO_PHY_RESET,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_MP_GPIO_MFGDIAG_JUMPER,
        .has_armflashreset  = 1,
        .resetarm_val       = 0xf,
        .resetarm_mask      = 0xf,
        .card_address_pin   = PL_MP_GPIO_CARD_ADDRESS,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_P81E,
        .fwcompat           = PALOFW_COMPAT_MONTEREYPARK,
        .has_bmcphy         = 1,
        .bmc_phyaddr        = 1,
        .has_mdiomux        = 1,
        .has_therm          = 1,
        .therm_type         = BOARD_CONF_THERM_MAX6698,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_MONTEREYPARK,
        .fru_datasize       = sizeof (sprom_ipmi_mpark_t),
        .has_pex            = 1,
        .cisco_card_mode    = 0,
        .bmc_lifid          = BMC_LIFID,
        .is_pcie_card       = 1,
    };

    *bi = info;

    bi->is_lowpower = !gpio_input(PL_MP_GPIO_MAINPOWER);
}

static int
palo_get_brd_type(void)
{
    int b0, b1, b2;

    gpio_setdirection(PL_PA_GPIO_BOARDTYPE_0, GPIO_DIR_INPUT);
    b0 = gpio_input(PL_PA_GPIO_BOARDTYPE_0);

    /*
     * xx1 == PaloAlto
     */
    if (b0) {
        return (BOARD_CONF_BRD_PALOALTO);
    }

    gpio_setdirection(PL_BL_GPIO_BOARDTYPE_1, GPIO_DIR_INPUT);
    gpio_setdirection(PL_BL_GPIO_BOARDTYPE_2, GPIO_DIR_INPUT);

    b1 = gpio_input(PL_BL_GPIO_BOARDTYPE_1);
    b2 = gpio_input(PL_BL_GPIO_BOARDTYPE_2);

    /*
     * 010 == MontereyPark
     */
    if (!b2 && b1) {
        return (BOARD_CONF_BRD_MONTEREYPARK);
    }

    /*
     * 000 == Baylands, but also make it default.
     */
    return (BOARD_CONF_BRD_BAYLANDS);
}

static void
palo_config(struct __board_conf_info *bi)
{
    switch (palo_get_brd_type()) {
    case BOARD_CONF_BRD_PALOALTO:
        palo_config_paloalto(bi);
        break;

    case BOARD_CONF_BRD_MONTEREYPARK:
        palo_config_montereypark(bi);
        break;

    default:
        palo_config_baylands(bi);
        break;
    }

#if !defined(__KERNEL__) && !defined(__UBOOT__)
    // default values for supported speed
    if (bi->supported_speed_cfg == 0) {
        bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_10G);
    }
#endif
}

static void
palito_config(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_PALITO,
        .brd_type_name      = "Palito",
        .brd_type_path      = "palito",
        .adapter_name       = "Palito",
        .memtop             = PALOMEM_MEMTOP,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_M81KR,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_PALO,
        .fru_datasize       = sizeof (sprom_ipmi_mezz_t),
        .bmc_lifid          = BMC_LIFID,
    };

    *bi = info;
}

static void
palote_config(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_PALOTE,
        .brd_type_name      = "Palote",
        .brd_type_path      = "palote",
        .adapter_name       = "Palote",
        .memtop             = PALOMEM_MEMTOP,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_M81KR,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_PALO,
        .fru_datasize       = sizeof (sprom_ipmi_mezz_t),
        .bmc_lifid          = BMC_LIFID,
    };

    *bi = info;
#if !defined(__KERNEL__) && !defined(__UBOOT__)
    bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_10G);
#endif
}

#elif defined CONFIG_BASE_SERENO

static int
greenwood_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = -1;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
    case BOARDCMD_SET_MDIOMUXBMC:
    case BOARDCMD_SET_PHYRESET:
    case BOARDCMD_BEGIN_I2C0RESET:
    case BOARDCMD_SET_I2C0RESET:
    case BOARDCMD_END_I2C0RESET:
    case BOARDCMD_ARM_FLASHRESET:
        break;

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static int
oakland_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = -1;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
        ret = -1;
        break;
    case BOARDCMD_SET_MDIOMUXBMC:
    case BOARDCMD_SET_PHYRESET:
    case BOARDCMD_BEGIN_I2C0RESET:
    case BOARDCMD_SET_I2C0RESET:
    case BOARDCMD_END_I2C0RESET:
    case BOARDCMD_ARM_FLASHRESET:
        break;

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return ret;
}

static int
walnutcreek_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = -1;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
        ret = -1;
        break;
    case BOARDCMD_SET_MDIOMUXBMC:
    case BOARDCMD_SET_PHYRESET:
    case BOARDCMD_BEGIN_I2C0RESET:
    case BOARDCMD_SET_I2C0RESET:
    case BOARDCMD_END_I2C0RESET:
    case BOARDCMD_ARM_FLASHRESET:
        break;

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return ret;
}

static int
lexington_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = 0;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
        ret = -1;
        break;
    case BOARDCMD_IS_LOWPOWER: {
        *(int *)arg = !gpio_input(PL_LX_GPIO_MAINPOWER);
        break;
    }
    case BOARDCMD_CISCO_CARD_MODE: {
        *(int *)arg = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
        break;
    }

#ifdef __KERNEL__
    case BOARDCMD_PWR_INTR_INIT: {
        int pin = PL_LX_GPIO_MAINPOWER;
        int pwr_st = gpio_input(pin);
        set_gpio_reg(GPIO_INTTYPE_LEVEL, 0, pin);
        set_gpio_reg(GPIO_INT_POLARITY, 1 ^ pwr_st, pin);
        set_gpio_reg(GPIO_INTEN, 1, pin);
        *(int *)arg = pin;
        break;
    }
    case BOARDCMD_PWR_INTR_CHG_POL: {
        int pwr_st = gpio_input(PL_LX_GPIO_MAINPOWER);
        int polarity = pwr_st ^ 1;
        set_gpio_reg(GPIO_INT_POLARITY, polarity, PL_LX_GPIO_MAINPOWER);
        *(int *)arg = pwr_st;
        break;
    }
#endif

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static int
kirkwoodlake_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = 0;
    int val;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
        ret = -1;
        break;

    case BOARDCMD_IS_LOWPOWER: {
        *(int *)arg = !gpio_input(PL_KL_GPIO_MAINPOWER);
        break;
    }
    case BOARDCMD_SET_MDIOMUXBMC:
        val = (*(int *)arg) ? 1 : 0;
        gpio_setdirection(PL_KL_GPIO_MDIO_MUX_SEL, GPIO_DIR_OUTPUT);
        gpio_output(PL_KL_GPIO_MDIO_MUX_SEL, val);
        break;
    case BOARDCMD_CISCO_CARD_MODE: {
        *(int *)arg = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
        break;
    }

#ifdef __KERNEL__
    case BOARDCMD_PWR_INTR_INIT: {
        int pin = PL_KL_GPIO_MAINPOWER;
        int pwr_st = gpio_input(pin);

        set_gpio_reg(GPIO_INTTYPE_LEVEL, 0, pin);
        set_gpio_reg(GPIO_INT_POLARITY, 1 ^ pwr_st, pin);
        set_gpio_reg(GPIO_INTEN, 1, pin);
        *(int *)arg = pin;
        break;
    }
    case BOARDCMD_PWR_INTR_CHG_POL: {
        int pwr_st = gpio_input(PL_KL_GPIO_MAINPOWER);
        int polarity = pwr_st ^ 1;

        set_gpio_reg(GPIO_INT_POLARITY, polarity, PL_KL_GPIO_MAINPOWER);
        *(int *)arg = pwr_st;
        break;
    }
#endif

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static int
icehouse_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = 0;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
        ret = -1;
        break;
    case BOARDCMD_IS_LOWPOWER: {
        *(int *)arg = !gpio_input(PL_ICH_GPIO_MAINPOWER);
        break;
    }
    case BOARDCMD_CISCO_CARD_MODE: {
        *(int *)arg = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
        break;
    }
#ifdef __KERNEL__
    case BOARDCMD_PWR_INTR_INIT: {
        int pin = PL_ICH_GPIO_MAINPOWER;
        int pwr_st = gpio_input(pin);
        set_gpio_reg(GPIO_INTTYPE_LEVEL, 0, pin);
        set_gpio_reg(GPIO_INT_POLARITY, 1 ^ pwr_st, pin);
        set_gpio_reg(GPIO_INTEN, 1, pin);
        *(int *)arg = pin;
        break;
    }
    case BOARDCMD_PWR_INTR_CHG_POL: {
        int pwr_st = gpio_input(PL_ICH_GPIO_MAINPOWER);
        int polarity = pwr_st ^ 1;
        set_gpio_reg(GPIO_INT_POLARITY, polarity, PL_ICH_GPIO_MAINPOWER);
        *(int *)arg = pwr_st;
        break;
    }
#endif
    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

// CHECK if needed
static int
icehouse2_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = 0;
    int val;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
        ret = -1;
        break;

    case BOARDCMD_IS_LOWPOWER: {
        *(int *)arg = !gpio_input(PL_IH2_GPIO_MAINPOWER);
        break;
    }
    case BOARDCMD_SET_MDIOMUXBMC:
        val = (*(int *)arg) ? 1 : 0;
        gpio_setdirection(PL_IH2_GPIO_MDIO_MUX_SEL, GPIO_DIR_OUTPUT);
        gpio_output(PL_IH2_GPIO_MDIO_MUX_SEL, val);
        break;
    case BOARDCMD_CISCO_CARD_MODE: {
        *(int *)arg = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
        break;
    }

#ifdef __KERNEL__
    case BOARDCMD_PWR_INTR_INIT: {
        int pin = PL_IH2_GPIO_MAINPOWER;
        int pwr_st = gpio_input(pin);

        set_gpio_reg(GPIO_INTTYPE_LEVEL, 0, pin);
        set_gpio_reg(GPIO_INT_POLARITY, 1 ^ pwr_st, pin);
        set_gpio_reg(GPIO_INTEN, 1, pin);
        *(int *)arg = pin;
        break;
    }
    case BOARDCMD_PWR_INTR_CHG_POL: {
        int pwr_st = gpio_input(PL_IH2_GPIO_MAINPOWER);
        int polarity = pwr_st ^ 1;

        set_gpio_reg(GPIO_INT_POLARITY, polarity, PL_KL_GPIO_MAINPOWER);
        *(int *)arg = pwr_st;
        break;
    }
#endif

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static int
susanville_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = 0;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
        ret = -1;
        break;
    case BOARDCMD_IS_LOWPOWER: {
        *(int *)arg = !gpio_input(PL_SVL_GPIO_MAINPOWER);
        break;
    }
    case BOARDCMD_CISCO_CARD_MODE: {
        *(int *)arg = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
        break;
    }

#ifdef __KERNEL__
    case BOARDCMD_PWR_INTR_INIT: {
        int pin = PL_SVL_GPIO_MAINPOWER;
        int pwr_st = gpio_input(pin);
        set_gpio_reg(GPIO_INTTYPE_LEVEL, 0, pin);
        set_gpio_reg(GPIO_INT_POLARITY, 1 ^ pwr_st, pin);
        set_gpio_reg(GPIO_INTEN, 1, pin);
        *(int *)arg = pin;
        break;
    }
    case BOARDCMD_PWR_INTR_CHG_POL: {
        int pwr_st = gpio_input(PL_SVL_GPIO_MAINPOWER);
        int polarity = pwr_st ^ 1;
        set_gpio_reg(GPIO_INT_POLARITY, polarity, PL_SVL_GPIO_MAINPOWER);
        *(int *)arg = pwr_st;
        break;
    }
#endif

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static int
torrance_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = 0;
    int val;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
        ret = -1;
        break;

    case BOARDCMD_IS_LOWPOWER: {
        *(int *)arg = !gpio_input(PL_TC_GPIO_MAINPOWER);
        break;
    }
    case BOARDCMD_SET_MDIOMUXBMC:
        val = (*(int *)arg) ? 1 : 0;
        gpio_setdirection(PL_TC_GPIO_MDIO_MUX_SEL, GPIO_DIR_OUTPUT);
        gpio_output(PL_TC_GPIO_MDIO_MUX_SEL, val);
        break;
    case BOARDCMD_CISCO_CARD_MODE: {
        *(int *)arg = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
        break;
    }

#ifdef __KERNEL__
    case BOARDCMD_PWR_INTR_INIT: {
        int pin = PL_TC_GPIO_MAINPOWER;
        int pwr_st = gpio_input(pin);

        set_gpio_reg(GPIO_INTTYPE_LEVEL, 0, pin);
        set_gpio_reg(GPIO_INT_POLARITY, 1 ^ pwr_st, pin);
        set_gpio_reg(GPIO_INTEN, 1, pin);
        *(int *)arg = pin;
        break;
    }
    case BOARDCMD_PWR_INTR_CHG_POL: {
        int pwr_st = gpio_input(PL_TC_GPIO_MAINPOWER);
        int polarity = pwr_st ^ 1;

        set_gpio_reg(GPIO_INT_POLARITY, polarity, PL_TC_GPIO_MAINPOWER);
        *(int *)arg = pwr_st;
        break;
    }
#endif

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static u_int8_t
is_cotati_in_40g_mode(void)
{
   u_int8_t ret = 0;
   ret |= gpio_input(PL_CT_GPIO_SEL_20G_40G);
   ret &= gpio_input(PL_CT_GPIO_MLOM_ON_BLADE);
   return (ret);
}

static int
cotati_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = -1;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
    case BOARDCMD_SET_MDIOMUXBMC:
    case BOARDCMD_SET_PHYRESET:
    case BOARDCMD_BEGIN_I2C0RESET:
    case BOARDCMD_SET_I2C0RESET:
    case BOARDCMD_END_I2C0RESET:
    case BOARDCMD_ARM_FLASHRESET:
        break;

    case BOARDCMD_IS_40G_MODE:
        if (!bi->is_40g_valid) {
            bi->is_40g_mode =  is_cotati_in_40g_mode();
            bi->is_40g_valid = 1;
        }
        *(int *)arg = bi->is_40g_mode;
        break;

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static int
vasona_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = -1;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
    case BOARDCMD_SET_MDIOMUXBMC:
    case BOARDCMD_SET_PHYRESET:
    case BOARDCMD_BEGIN_I2C0RESET:
    case BOARDCMD_SET_I2C0RESET:
    case BOARDCMD_END_I2C0RESET:
    case BOARDCMD_ARM_FLASHRESET:
        break;

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}


static int
sereno_get_brd_type(void)
{
    int brd;

    gpio_setdirection(PL_GW_GPIO_BOARDTYPE_0, GPIO_DIR_INPUT);
    gpio_setdirection(PL_GW_GPIO_BOARDTYPE_1, GPIO_DIR_INPUT);
    gpio_setdirection(PL_GW_GPIO_BOARDTYPE_2, GPIO_DIR_INPUT);

    brd  = gpio_input(PL_GW_GPIO_BOARDTYPE_2) << 2;
    brd |= gpio_input(PL_GW_GPIO_BOARDTYPE_1) << 1;
    brd |= gpio_input(PL_GW_GPIO_BOARDTYPE_0);

    switch (brd) {
    case 0:
    default:
        return (BOARD_CONF_BRD_GREENWOOD);

    case 1:
        return (BOARD_CONF_BRD_VASONA);

    case 2:
        return (BOARD_CONF_BRD_LEXINGTON);

    case 3:
        return (BOARD_CONF_BRD_KIRKWOODLAKE);

    case 4:
        brd = get_flash_boardid();
        if (brd < 0) {
            return (BOARD_CONF_BRD_GREENWOOD);
        }
        return (brd);

    case 5:
        return (BOARD_CONF_BRD_COTATI);

    case 6:
        return BOARD_CONF_BRD_OAKLAND;

    case 7:
        return (BOARD_CONF_BRD_ICEHOUSE);
    }
}

static void
serenote_config(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_SERENOTE,
        .brd_type_name      = "Serenote",
        .brd_type_path      = "serenote",
        .adapter_name       = "Serenote",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_GREENWOOD,
        .memtop             = PALOMEM128_MEMTOP,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_VASONA,
        .has_i2c            = 1,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_VASONA,
        .fru_datasize       = sizeof (sprom_ipmi_mezz_t),
        .cisco_card_mode    = 0,
        .bmc_lifid          = BMC_LIFID,
        .is_netflow_supp    = 1,
    };

    *bi = info;
}

static void
sereno_config_greenwood(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_GREENWOOD,
        .brd_type_name      = "Greenwood",
        .brd_type_path      = "greenwood",
        .adapter_name       = "Greenwood",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_GREENWOOD,
        .memtop             = PALOMEM256_MEMTOP,
        .ioctl              = greenwood_ioctl,
        .gpio_init_ddr      = PL_GW_GPIO_DDR_INIT,
        .gpio_init_val      = PL_GW_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_GW_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_GW_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .has_i2c0reset      = 1,
        .i2c0reset_pin      = PL_LX_GPIO_I2C0_SCL,
        .card_address_pin   = PL_GW_GPIO_CARD_ADDRESS,
        .has_statusled      = 1,
        .statusled_pin      = PL_GW_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_GW_GPIO_MFGDIAG,
        .has_therm          = 1,
        .therm_type         = BOARD_CONF_THERM_AD7416,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .fwcompat           = PALOFW_COMPAT_GREENWOOD,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_LEXINGTON,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_VASONA,
        .fru_datasize       = sizeof (sprom_ipmi_mezz_t),
        .cisco_card_mode    = 0,
        .bmc_lifid          = BMC_LIFID,
    };

    *bi = info;
}

static void
sereno_config_vasona(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_VASONA,
        .brd_type_name      = "Vasona",
        .brd_type_path      = "vasona",
        .adapter_name       = "UCS VIC 1280",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_VASONA,
        .memtop             = PALOMEM256_MEMTOP,
        .ioctl              = vasona_ioctl,
        .gpio_init_ddr      = PL_VS_GPIO_DDR_INIT,
        .gpio_init_val      = PL_VS_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_VS_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_VS_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .card_address_pin   = PL_VS_GPIO_CARD_ADDRESS,
        .has_statusled      = 1,
        .statusled_pin      = PL_VS_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_VS_GPIO_MFGDIAG,
        .fwcompat           = PALOFW_COMPAT_VASONA,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_VASONA,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_VASONA,
        .fru_datasize       = sizeof (sprom_ipmi_mezz_t),
        .cisco_card_mode    = 0,
        .bmc_lifid          = BMC_LIFID,
        .retimer_cnt        = VASONA_MAX_RETIMER,
        .is_netflow_supp    = 1,
        .reset_with_host = 1,
    };

    *bi = info;
}

static void
sereno_config_cotati(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_COTATI,
        .brd_type_name      = "Cotati",
        .brd_type_path      = "cotati",
        .adapter_name       = "UCS VIC 1240",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_COTATI,
        .memtop             = PALOMEM256_MEMTOP,
        .ioctl              = cotati_ioctl,
        .gpio_init_ddr      = PL_CT_GPIO_DDR_INIT,
        .gpio_init_val      = PL_CT_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_CT_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_CT_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .card_address_pin   = PL_CT_GPIO_CARD_ADDRESS,
        .has_statusled      = 1,
        .statusled_pin      = PL_CT_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_CT_GPIO_MFGDIAG,
        .fwcompat           = PALOFW_COMPAT_COTATI,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_COTATI,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_COTATI,
        .fru_datasize       = sizeof (sprom_ipmi_mezz_t),
        .cisco_card_mode    = 0,
        .bmc_lifid          = BMC_LIFID,
        .retimer_cnt        = COTATI_MAX_RETIMER,
        .is_netflow_supp    = 1,
        .reset_with_host = 1,
    };

    *bi = info;
}

static void
sereno_config_lexington(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_LEXINGTON,
        .brd_type_name      = "Lexington",
        .brd_type_path      = "lexington",
        .adapter_name       = "UCS VIC 1225",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_LEXINGTON,
        .memtop             = PALOMEM256_MEMTOP,
        .ioctl              = lexington_ioctl,
        .gpio_init_ddr      = PL_LX_GPIO_DDR_INIT,
        .gpio_init_val      = PL_LX_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_LX_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_LX_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .has_i2c0reset      = 1,
        .i2c0reset_pin      = PL_LX_GPIO_I2C0_SCL,
        .has_statusled      = 1,
        .statusled_pin      = PL_LX_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_LX_GPIO_MFGDIAG,
        .card_address_pin   = PL_LX_GPIO_CARD_ADDRESS,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_LEXINGTON,
        .fwcompat           = PALOFW_COMPAT_LEXINGTON,
        .has_bmcphy         = 1,
        .bmc_phyaddr        = 0xC,
        .has_mdiomux        = 1,
        .has_therm          = 1,
        .therm_type         = BOARD_CONF_THERM_AD7416,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_LEXINGTON,
        // Lexington TODO: data size currently is like Monterey Park
        .fru_datasize       = sizeof (sprom_ipmi_mpark_t),
        .is_lowpower        = 1,
        .has_retimer        = 1,
        .retimer_cnt        = LEXINGTON_MAX_RETIMER,
        .bmc_lifid          = BMC_LIFID_NIV,
        .is_pcie_card       = 1,
        .has_blinker        = 1,
        .poll_xcvr          = 1,
        .is_netflow_supp    = 1,
        .reset_with_host    = 1,
        .net_rst_with_host  = 1,
    };

    *bi = info;
    bi->is_lowpower = !gpio_input(PL_LX_GPIO_MAINPOWER);
    bi->cisco_card_mode = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);

#if !defined(__KERNEL__) && !defined(__UBOOT__)
    bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_AN) |
                              (1 << PALO_UPLINK_SPEED_1G) |
                              (1 << PALO_UPLINK_SPEED_10G);
#endif // !__KERNEL__ && !__UBOOT__
}

static void
sereno_config_kirkwoodlake(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_KIRKWOODLAKE,
        .brd_type_name      = "Kirkwood Lake",
        .brd_type_path      = "kirkwoodlake",
        .adapter_name       = "UCS VIC 1225T",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_KIRKWOODLAKE,
        .memtop             = PALOMEM256_MEMTOP,
        .ioctl              = kirkwoodlake_ioctl,
        .gpio_init_ddr      = PL_KL_GPIO_DDR_INIT,
        .gpio_init_val      = PL_KL_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_KL_GPIO_DDR_INIT,
        .gpio_boot_val      = PL_KL_GPIO_VAL_INIT,
        .has_i2c            = 1,
        .has_i2c0reset      = 1,
        .i2c0reset_pin      = PL_KL_GPIO_I2C0_SCL,
        .has_statusled      = 1,
        .statusled_pin      = PL_KL_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_KL_GPIO_MFGDIAG,
        .card_address_pin   = PL_KL_GPIO_CARD_ADDRESS,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_KIRKWOODLAKE,
        .fwcompat           = PALOFW_COMPAT_KIRKWOODLAKE,
        .has_bmcphy         = 1,
        .bmc_phyaddr        = 0xC,
        .has_mdiomux        = 1,
        .has_therm          = 1,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_KIRKWOOD,
        .fru_datasize       = sizeof (sprom_ipmi_mpark_t),
        .is_lowpower        = 1,
        .has_retimer        = 0,
        .bmc_lifid          = BMC_LIFID_NIV,
        .is_pcie_card       = 1,
        .reset_usercfg      = 1,
        .is_netflow_supp    = 1,
        .reset_with_host = 1,
    };

    *bi = info;
    bi->is_lowpower = !gpio_input(PL_KL_GPIO_MAINPOWER);
    bi->cisco_card_mode = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
#if !defined(__KERNEL__) && !defined(__UBOOT__)
    bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_AN) |
                              (1 << PALO_UPLINK_SPEED_1G) |
                              (1 << PALO_UPLINK_SPEED_10G);
#endif // !__KERNEL__ && !__UBOOT__
}

static void
sereno_config_icehouse(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_ICEHOUSE,
        .brd_type_name      = "icehouse",
        .brd_type_path      = "icehouse",
        .adapter_name       = "UCS VIC 1285", // UCSC-PCIE-CQSFP-02
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_ICEHOUSE,
        .memtop             = PALOMEM256_MEMTOP,
        .ioctl              = icehouse_ioctl,
        .gpio_init_ddr      = PL_ICH_GPIO_DDR_INIT,
        .gpio_init_val      = PL_ICH_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_ICH_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_ICH_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .has_i2c0reset      = 1,
        .i2c0reset_pin      = PL_ICH_GPIO_I2C0_SCL,
        .has_statusled      = 1,
        .statusled_pin      = PL_ICH_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_ICH_GPIO_MFGDIAG,
        .card_address_pin   = PL_ICH_GPIO_CARD_ADDRESS,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_ICEHOUSE,
        .fwcompat           = PALOFW_COMPAT_ICEHOUSE,
        .has_bmcphy         = 1,
        .bmc_phyaddr        = 0xC,
        .has_mdiomux        = 0,
        .has_therm          = 1,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_ICEHOUSE,
        .fru_datasize       = sizeof (sprom_ipmi_mpark_t),
        .is_lowpower        = 1,   // Will be reset below based on GPIO input
        .has_retimer        = 1,
        .retimer_cnt        = ICEHOUSE_MAX_RETIMER,
        .bmc_lifid          = BMC_LIFID_NIV,
        .is_pcie_card       = 1,
        .has_blinker        = 1,
        .reset_usercfg      = 1,
        .poll_xcvr          = 1,
        .reset_with_host    = 1,
        .net_rst_with_host  = 1,
    };

    *bi = info;
    bi->is_lowpower = !gpio_input(PL_ICH_GPIO_MAINPOWER);
    bi->cisco_card_mode = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
#if !defined(__KERNEL__) && !defined(__UBOOT__)
    bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_4x10G) |
                              (1 << PALO_UPLINK_SPEED_40G);
#endif // !__KERNEL__ && !__UBOOT__
}

static void
sereno_config_oakland(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_OAKLAND,
        .brd_type_name      = "Oakland",
        .brd_type_path      = "oakland",
        .adapter_name       = "Oakland",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_OAKLAND,
        .memtop             = PALOMEM256_MEMTOP,
        .ioctl              = oakland_ioctl,
        .gpio_init_ddr      = PL_OL_GPIO_DDR_INIT,
        .gpio_init_val      = PL_OL_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_OL_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_OL_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .has_i2c0reset      = 0,
        .has_statusled      = 1,
        .statusled_pin      = PL_OL_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_OL_GPIO_MFGDIAG,
        .card_address_pin   = PL_OL_GPIO_CARD_ADDRESS,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_OAKLAND_MLOM,
        .fwcompat           = PALOFW_COMPAT_OAKLAND,
        .has_bmcphy         = 0,
        .has_mdiomux        = 0,
        .has_therm          = 0,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_OAKLAND_LOM,
        .fru_datasize       = sizeof (sprom_ipmi_mpark_t),
        .is_lowpower        = 0,
        .has_retimer        = 1,
        .bmc_lifid          = BMC_LIFID,
        .is_pcie_card       = 0,
        .retimer_cnt        = OAKLAND_MAX_RETIMER,
    };

    *bi = info;
}

static void
sereno_config_walnutcreek(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_WALNUTCREEK,
        .brd_type_name      = "walnutcreek",
        .brd_type_path      = "walnutcreek",
        .adapter_name       = "Walnutcreek",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_WALNUTCREEK,
        .memtop             = PALOMEM256_MEMTOP,
        .ioctl              = walnutcreek_ioctl,
        .gpio_init_ddr      = PL_OL_GPIO_DDR_INIT,
        .gpio_init_val      = PL_OL_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_OL_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_OL_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .has_i2c0reset      = 0,
        .has_statusled      = 1,
        .statusled_pin      = PL_OL_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_OL_GPIO_MFGDIAG,
        .card_address_pin   = PL_OL_GPIO_CARD_ADDRESS,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_WALNUTCREEK,
        .fwcompat           = PALOFW_COMPAT_WALNUTCREEK,
        .has_bmcphy         = 0,
        .has_mdiomux        = 0,
        .has_therm          = 0,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_WALNUTCREEK_LOM,
        .fru_datasize       = sizeof (sprom_ipmi_mpark_t),
        .is_lowpower        = 0,
        .has_retimer        = 1,
        .bmc_lifid          = BMC_LIFID,
        .is_pcie_card       = 0,
        .retimer_cnt        = WALNUTCREEK_MAX_RETIMER,
    };

    *bi = info;
}

static void
sereno_config_icehouse2(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_ICEHOUSE2,
        .brd_type_name      = "icehouse2",
        .brd_type_path      = "icehouse2",
        .adapter_name       = "UCS VIC 1285", // UCSC-PCIE-CQSFP-02, TODO
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_ICEHOUSE,
        .memtop             = PALOMEM256_MEMTOP,
        .ioctl              = icehouse2_ioctl,
        .gpio_init_ddr      = PL_IH2_GPIO_DDR_INIT,
        .gpio_init_val      = PL_IH2_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_IH2_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_IH2_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .has_i2c0reset      = 0,
        .has_statusled      = 1,
        .statusled_pin      = PL_IH2_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_IH2_GPIO_MFGDIAG,
        .card_address_pin   = PL_IH2_GPIO_CARD_ADDRESS,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_ICEHOUSE,
        .fwcompat           = PALOFW_COMPAT_ICEHOUSE2,
        .has_bmcphy         = 1,
        .bmc_phyaddr        = 0xC,
        .has_mdiomux        = 1,
        .has_therm          = 1,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_ICEHOUSE,
        .fru_datasize       = sizeof (sprom_ipmi_mpark_t),
        .is_lowpower        = 1,   // Will be reset below based on GPIO input
        .has_retimer        = 0,
        .bmc_lifid          = BMC_LIFID_NIV,
        .is_pcie_card       = 1,
        .has_blinker        = 1,
        .reset_usercfg      = 1,
        .poll_xcvr          = 1,
        .is_netflow_supp    = 1,
        .reset_with_host    = 1,
        .net_rst_with_host  = 1,
    };

    *bi = info;
    bi->is_lowpower = !gpio_input(PL_ICH_GPIO_MAINPOWER);
    bi->cisco_card_mode = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
#if !defined(__KERNEL__) && !defined(__UBOOT__)
    bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_4x10G) |
                              (1 << PALO_UPLINK_SPEED_40G);
#endif // !__KERNEL__ && !__UBOOT__
}

static void
sereno_config_susanville(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_SUSANVILLE,
        .brd_type_name      = "Susanville",
        .brd_type_path      = "susanville",
        .adapter_name       = "UCS VIC 1227",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_SUSANVILLE,
        .memtop             = PALOMEM256_MEMTOP,
        .ioctl              = susanville_ioctl,
        .gpio_init_ddr      = PL_SVL_GPIO_DDR_INIT,
        .gpio_init_val      = PL_SVL_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_SVL_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_SVL_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .has_i2c0reset      = 1,
        .i2c0reset_pin      = PL_SVL_GPIO_I2C0_SCL,
        .has_statusled      = 1,
        .statusled_pin      = PL_SVL_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_SVL_GPIO_MFGDIAG,
        .card_address_pin   = PL_SVL_GPIO_CARD_ADDRESS,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_SUSANVILLE,
        .fwcompat           = PALOFW_COMPAT_SUSANVILLE,
        .has_bmcphy         = 1,
        .bmc_phyaddr        = 0xC,
        .has_mdiomux        = 0,
        .has_therm          = 1,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_SUSANVILLE,
        .fru_datasize       = sizeof (sprom_ipmi_mpark_t),
        .is_lowpower        = 1,
        .has_retimer        = 1,
        .retimer_cnt        = SUSANVILLE_MAX_RETIMER,
        .bmc_lifid          = BMC_LIFID_NIV,
        .is_pcie_card       = 1,
        .has_blinker        = 1,
        .poll_xcvr          = 1,
        .is_netflow_supp    = 1,
        .reset_with_host    = 1,
        .net_rst_with_host  = 1,
    };

    *bi = info;
    bi->is_lowpower = !gpio_input(PL_SVL_GPIO_MAINPOWER);
    bi->cisco_card_mode = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
#if !defined(__KERNEL__) && !defined(__UBOOT__)
    bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_AN) |
                              (1 << PALO_UPLINK_SPEED_1G) |
                              (1 << PALO_UPLINK_SPEED_10G);
#endif // !__KERNEL__ && !__UBOOT__
}

static void
sereno_config_torrance(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_TORRANCE,
        .brd_type_name      = "Torrance",
        .brd_type_path      = "torrance",
        .adapter_name       = "UCS VIC 1227T",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_TORRANCE,
        .memtop             = PALOMEM256_MEMTOP,
        .ioctl              = torrance_ioctl,
        .gpio_init_ddr      = PL_TC_GPIO_DDR_INIT,
        .gpio_init_val      = PL_TC_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_TC_GPIO_DDR_INIT,
        .gpio_boot_val      = PL_TC_GPIO_VAL_INIT,
        .has_i2c            = 1,
        .has_i2c0reset      = 1,
        .i2c0reset_pin      = PL_TC_GPIO_I2C0_SCL,
        .has_statusled      = 1,
        .statusled_pin      = PL_TC_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_TC_GPIO_MFGDIAG,
        .card_address_pin   = PL_TC_GPIO_CARD_ADDRESS,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_TORRANCE,
        .fwcompat           = PALOFW_COMPAT_TORRANCE,
        .has_bmcphy         = 1,
        .bmc_phyaddr        = 0xC,
        .has_mdiomux        = 1,
        .has_therm          = 1,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_TORRANCE,
        .fru_datasize       = sizeof (sprom_ipmi_mpark_t),
        .is_lowpower        = 1,
        .has_retimer        = 0,
        .bmc_lifid          = BMC_LIFID_NIV,
        .is_pcie_card       = 1,
        .reset_usercfg      = 1,
        .is_netflow_supp    = 1,
        .reset_with_host    = 1,
        .net_rst_with_host  = 1,
    };

    *bi = info;
    bi->is_lowpower = !gpio_input(PL_TC_GPIO_MAINPOWER);
    bi->cisco_card_mode = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
#if !defined(__KERNEL__) && !defined(__UBOOT__)
    bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_AN) |
                              (1 << PALO_UPLINK_SPEED_1G) |
                              (1 << PALO_UPLINK_SPEED_10G);
#endif // !__KERNEL__ && !__UBOOT__
}

static void
sereno_config(struct __board_conf_info *bi)
{
    switch (sereno_get_brd_type()) {
    case BOARD_CONF_BRD_GREENWOOD:
    default:
        sereno_config_greenwood(bi);
        break;

    case BOARD_CONF_BRD_VASONA:
        sereno_config_vasona(bi);
        break;

    case BOARD_CONF_BRD_COTATI:
        sereno_config_cotati(bi);
        break;

    case BOARD_CONF_BRD_LEXINGTON:
        sereno_config_lexington(bi);
        break;

    case BOARD_CONF_BRD_ICEHOUSE:
        sereno_config_icehouse(bi);
        break;
    case BOARD_CONF_BRD_OAKLAND:
        sereno_config_oakland(bi);
        break;

    case BOARD_CONF_BRD_KIRKWOODLAKE:
        sereno_config_kirkwoodlake(bi);
        break;

    case BOARD_CONF_BRD_ICEHOUSE2:
        sereno_config_icehouse2(bi);
        break;

    case BOARD_CONF_BRD_WALNUTCREEK:
        sereno_config_walnutcreek(bi);
        break;

    case BOARD_CONF_BRD_SUSANVILLE:
        sereno_config_susanville(bi);
        break;

    case BOARD_CONF_BRD_TORRANCE:
        sereno_config_torrance(bi);
        break;
    }

    // default supported speeds for mezz, pcie cards
#if !defined(__KERNEL__) && !defined(__UBOOT__)
    if (bi->supported_speed_cfg == 0) {
        if (bi->is_pcie_card) {
            bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_10G);
        } else {
            bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_4x10G);
        }
    }
#endif
}

#elif defined CONFIG_BASE_CRUZ

static int
cruzita_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = -1;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
    case BOARDCMD_SET_MDIOMUXBMC:
    case BOARDCMD_SET_PHYRESET:
    case BOARDCMD_BEGIN_I2C0RESET:
    case BOARDCMD_SET_I2C0RESET:
    case BOARDCMD_END_I2C0RESET:
    case BOARDCMD_ARM_FLASHRESET:
        break;

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static void
cruzita_config(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_CRUZITA,
        .brd_type_name      = "Cruzita",
        .brd_type_path      = "cruzita",
        .adapter_name       = "Cruzita",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_CALIMESA,
        .memtop             = PALOMEM_MEMTOP,
        .ioctl              = cruzita_ioctl,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_VASONA, // XXX
        .has_i2c            = 1,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_VASONA, // XXX
        .fru_datasize       = sizeof (sprom_ipmi_mezz_t),
        .pcie_linkwidths    = "4,1,,,4,1,,",
        .has_rc             = 1,
    };

    *bi = info;
}

static int
clearlake_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = -1;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
    case BOARDCMD_SET_MDIOMUXBMC:
    case BOARDCMD_SET_PHYRESET:
    case BOARDCMD_BEGIN_I2C0RESET:
    case BOARDCMD_SET_I2C0RESET:
    case BOARDCMD_END_I2C0RESET:
    case BOARDCMD_ARM_FLASHRESET:
        break;

    case BOARDCMD_IS_LOWPOWER: {
        *(int *)arg = !gpio_input(PL_CL_GPIO_MAINPOWER);
        break;
    }

    case BOARDCMD_CISCO_CARD_MODE: {
        *(int *)arg = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
        break;
    }

#ifdef __KERNEL__
    case BOARDCMD_PWR_INTR_INIT: {
        int pin = PL_CL_GPIO_MAINPOWER;
        int pwr_st = gpio_input(pin);

        set_gpio_reg(GPIO_INTTYPE_LEVEL, 0, pin);
        set_gpio_reg(GPIO_INT_POLARITY, 1 ^ pwr_st, pin);
        set_gpio_reg(GPIO_INTEN, 1, pin);
        *(int *)arg = pin;
        break;
    }
    case BOARDCMD_PWR_INTR_CHG_POL: {
        int pwr_st = gpio_input(PL_CL_GPIO_MAINPOWER);
        int polarity = pwr_st ^ 1;

        set_gpio_reg(GPIO_INT_POLARITY, polarity, PL_CL_GPIO_MAINPOWER);
        *(int *)arg = pwr_st;
        break;
    }
#endif

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static int
claremont_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = -1;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
    case BOARDCMD_SET_MDIOMUXBMC:
    case BOARDCMD_SET_PHYRESET:
    case BOARDCMD_BEGIN_I2C0RESET:
    case BOARDCMD_SET_I2C0RESET:
    case BOARDCMD_END_I2C0RESET:
    case BOARDCMD_ARM_FLASHRESET:
        break;

    case BOARDCMD_IS_LOWPOWER:
        *(int *)arg = !gpio_input(PL_CLM_GPIO_MAINPOWER);
        break;

    case BOARDCMD_CISCO_CARD_MODE:
        *(int *)arg = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
        break;

#ifdef __KERNEL__
    case BOARDCMD_PWR_INTR_INIT: {
        int pin = PL_CLM_GPIO_MAINPOWER;
        int pwr_st = gpio_input(pin);

        set_gpio_reg(GPIO_INTTYPE_LEVEL, 0, pin);
        set_gpio_reg(GPIO_INT_POLARITY, 1 ^ pwr_st, pin);
        set_gpio_reg(GPIO_INTEN, 1, pin);
        *(int *)arg = pin;
        break;
    }

    case BOARDCMD_PWR_INTR_CHG_POL: {
        int pwr_st = gpio_input(PL_CLM_GPIO_MAINPOWER);
        int polarity = pwr_st ^ 1;

        set_gpio_reg(GPIO_INT_POLARITY, polarity, PL_CLM_GPIO_MAINPOWER);
        *(int *)arg = pwr_st;
        break;
    }
#endif

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static int
cruz_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = -1;

    switch (cmd) {
    case BOARDCMD_SET_FAULTLED:
    case BOARDCMD_SET_MDIOMUXBMC:
    case BOARDCMD_SET_PHYRESET:
    case BOARDCMD_BEGIN_I2C0RESET:
    case BOARDCMD_SET_I2C0RESET:
    case BOARDCMD_END_I2C0RESET:
    case BOARDCMD_ARM_FLASHRESET:
        break;

    default:
        ret = palo_common_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static void
calimesa_config(struct __board_conf_info *bi)
{
    static hostid_map_t hostid_map[] = {
      { 0, 0, 0 }, { 1, 0, 0 }, { 2, 0, 0 }, { 3, 0, 0 },
      { 4, 0, 0 }, { 5, 0, 0 }, { 6, 0, 0 }, { 7, 0, 0 } };
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_CALIMESA,
        .brd_type_name      = "Calimesa",
        .brd_type_path      = "calimesa",
        .adapter_name       = "Calimesa",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_CALIMESA,
        .memtop             = PALOMEM_MEMTOP,
        .ioctl              = cruz_ioctl,
        .gpio_init_ddr      = PL_CM_GPIO_DDR_INIT,
        .gpio_init_val      = PL_CM_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_CM_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_CM_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        // .has_i2c0reset      = 1,
        // .i2c0reset_pin      = PL_CM_GPIO_I2C0_SCL,
        .card_address_pin   = PL_CM_GPIO_CARD_ADDRESS,
        .has_statusled      = 1,
        .statusled_pin      = PL_CM_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_CM_GPIO_MFGDIAG,
        .has_therm          = 1,
        .run_thermmon       = 1,
        .therm_type         = BOARD_CONF_THERM_MAX6654,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_VASONA, // XXX
        .fwcompat           = PALOFW_COMPAT_CALIMESA,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_VASONA, // XXX
        .fru_datasize       = sizeof (sprom_ipmi_mezz_t),
        .pcie_linkwidths    = "4,,4,,4,,4,,4,,4,,4,,4,",
        .pcie_poll_compliance = 1,
        .has_bmcphy         = 1,
        .bmc_phyaddr        = 8,
        .bmc_lifid          = BMC_LIFID_NIV,
        .hostid_map      = { NMEL(hostid_map), hostid_map },
    };

    *bi = info;
}

static void
calimesab_config(struct __board_conf_info *bi)
{
    static hostid_map_t hostid_map[] = {
      { 0, 0, 0 }, { 1, 0, 0 }, { 2, 0, 0 }, { 3, 0, 0 },
      { 4, 0, 0 }, { 5, 0, 0 }, { 6, 0, 0 }, { 7, 0, 0 } };
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_CALIMESAB,
        .brd_type_name      = "Calimesa B",
        .brd_type_path      = "calimesab",
        .adapter_name       = "Calimesa B",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_CALIMESA,
        .memtop             = PALOMEM_MEMTOP,
        .ioctl              = cruz_ioctl,
        .gpio_init_ddr      = PL_CB_GPIO_DDR_INIT,
        .gpio_init_val      = PL_CB_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_CB_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_CB_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        // .has_i2c0reset      = 1,
        // .i2c0reset_pin      = PL_CB_GPIO_I2C0_SCL,
        .card_address_pin   = PL_CB_GPIO_CARD_ADDRESS,
        .has_statusled      = 1,
        .statusled_pin      = PL_CB_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_CB_GPIO_MFGDIAG,
        .has_therm          = 1,
        .run_thermmon       = 1,
        .therm_type         = BOARD_CONF_THERM_MAX6654,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_VASONA, // XXX
        .fwcompat           = PALOFW_COMPAT_CALIMESA,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_VASONA, // XXX
        .fru_datasize       = sizeof (sprom_ipmi_mezz_t),
        .pcie_linkwidths    = "4,,4,,4,,4,,4,,4,,4,,4,",
        .pcie_poll_compliance = 1,
        .has_bmcphy         = 1,
        .bmc_phyaddr        = 8,
        .bmc_lifid          = BMC_LIFID_NIV,
        .hostid_map      = { NMEL(hostid_map), hostid_map },
        .has_rc             = 1,
        .has_blinker        = 1,
    };

    *bi = info;
}

static int
montclair_ioctl(struct __board_conf_info *bi, u_int32_t cmd, void *arg)
{
    int ret = 0;

    switch (cmd) {
    case BOARDCMD_GET_CARD_ADDRESS:
        *(int *)arg = 1;
        break;

    default:
        ret = cruz_ioctl(bi, cmd, arg);
        break;
    }
    return (ret);
}

static void
montclair_config(struct __board_conf_info *bi)
{
    static hostid_map_t hostid_map[] = {
      { 0, 0, 0 }, { 0, 1, 0 }, { 1, 0, 0 }, { 1, 1, 0 },
      { 2, 0, 0 }, { 2, 1, 0 }, { 3, 0, 0 }, { 3, 1, 0 },
      { 4, 0, 0 }, { 4, 1, 0 }, { 5, 0, 0 }, { 5, 1, 0 },
      { 6, 0, 0 }, { 6, 1, 0 }, { 7, 0, 0 }, { 7, 1, 0 } };
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_MONTCLAIR,
        .brd_type_name      = "Montclair",
        .brd_type_path      = "montclair",
        .adapter_name       = "Montclair",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_MONTCLAIR,
        .memtop             = PALOMEM_MEMTOP,
        .ioctl              = montclair_ioctl,
        .gpio_init_ddr      = PL_MC_GPIO_DDR_INIT,
        .gpio_init_val      = PL_MC_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_MC_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_MC_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .has_statusled      = 1,
        .statusled_pin      = PL_MC_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_MC_GPIO_MFGDIAG,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_MONTCLAIR,
        .fwcompat           = PALOFW_COMPAT_MONTCLAIR,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_MONTCLAIR_CRUZ,
        .fru_datasize       = sizeof (sprom_ipmi_mezz_t),
        .pcie_linkwidths    = ("2=8,2=9,2=10,2=11,"
                               "2=0,2=1,2=2,2=3,"
                               "2=12,2=13,2=14,2=15,"
                               "2=4,2=5,2=6,2=7"),
        .bmc_lifid          = BMC_LIFID_NIV,
        .hostid_map      = { NMEL(hostid_map), hostid_map },
        .has_rc             = 1,
        .is_multihost       = 1,
        .has_blinker        = 1,
        .has_krtools        = 1,
        .poll_xcvr          = 1,
    };

    *bi = info;
#if !defined(__KERNEL__) && !defined(__UBOOT__)
    bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_AN) |
                              (1 << PALO_UPLINK_SPEED_4x10G) |
                              (1 << PALO_UPLINK_SPEED_40G);
#endif // !__KERNEL__ && !__UBOOT__
}

static void
calistoga_config(struct __board_conf_info *bi)
{
    static hostid_map_t hostid_map[] = { { 0, 0, 0 }, { 0, 0, 1 } };
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_CALISTOGA,
        .brd_type_name      = "Calistoga",
        .brd_type_path      = "calistoga",
        .adapter_name       = "UCS VIC 1340",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_CALISTOGA,
        .memtop             = PALOMEM_MEMTOP,
        .ioctl              = cruz_ioctl,
        .gpio_init_ddr      = PL_CS_GPIO_DDR_INIT,
        .gpio_init_val      = PL_CS_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_CS_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_CS_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .card_address_pin   = PL_CS_GPIO_CARD_ADDRESS,
        .has_statusled      = 1,
        .statusled_pin      = PL_CS_GPIO_LED_STATUS,
        .has_therm          = 1,
        .therm_type         = BOARD_CONF_THERM_AD7416,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_CS_GPIO_MFGDIAG,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_CALISTOGA,
        .fwcompat           = PALOFW_COMPAT_CALISTOGA,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_CALISTOGA,
        .fru_datasize       = sizeof (sprom_ipmi_mezz_t),
        .pcie_linkwidths    = "8,,,,8,,,,,,,,,,,",
        .bmc_lifid          = BMC_LIFID,
        .hostid_map         = { NMEL(hostid_map), hostid_map },
        .is_netflow_supp    = 1,
        .has_krtools        = 1,
        .reset_with_host = 1,
    };

    *bi = info;
}

static void
mountainview_config(struct __board_conf_info *bi)
{
    static hostid_map_t hostid_map[] = { { 0, 0, 0 }, { 0, 0, 1 } };
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_MOUNTAINVIEW,
        .brd_type_name      = "Mountain View",
        .brd_type_path      = "mountainview",
        .adapter_name       = "UCS VIC 1380",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_MOUNTAINVIEW,
        .memtop             = PALOMEM_MEMTOP,
        .ioctl              = cruz_ioctl,
        .gpio_init_ddr      = PL_MV_GPIO_DDR_INIT,
        .gpio_init_val      = PL_MV_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_MV_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_MV_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .card_address_pin   = PL_MV_GPIO_CARD_ADDRESS,
        .has_statusled      = 1,
        .statusled_pin      = PL_MV_GPIO_LED_STATUS,
        .has_therm          = 1,
        .therm_type         = BOARD_CONF_THERM_AD7416,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_MV_GPIO_MFGDIAG,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_MOUNTAINVIEW,
        .fwcompat           = PALOFW_COMPAT_MOUNTAINVIEW,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_MOUNTAINVIEW,
        .fru_datasize       = sizeof (sprom_ipmi_mezz_t),
        .pcie_linkwidths    = "8TR=1,,,,8TR=0,,,,,,,,,,,",
        .bmc_lifid          = BMC_LIFID,
        .hostid_map         = { NMEL(hostid_map), hostid_map },
        .is_netflow_supp    = 1,
        .has_krtools        = 1,
        .reset_with_host = 1,
    };

    *bi = info;
}

static void
clearlake_config(struct __board_conf_info *bi)
{
    static hostid_map_t hostid_map[] = { { 0, 0, 0 }, { 0, 0, 1 } };
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_CLEARLAKE,
        .brd_type_name      = "Clearlake",
        .brd_type_path      = "clearlake",
        .adapter_name       = "UCS VIC 1385",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_CLEARLAKE,
        .memtop             = PALOMEM_MEMTOP,
        .ioctl              = clearlake_ioctl,
        .gpio_init_ddr      = PL_CL_GPIO_DDR_INIT,
        .gpio_init_val      = PL_CL_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_CL_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_CL_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .has_i2c0reset      = 1,
        .i2c0reset_pin      = PL_CL_GPIO_ARB_RESET,
        .card_address       = 1,
        .card_address_valid = 1,
        .has_statusled      = 1,
        .statusled_pin      = PL_CL_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_CL_GPIO_MFGDIAG,
        .has_therm          = 1,
        .therm_type         = BOARD_CONF_THERM_AD7416,
        .therm_i2c_port     = 0x49,
        .is_therm_protected = 1,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_CLEARLAKE,
        .fwcompat           = PALOFW_COMPAT_CLEARLAKE,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_CLEARLAKE,
        .fru_datasize       = sizeof (sprom_ipmi_mpark_t),
        .pcie_linkwidths    = "8,,,,8,,,,,,,,,,,",
        .has_bmcphy         = 1,
        .bmc_phyaddr        = 0xC,
        .has_phyreset       = 1,
        .phyreset_pin       = PL_CL_GPIO_BMC_PHY_RESET,
        .bmc_lifid          = BMC_LIFID_NIV,
        .is_pcie_card       = 1,
        .is_lowpower        = 1,
        .has_blinker        = 1,
        .reset_usercfg      = 1,
        .poll_xcvr          = 1,
        .has_krtools        = 1,
        .hostid_map         = { NMEL(hostid_map), hostid_map },
        .reset_with_host    = 1,
    };

    *bi = info;
    bi->is_lowpower = !gpio_input(PL_CL_GPIO_MAINPOWER);
    bi->cisco_card_mode = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
#if !defined(__KERNEL__) && !defined(__UBOOT__)
    bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_AN) |
                              (1 << PALO_UPLINK_SPEED_4x10G) |
                              (1 << PALO_UPLINK_SPEED_40G);
#endif // !__KERNEL__ && !__UBOOT__
}

static void
mounttian_config(struct __board_conf_info *bi)
{
    static hostid_map_t hostid_map[] = {
      { 0, 0, 0 }, { 0, 1, 0 } };
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_MOUNTTIAN,
        .brd_type_name      = "Mount Tian",
        .brd_type_path      = "mounttian",
        .adapter_name       = "UCSC-C3260-SIOC",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_MOUNTTIAN,
        .memtop             = PALOMEM_MEMTOP,
        .ioctl              = cruz_ioctl,
        .gpio_init_ddr      = PL_MT_GPIO_DDR_INIT,
        .gpio_init_val      = PL_MT_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_MT_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_MT_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .card_address_pin   = PL_MT_GPIO_CARD_ADDRESS,
        .has_statusled      = 1,
        .statusled_pin      = PL_MT_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_MT_GPIO_MFGDIAG,
        .has_therm          = 1,
        .therm_type         = BOARD_CONF_THERM_AD7416,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_MOUNTTIAN,
        .fwcompat           = PALOFW_COMPAT_MOUNTTIAN,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_MOUNTTIAN_CRUZ,
        .fru_datasize       = sizeof (sprom_ipmi_mpark_t),
        .pcie_linkwidths    = "8,,,,8,,,,,,,,,,,",
        .bmc_lifid          = BMC_LIFID_NIV,
        .poll_xcvr          = 1,
        .has_blinker        = 1,
        .is_pcie_card       = 1,
        .hostid_map         = { NMEL(hostid_map), hostid_map },
        .has_krtools        = 1,
        .net_rst_with_host  = 1,
    };

    *bi = info;
}

static void
mounttian2_config(struct __board_conf_info *bi)
{
    static hostid_map_t hostid_map[] = {
      { 0, 0, 0 }, { 1, 0, 0 } };
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_MOUNTTIAN2,
        .brd_type_name      = "Mount Tian2",
        .brd_type_path      = "mounttian2",
        .adapter_name       = "UCSC-C3260-SIOC",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_MOUNTTIAN2,
        .memtop             = PALOMEM_MEMTOP,
        .ioctl              = cruz_ioctl,
        .gpio_init_ddr      = PL_MT_GPIO_DDR_INIT,
        .gpio_init_val      = PL_MT_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_MT_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_MT_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .card_address_pin   = PL_MT_GPIO_CARD_ADDRESS,
        .has_statusled      = 1,
        .statusled_pin      = PL_MT_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_MT_GPIO_MFGDIAG,
        .has_therm          = 1,
        .therm_type         = BOARD_CONF_THERM_AD7416,
        .therm_i2c_port     = 0,
        .is_therm_protected = 1,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_MOUNTTIAN2,
        .fwcompat           = PALOFW_COMPAT_MOUNTTIAN2,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_MOUNTTIAN2_CRUZ,
        .fru_datasize       = sizeof (sprom_ipmi_mpark_t),
        .pcie_linkwidths    = "8,,,,8,,,,,,,,,,,",
        .bmc_lifid          = BMC_LIFID_NIV,
        .poll_xcvr          = 1,
        .has_blinker        = 1,
        .has_krtools        = 1,
        .is_pcie_card       = 1, // to activate muli-record part
        .hostid_map         = { NMEL(hostid_map), hostid_map },
        .is_multihost       = 0,
    };

    *bi = info;
    bi->is_lowpower = !gpio_input(PL_MT2_GPIO_MAIN_POWER);
    bi->cisco_card_mode = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
#if !defined(__KERNEL__) && !defined(__UBOOT__)
    bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_AN) |
                              (1 << PALO_UPLINK_SPEED_4x10G) |
                              (1 << PALO_UPLINK_SPEED_40G);
#endif // !__KERNEL__ && !__UBOOT__
}

static void
claremont_config(struct __board_conf_info *bi)
{
    static hostid_map_t hostid_map[] = { { 0, 0, 0 } };
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_CLAREMONT,
        .brd_type_name      = "Claremont",
        .brd_type_path      = "claremont",
        .adapter_name       = "UCS VIC 1387",
        .adaptercompat      = ADAPTERTYPE_PALO,
        .adaptertype        = ADAPTERTYPE_CLAREMONT,
        .memtop             = PALOMEM_MEMTOP,
        .ioctl              = claremont_ioctl,
        .gpio_init_ddr      = PL_CLM_GPIO_DDR_INIT,
        .gpio_init_val      = PL_CLM_GPIO_VAL_INIT,
        .gpio_boot_ddr      = PL_CLM_GPIO_DDR_BOOT_INIT,
        .gpio_boot_val      = PL_CLM_GPIO_VAL_BOOT_INIT,
        .has_i2c            = 1,
        .has_i2c0reset      = 1,
        .i2c0reset_pin      = PL_CLM_GPIO_ARB_RESET,
        .card_address       = 1,
        .card_address_valid = 1,
        .has_statusled      = 1,
        .statusled_pin      = PL_CLM_GPIO_LED_STATUS,
        .has_mfgdiag        = 1,
        .mfgdiag_pin        = PL_CLM_GPIO_MFGDIAG,
        .has_therm          = 1,
        .therm_type         = BOARD_CONF_THERM_AD7416,
        .therm_i2c_port     = 0x49,
        .is_therm_protected = 1,
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_CLAREMONT,
        .fwcompat           = PALOFW_COMPAT_CLAREMONT,
        .fru_cardtype       = IPMI_IU_CARD_TYPE_CLAREMONT,
        .fru_datasize       = sizeof (sprom_ipmi_mpark_t),
        .pcie_linkwidths    = ",,,,8,,,,,,,,,,,,",
        .has_bmcphy         = 1,
        .bmc_phyaddr        = 0xC,
        .has_phyreset       = 1,
        .phyreset_pin       = PL_CLM_GPIO_BMC_PHY_RESET,
        .bmc_lifid          = BMC_LIFID_NIV,
        .is_pcie_card       = 1,
        .is_lowpower        = 1,
        .has_blinker        = 1,
        .has_krtools        = 1,
        .reset_usercfg      = 1,
        .poll_xcvr          = 1,
        .hostid_map         = { NMEL(hostid_map), hostid_map },
        .reset_with_host    = 1,
    };

    *bi = info;
    bi->is_lowpower = !gpio_input(PL_CLM_GPIO_MAINPOWER);
    bi->cisco_card_mode = ((readreg(MISC_MREGS_STA_PIN_SPARE) & 0x8) ? 0 : 1);
#if !defined(__KERNEL__) && !defined(__UBOOT__)
    bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_AN) |
                              (1 << PALO_UPLINK_SPEED_4x10G) |
                              (1 << PALO_UPLINK_SPEED_40G);
#endif // !__KERNEL__ && !__UBOOT__
}

static void
cruz_config(struct __board_conf_info *bi)
{
    switch (get_flash_boardid()) {
    case BOARD_CONF_BRD_CALIMESAB:
        calimesab_config(bi);
        break;

    case BOARD_CONF_BRD_MONTCLAIR:
        montclair_config(bi);
        break;

    case BOARD_CONF_BRD_CALISTOGA:
        calistoga_config(bi);
        break;

    case BOARD_CONF_BRD_MOUNTAINVIEW:
        mountainview_config(bi);
        break;

    case BOARD_CONF_BRD_MOUNTTIAN:
        mounttian_config(bi);
        break;

    case BOARD_CONF_BRD_MOUNTTIAN2:
        mounttian2_config(bi);
        break;

   case BOARD_CONF_BRD_CLEARLAKE:
        clearlake_config(bi);
        break;

    case BOARD_CONF_BRD_CLAREMONT:
        claremont_config(bi);
        break;

    default:
        calimesa_config(bi);
        break;
    }

    // default supported speed
#if !defined(__KERNEL__) && !defined(__UBOOT__)
    if (bi->supported_speed_cfg == 0) {
        bi->supported_speed_cfg = (1 << PALO_UPLINK_SPEED_4x10G);
    }
#endif
}
#endif /* CONFIG_BASE */

#if defined MISC_EMULATION_SIM
static void
sim_config(struct __board_conf_info *bi)
{
    static struct __board_conf_info info = {
        .brd_type           = BOARD_CONF_BRD_SIM,
        .brd_type_name      = "Sim",
        .brd_type_path      = "sim",
        .adapter_name       = "Sim",
        .bmc_lifid          = BMC_LIFID,
    };

    *bi = info;
}
#endif // MISC_EMULATION_SIM

void
__board_conf_init(void)
{
    static struct __board_conf_info unknown_board_info = {
        .brd_type           = BOARD_CONF_BRD_UNKNOWN,
        .brd_type_name      = "Unknown",
        .brd_type_path      = "unknown",
        .adapter_name       = "Unknown",
        .subsys_id          = PCI_SUBDEVICE_ID_CISCO_M81KR,
        .has_retimer        = 0,
        .bmc_lifid          = BMC_LIFID,
    };
    struct __board_conf_info *bi = &__board_conf_info;

    if (bi->init) {
        return;
    }

    *bi = unknown_board_info;

#ifndef __KERNEL__
    regio_init();
#endif
    switch (readreg(MISC_MREGS_EMULATION)) {

#if defined CONFIG_BASE_PALO
    case MISC_EMULATION_PALITO:
        palito_config(bi);
        break;
    case MISC_EMULATION_PALOTE:
        palote_config(bi);
        break;
    case MISC_EMULATION_PALO:
        palo_config(bi);
        break;
#endif /* CONFIG_BASE_PALO */

#if defined CONFIG_BASE_SERENO
    case MISC_EMULATION_SERENOTE:
        serenote_config(bi);
        break;
    case MISC_EMULATION_SERENO:
        sereno_config(bi);
        break;
#endif /* CONFIG_BASE_SERENO */

#if defined CONFIG_BASE_CRUZ
    case MISC_EMULATION_CRUZITA:
        cruzita_config(bi);
        break;
    case MISC_EMULATION_CRUZ:
        cruz_config(bi);
        break;
#endif /* CONFIG_BASE_CRUZ */

#if defined MISC_EMULATION_SIM
    case MISC_EMULATION_SIM:
        sim_config(bi);
        break;
#endif // MISC_EMULATION_SIM

    default:
        break;
    }
    bi->init = 1;
}

int
__board_conf_ioctl(u_int32_t cmd, void *arg)
{
    struct __board_conf_info *bi = &__board_conf_info;
    int ret = -1;

    __board_conf_init();
    if (bi->ioctl != NULL)
        ret = (*bi->ioctl)(bi, cmd, arg);

    return (ret);
}
