
/*
 * Copyright (c) 2008 Nuova Systems, Inc.  All rights reserved.
 */
#ident "$Id: palo_gpio.h 57549 2010-08-30 00:34:24Z brasmit2 $"

#ifndef _PALOGPIO_H_
#define _PALOGPIO_H_

#define PALO_NGPIOS     16

/******************************************************************************
 **                             Palo Boards                                  **
 ******************************************************************************/

/*
 *  GPIO    Baylands        PaloAlto        MontereyPark
 *  ----------------------------------------------------
 *  0       nc              o:I2C0_SCL(pu)  o:I2C0_SCL(pu)
 *  1       nc              pu              i:MAINPOWER
 *  2       i:LSI0          i:LSI0          i:LSI0
 *  3       i:LSI1          i:LSI1          i:LSI1
 *  4       i:PLOS0         i:PLOS0         i:BMC_AUTH(pu)
 *  5       i:PLOS1         i:PLOS1         to CPLD (unused)
 *  6       o:W2_FLSH0(pd)  nc              i:BOARDTYPE1(pu)
 *  7       o:W2_FLSH1(pd)  nc              i:BOARDTYPE2(pd)
 *  8       o:PHY_RESET(pd) o:PHY_RESET(pd) o:PHY_RESET(pd)
 *  9       i:CARD_ADDR     i:CARD_ADDR     i:CARD_ADDR
 *  10      o:LED10(fl)     nc              o:BMC_MDC(nc)
 *  11      o:LED11(fl)     nc              o:BMC_MDIO(nc)
 *  12      o:LED12(fl)     i:MFGDIAG(pu)   i:MFGDIAG(pu)
 *  13      i:BOARDTYPE(pd) i:BOARDTYPE(pu) i:BOARDTYPE(pd)
 *  14      o:LED_STATUS    o:LED_STATUS    o:LED_STATUS
 *  15      o:LED_FAULT     o:LED_FAULT     o:MDIO_MUX_SEL
 *
 *                  BOARDTYPE2  BOARDTYPE1  BOARDTYPE
 *  Monterey Park       0           1           0
 *  Baylands            0           0           0
 *  PaloAlto            x(*)        x(*)        1
 *
 * (*) Palo GPIOs may have internal pull-downs making these zero (unverified).
 *
 * input(i:), output(o:)
 * no-connect(nc), floating(fl), pull-up(pu), pull-down(pd)
 */

/******************************************************************************
 * Baylands
 */
#define PL_BL_GPIO_0                0   // (nc)
#define PL_BL_GPIO_1                1   // (nc)
#define PL_BL_GPIO_PHY_LSI0         2   // in: phy link state interrupt 0
#define PL_BL_GPIO_PHY_LSI1         3   // in: phy link state interrupt 1
#define PL_BL_GPIO_PHY_PLOS0        4   // in: phy pmd loss of signal 0
#define PL_BL_GPIO_PHY_PLOS1        5   // in: phy pmd loss of signal 1
#define PL_BL_GPIO_PHY_W2_FLSH0     6   // out: phy 2wire flush 0
#define PL_BL_GPIO_BOARDTYPE_1      6   // in: board type 1 (pd)
#define PL_BL_GPIO_PHY_W2_FLSH1     7   // out: phy 2wire flush 1
#define PL_BL_GPIO_BOARDTYPE_2      7   // in: board type 2 (pd)
#define PL_BL_GPIO_PHY_RESET        8   // out: broadcom PHY reset signal
#define PL_BL_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_BL_GPIO_LED10            10  // out: LED
#define PL_BL_GPIO_LED11            11  // out: LED
#define PL_BL_GPIO_LED12            12  // out: LED
#define PL_BL_GPIO_BOARDTYPE_0      13  // in: board type (pd)
#define PL_BL_GPIO_LED_STATUS       14  // out: status LED
#define PL_BL_GPIO_LED_FAULT        15  // out: fault LED

/*
 * Boot block initialization
 * -------------------------
 * Leave the GPIOs alone.
 */
#define PL_BL_GPIO_DDR_BOOT_INIT    0
#define PL_BL_GPIO_VAL_BOOT_INIT    0

/*
 * Main firmware initialization
 * ----------------------------
 * W2_FLSH0 and W2_FLSH1 are unused.  Keep as inputs (pulled down).
 * LED10-12 are unsed.  Keep as inputs.
 * Assert PHY_RESET and keep LEDs off.
 */
#define PL_BL_GPIO_DDR_INIT ( \
            (1 << PL_BL_GPIO_PHY_RESET) | \
            (1 << PL_BL_GPIO_LED_STATUS) | \
            (1 << PL_BL_GPIO_LED_FAULT))

#define PL_BL_GPIO_VAL_INIT ( \
            (0 << PL_BL_GPIO_PHY_RESET) | \
            (0 << PL_BL_GPIO_LED_STATUS) | \
            (0 << PL_BL_GPIO_LED_FAULT))

/******************************************************************************
 * Palo Alto
 */
#define PL_PA_GPIO_I2C0_SCL         0   // in/out: i2c bus 0 clock override
#define PL_PA_GPIO_1                1   // (pu)
#define PL_PA_GPIO_PHY_LSI0         2   // in: phy link state interrupt 0
#define PL_PA_GPIO_PHY_LSI1         3   // in: phy link state interrupt 1
#define PL_PA_GPIO_PHY_PLOS0        4   // in: phy pmd loss of signal 0
#define PL_PA_GPIO_PHY_PLOS1        5   // in: phy pmd loss of signal 1
#define PL_PA_GPIO_6                6   // (nc)
#define PL_PA_GPIO_7                7   // (nc)
#define PL_PA_GPIO_PHY_RESET        8   // out: broadcom PHY reset signal
#define PL_PA_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_PA_GPIO_10               10  // (nc)
#define PL_PA_GPIO_11               11  // (nc)
#define PL_PA_GPIO_MFGDIAG_JUMPER   12  // in: mfg diag jumper (on=0)
#define PL_PA_GPIO_BOARDTYPE_0      13  // in: board type 0 (pu)
#define PL_PA_GPIO_LED_STATUS       14  // out: status LED
#define PL_PA_GPIO_LED_FAULT        15  // out: fault LED

/*
 * Boot block initialization
 * -------------------------
 * Leave the GPIOs alone.
 */
#define PL_PA_GPIO_DDR_BOOT_INIT    0
#define PL_PA_GPIO_VAL_BOOT_INIT    0

/*
 * Main firmware initialization
 * ----------------------------
 * The I20_SCL pin defaults to input, as it is an overide.
 * Assert PHY_RESET and keep LEDs off.
 */
#define PL_PA_GPIO_DDR_INIT ( \
            (1 << PL_PA_GPIO_PHY_RESET) | \
            (1 << PL_PA_GPIO_LED_STATUS) | \
            (1 << PL_PA_GPIO_LED_FAULT))

#define PL_PA_GPIO_VAL_INIT ( \
            (0 << PL_PA_GPIO_PHY_RESET) | \
            (0 << PL_PA_GPIO_LED_STATUS) | \
            (0 << PL_PA_GPIO_LED_FAULT))

/******************************************************************************
 * Monterey Park
 */
#define PL_MP_GPIO_I2C0_SCL         0   // in/out: i2c bus 0 clock override
#define PL_MP_GPIO_MAINPOWER        1   // in: main power mode (on=1)
#define PL_MP_GPIO_PHY_LSI0         2   // in: phy link state interrupt 0
#define PL_MP_GPIO_PHY_LSI1         3   // in: phy link state interrupt 1
#define PL_MP_GPIO_BMC_AUTH         4   // in: bmc authentication
#define PL_MP_GPIO_5                5   // CPLD (unused)
#define PL_MP_GPIO_BOARDTYPE_1      6   // in: board type 1 (pu)
#define PL_MP_GPIO_BOARDTYPE_2      7   // in: board type 2 (pd)
#define PL_MP_GPIO_PHY_RESET        8   // out: broadcom PHY reset signal
#define PL_MP_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_MP_GPIO_BMC_MDC          10  // out: BMC PHY MDC option
#define PL_MP_GPIO_BMC_MDIO         11  // in/out: BMC PHY MDIO option
#define PL_MP_GPIO_MFGDIAG_JUMPER   12  // in: mfg diag jumper (on=0)
#define PL_MP_GPIO_BOARDTYPE_0      13  // in: board type 0 (pd)
#define PL_MP_GPIO_LED_STATUS       14  // out: status LED
#define PL_MP_GPIO_MDIO_MUX_SEL     15  // out: MDIO mux select option

/*
 * Boot block initialization
 * -------------------------
 * Leave the GPIOs alone.
 */
#define PL_MP_GPIO_DDR_BOOT_INIT    0
#define PL_MP_GPIO_VAL_BOOT_INIT    0

/*
 * Main firmware initialization
 * ----------------------------
 * The I20_SCL pin defaults to input, as it is an overide.
 * BMC_MDC and BMC_MDIO default to inputs as they are no-stuff options.
 * Assert PHY_RESET and keep LEDs off.
 * Select the BRCM PHY by driving MDIO_MUX_SEL to 1.
 */
#define PL_MP_GPIO_DDR_INIT ( \
            (1 << PL_MP_GPIO_PHY_RESET) | \
            (1 << PL_MP_GPIO_LED_STATUS) | \
            (1 << PL_MP_GPIO_MDIO_MUX_SEL))

#define PL_MP_GPIO_VAL_INIT ( \
            (0 << PL_MP_GPIO_PHY_RESET) | \
            (0 << PL_MP_GPIO_LED_STATUS) | \
            (1 << PL_MP_GPIO_MDIO_MUX_SEL))

/******************************************************************************
 **                           Sereno Boards                                  **
 ******************************************************************************/
 /*
  *  GPIO    Greenwood        Vasona          Lexington          KirkwoodLake
  *  ------------------------------------------------------------------------
  *  0       i:I2C0_SCL       i:I2C0_SCL       b:BMC_AUTH(pu)    i:BMC_AUTH
  *  1       o:SFP_SEL0       nc               o:BMCPHYINIT(pu)  i:BMCPHYINT
  *  2       o:SFP_SEL1       nc               i:RETIMER_INT(pu) i:PCIE reset
  *  3       o:SFP_SEL2       nc               i:MAINPOWER       i:MainPower
  *  4       i:SFP_MODPRES    nc               i:SFP_MODPRE      nc
  *  5       i:SFP_LOS        nc               o:SFP_SEL0        i:BCM Port0IRQ
  *  6       i:BOARDTYPE1(pd) i:BOARDTYPE1(pd) i:BOARDTYPE1(pu)  i:BOARDTYPE1
  *  7       i:BOARDTYPE2(pd) i:BOARDTYPE2(pd) i:BOARDTYPE2(pd)  i:BOARDTYPE2
  *  8       o:RESET_PCIE     o:RESET_PCIE     o:RESET_PCIE(pu)  o:RESETMARVELL
  *  9       i:CARD_ADDR      i:CARD_ADDR      i:CARD_ADDR       i:CARD_ADDR
  *  10      unused(pu)       nc               o:I2C0_SCL(pu)    b:Mon I2C0 CLK
  *  11      unused(pu)       nc               o:I2C_RST         o:MDIO Select
  *  12      i:MFGDIAG        i:MFGDIAG        i:MFGDIAG         i:MFGDIAG
  *  13      i:BOARDTYPE0(pd) i:BOARDTYPE0(pu) i:BOARDTYPE0(pd)  i:BOARDTYPE0
  *  14      o:LED_STATUS     o:LED_STATUS     o:LED_STATUS      o:LED_STATUS
  *  15      i:BMCPHYINT      nc               o:SFP_SEL1        i:BCM Port1IRQ
  *
  *                  BOARDTYPE2  BOARDTYPE1  BOARDTYPE0
  *  Greenwood           0           0           0
  *  Lexington           0           1           0
  *  Vasona              0           0           1
  *  KirkwoodLake        0           1           1
  *  Icehouse            1           1           1
  *  Icehouse2           1           0           0       (boardidrec schema)
  *  Susanville          1           0           0       (boardidrec schema)
  *  Torrance            1           0           0       (boardidrec schema)
  *
  * input(i:), output(o:)
  * no-connect(nc), floating(fl), pull-up(pu), pull-down(pd)
  */

/******************************************************************************
 * Greenwood
 */
#define PL_GW_GPIO_I2C0_SCL         0   // in: I2C Bus 0 Clock
#define PL_GW_GPIO_SFP_SEL0         1   // out: SFP+ module select [0]
#define PL_GW_GPIO_SFP_SEL1         2   // out: SFP+ module select [1]
#define PL_GW_GPIO_SFP_SEL2         3   // out: SFP+ module select [2]
#define PL_GW_GPIO_SFP_MODPRES      4   // in: SFP+ module present (low = pres)
#define PL_GW_GPIO_SFP_LOS          5   // in: SFP+ Loss of Signal
#define PL_GW_GPIO_BOARDTYPE_1      6   // in: board type 1 (pd)
#define PL_GW_GPIO_BOARDTYPE_2      7   // in: board type 2 (pd)
#define PL_GW_GPIO_RESET_PCIE       8   // out: PCIE reset (low = reset)
#define PL_GW_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_GW_GPIO_10               10  // in: unused pull-up
#define PL_GW_GPIO_11               11  // in: unused pull-up
#define PL_GW_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_GW_GPIO_BOARDTYPE_0      13  // in: board type (pd)
#define PL_GW_GPIO_LED_STATUS       14  // out: status LED
#define PL_GW_GPIO_BMCPHYINT        15  // in: BMC PHY Interrupt

/*
 * Boot block initialization
 * -------------------------
 * Leave the GPIOs alone.
 */
#define PL_GW_GPIO_DDR_BOOT_INIT    0
#define PL_GW_GPIO_VAL_BOOT_INIT    0

/*
 * Main firmware initialization
 * ----------------------------
 * Configure the SFP select pins as output and drive them low.
 * Leave RESET_PCIE alone (pulled up on the board).
 * Configure the LED_STATUS pin as output and turn off.
 */
#define PL_GW_GPIO_DDR_INIT ( \
            (1 << PL_GW_GPIO_SFP_SEL0) | \
            (1 << PL_GW_GPIO_SFP_SEL1) | \
            (1 << PL_GW_GPIO_SFP_SEL2) | \
            (1 << PL_GW_GPIO_LED_STATUS))

#define PL_GW_GPIO_VAL_INIT ( \
            (0 << PL_GW_GPIO_SFP_SEL0) | \
            (0 << PL_GW_GPIO_SFP_SEL1) | \
            (0 << PL_GW_GPIO_SFP_SEL2) | \
            (0 << PL_GW_GPIO_LED_STATUS))

/******************************************************************************
 * Vasona
 */
#define PL_VS_GPIO_I2C0_SCL         0   // in: I2C Bus 0 Clock
#define PL_VS_GPIO_1                1   // (nc)
#define PL_VS_GPIO_2                2   // (nc)
#define PL_VS_GPIO_3                3   // (nc)
#define PL_VS_GPIO_4                4   // (nc)
#define PL_VS_GPIO_5                5   // (nc)
#define PL_VS_GPIO_BOARDTYPE_1      6   // in: board type 1 (pd)
#define PL_VS_GPIO_BOARDTYPE_2      7   // in: board type 2 (pd)
#define PL_VS_GPIO_RESET_PCIE       8   // out: PCIE reset (low = reset)
#define PL_VS_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_VS_GPIO_10               10  // (nc)
#define PL_VS_GPIO_11               11  // (nc)
#define PL_VS_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_VS_GPIO_BOARDTYPE_0      13  // in: board type (pu)
#define PL_VS_GPIO_LED_STATUS       14  // out: status LED
#define PL_VS_GPIO_15               15  // (nc)

/*
 * Boot block initialization
 * -------------------------
 * Leave the GPIOs alone.
 */
#define PL_VS_GPIO_DDR_BOOT_INIT    0
#define PL_VS_GPIO_VAL_BOOT_INIT    0

/*
 * Main firmware initialization
 * ----------------------------
 * Leave RESET_PCIE alone (pulled up on the board).
 * Configure the LED_STATUS pin as output and turn off.
 */
#define PL_VS_GPIO_DDR_INIT ( \
            (1 << PL_VS_GPIO_LED_STATUS))

#define PL_VS_GPIO_VAL_INIT ( \
            (0 << PL_VS_GPIO_LED_STATUS))

/******************************************************************************
 * Cotati
 */
#define PL_CT_GPIO_I2C0_SCL         0   // in: I2C Bus 0 Clock
#define PL_CT_GPIO_1                1   // in: not used (pu)
#define PL_CT_GPIO_2                2   // in: not used (pu)
#define PL_CT_GPIO_3                3   // in: not used from I2C GPIO
#define PL_CT_GPIO_SEL_20G_40G      4   // in: BMC sel 20G/40G (0=20G/1=40G)
#define PL_CT_GPIO_5                5   // in: not used from I2C GPIO
#define PL_CT_GPIO_BOARDTYPE_1      6   // in: board type 1 (pd)
#define PL_CT_GPIO_BOARDTYPE_2      7   // in: board type 2 (pd)
#define PL_CT_GPIO_RESET_PCIE       8   // out: PCIE reset (low = reset)
#define PL_CT_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_CT_GPIO_MLOM_ON_BLADE    10  // in: mLOM lane config (0=20G/1=40G)
#define PL_CT_GPIO_FLASH_RY_BY      11  // in: FLASH ready
#define PL_CT_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_CT_GPIO_BOARDTYPE_0      13  // in: board type (pu)
#define PL_CT_GPIO_LED_STATUS       14  // out: status LED
#define PL_CT_GPIO_GPHY_INT         15  // in: interrupt from BMC PHY

/*
 * Boot block initialization
 * -------------------------
 * Leave the GPIOs alone.
 */
#define PL_CT_GPIO_DDR_BOOT_INIT    0
#define PL_CT_GPIO_VAL_BOOT_INIT    0

/*
 * Main firmware initialization
 * ----------------------------
 * Leave RESET_PCIE alone (pulled up on the board).
 * Configure the LED_STATUS pin as output and turn off.
 */
#define PL_CT_GPIO_DDR_INIT ( \
            (1 << PL_CT_GPIO_LED_STATUS))

#define PL_CT_GPIO_VAL_INIT ( \
            (0 << PL_CT_GPIO_LED_STATUS))

/******************************************************************************
 * Lexington
 */
#define PL_LX_GPIO_BMC_AUTH         0   // in: BMC Authentication
#define PL_LX_GPIO_BMCPHYINT        1   // in: interrupt from BMC PHY
#define PL_LX_GPIO_RETIMER_INT      2   // in: BMC PHY reset (low = reset)
#define PL_LX_GPIO_MAINPOWER        3   // in: Main Power On
#define PL_LX_GPIO_SFP_MODPRES      4   // in: SFP+ module present (low = pres)
#define PL_LX_GPIO_SFP_SEL0         5   // out: SFP+ module select [0]
#define PL_LX_GPIO_BOARDTYPE_1      6   // in: board type 1 (pd)
#define PL_LX_GPIO_BOARDTYPE_2      7   // in: board type 2 (pu)
#define PL_LX_GPIO_RESET_BPHY       8   // out: BMC PHY reset (low = reset)
#define PL_LX_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_LX_GPIO_I2C0_SCL         10  // in: I2C Bus 0 Clock
#define PL_LX_GPIO_I2C_RESET        11  // out: I2C Reset
#define PL_LX_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_LX_GPIO_BOARDTYPE_0      13  // in: board type (pd)
#define PL_LX_GPIO_LED_STATUS       14  // out: status LED
#define PL_LX_GPIO_SFP_SEL1         15  // out: SFP+ module select [1]

/*
 * Boot block initialization
 * -------------------------
 * Leave the GPIOs alone.
 */
#define PL_LX_GPIO_DDR_BOOT_INIT    0
#define PL_LX_GPIO_VAL_BOOT_INIT    0

/*
 * Main firmware initialization
 * ----------------------------
 * Leave RESET_PCIE alone (pulled up on the board).
 * Configure the LED_STATUS pin as output and turn off.
 */

#define PL_LX_GPIO_DDR_INIT ( \
            (1 << PL_LX_GPIO_SFP_SEL0) | \
            (1 << PL_LX_GPIO_SFP_SEL1) | \
            (1 << PL_LX_GPIO_LED_STATUS))

#define PL_LX_GPIO_VAL_INIT ( \
            (0 << PL_LX_GPIO_SFP_SEL0) | \
            (0 << PL_LX_GPIO_SFP_SEL1) | \
            (0 << PL_LX_GPIO_LED_STATUS))

/******************************************************************************
 * KirkwoodLake
 */
#define PL_KL_GPIO_BMC_AUTH         0   // in: BMC Authentication
#define PL_KL_GPIO_BMCPHYINT        1   // in: interrupt from BMC PHY
#define PL_KL_GPIO_PCIE_RESET       2   // in: PCIE reset input
#define PL_KL_GPIO_MAINPOWER        3   // in: Main Power On
#define PL_KL_GPIO_NC4              4   // nc
#define PL_KL_GPIO_BCM_PORT0_IRQ    5   // in: BCM Port0 IRQ
#define PL_KL_GPIO_BOARDTYPE_1      6   // in: board type 1 (pu)
#define PL_KL_GPIO_BOARDTYPE_2      7   // in: board type 2 (pd)
#define PL_KL_GPIO_RESET_BPHY       8   // out: BMC PHY reset (low = reset)
#define PL_KL_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_KL_GPIO_I2C0_SCL         10  // in: I2C Bus 0 Clock
#define PL_KL_GPIO_MDIO_MUX_SEL     11  // out: Kirkwood MDIO mux select option
#define PL_KL_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_KL_GPIO_BOARDTYPE_0      13  // in: board type (pu)
#define PL_KL_GPIO_LED_STATUS       14  // out: status LED
#define PL_KL_GPIO_BCM_PORT1_IRQ    15  // in: BCM Port1 IRQ

/*
 * Boot block initialization
 * -------------------------
 * Leave the GPIOs alone.
 */
#define PL_KL_GPIO_DDR_BOOT_INIT    0
#define PL_KL_GPIO_VAL_BOOT_INIT    0

/*
 * Main firmware initialization
 * ----------------------------
 * Leave RESET_PCIE alone (pulled up on the board).
 * Configure the LED_STATUS pin as output and turn off.
 */

#define PL_KL_GPIO_DDR_INIT ( \
            (1 << PL_KL_GPIO_LED_STATUS))

#define PL_KL_GPIO_VAL_INIT ( \
            (0 << PL_KL_GPIO_LED_STATUS))

/*****************************************************************************
 * GPIO pins for Icehouse card
 */
#define PL_ICH_GPIO_BMC_AUTH         0   // in: BMC Authentication
#define PL_ICH_GPIO_QSFP_INT         1   // in: interrupt for QSFP plug in/out
#define PL_ICH_GPIO_PCIE_RESET       2   // in: BMC PHY reset (low = reset)
#define PL_ICH_GPIO_MAINPOWER        3   // in: Main Power On
#define PL_ICH_GPIO_NOT_USED_4       4   // Not used
#define PL_ICH_GPIO_NOT_USED_5       5   // Not used
#define PL_ICH_GPIO_BOARDTYPE_1      6   // in: board type 1 (pd)
#define PL_ICH_GPIO_BOARDTYPE_2      7   // in: board type 2 (pu)
#define PL_ICH_GPIO_RESET_BPHY       8   // out: BMC PHY reset (low = reset)
#define PL_ICH_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_ICH_GPIO_I2C0_SCL         10  // in: I2C Bus 0 Clock
#define PL_ICH_GPIO_I2C_RESET        11  // out: I2C Reset
#define PL_ICH_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_ICH_GPIO_BOARDTYPE_0      13  // in: board type 0 (pd)
#define PL_ICH_GPIO_LED_STATUS       14  // out: Sereno Heartbeat LED
#define PL_ICH_GPIO_RETIMER_LOADED   15  // in: Retimer firmware load done


/*
 * Main firmware initialization
 * ----------------------------
 * Configure the LED_STATUS pin as output and turn off.
 */
#define PL_ICH_GPIO_DDR_INIT ( \
            (1 << PL_ICH_GPIO_LED_STATUS))

#define PL_ICH_GPIO_VAL_INIT ( \
            (0 << PL_ICH_GPIO_LED_STATUS))

#define PL_ICH_GPIO_DDR_BOOT_INIT ( \
            (1 << PL_ICH_GPIO_LED_STATUS))

#define PL_ICH_GPIO_VAL_BOOT_INIT ( \
            (0 << PL_ICH_GPIO_LED_STATUS))

/******************************************************************************
 * Oakland
 */
#define PL_OL_GPIO_I2C0_SCL         0   // in: I2C Bus 0 Clock
#define PL_OL_GPIO_1                1   // nc
#define PL_OL_GPIO_2                2   // nc
#define PL_OL_GPIO_UART_INTER_CLK   3   // in: use sereno clk for UART
#define PL_OL_GPIO_SEL_20G_40G      4   // in: enable 2 or 4 lane
#define PL_OL_GPIO_SN_BACKUP        5   // obsolete, place holder
#define PL_OL_GPIO_BOARDTYPE_1      6   // in: board type 1 (pd)
#define PL_OL_GPIO_BOARDTYPE_2      7   // in: board type 2 (pu)
#define PL_OL_GPIO_RESET_BPHY       8   // out: BMC PHY reset (low = reset)
#define PL_OL_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_OL_GPIO_MLOM_LANE_CFG    10  // in: mlom lane config
#define PL_OL_GPIO_FLASH_RY_BY      11  // obsolete, flash ready ob
#define PL_OL_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_OL_GPIO_BOARDTYPE_0      13  // in: board type (pd)
#define PL_OL_GPIO_LED_STATUS       14  // out: status LED
#define PL_OL_GPIO_RETIMER_INT      15  // in: BMC PHY reset (low = reset)

/*
 * Boot block initialization
 * -------------------------
 * Leave the GPIOs alone.
 */
#define PL_OL_GPIO_DDR_BOOT_INIT    0
#define PL_OL_GPIO_VAL_BOOT_INIT    0
#define PL_OL_GPIO_DDR_INIT ( \
            (1 << PL_OL_GPIO_LED_STATUS))
#define PL_OL_GPIO_VAL_INIT ( \
            (0 << PL_OL_GPIO_LED_STATUS))

/******************************************************************************
 * Walnutcreek
 */
#define PL_WC_GPIO_I2C0_SCL         0   // in: I2C Bus 0 Clock
#define PL_WC_GPIO_1                1   // nc
#define PL_WC_GPIO_2                2   // nc
#define PL_WC_GPIO_UART_INTER_CLK   3   // in: use sereno clk for UART
#define PL_WC_GPIO_SEL_20G_40G      4   // in: enable 2 or 4 lane
#define PL_WC_GPIO_SN_BACKUP        5   // obsolete, place holder
#define PL_WC_GPIO_BOARDTYPE_1      6   // in: board type 1 (pd)
#define PL_WC_GPIO_BOARDTYPE_2      7   // in: board type 2 (pu)
#define PL_WC_GPIO_RESET_BPHY       8   // out: BMC PHY reset (low = reset)
#define PL_WC_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_WC_GPIO_MLOM_LANE_CFG    10  // in: mlom lane config
#define PL_WC_GPIO_FLASH_RY_BY      11  // obsolete, flash ready ob
#define PL_WC_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_WC_GPIO_BOARDTYPE_0      13  // in: board type (pd)
#define PL_WC_GPIO_LED_STATUS       14  // out: status LED
#define PL_WC_GPIO_RETIMER_INT      15  // in: BMC PHY reset (low = reset)

/*
 * Boot block initialization
 * -------------------------
 * Leave the GPIOs alone.
 */
#define PL_WC_GPIO_DDR_BOOT_INIT    0
#define PL_WC_GPIO_VAL_BOOT_INIT    0
#define PL_WC_GPIO_DDR_INIT ( \
            (1 << PL_WC_GPIO_LED_STATUS))
#define PL_WC_GPIO_VAL_INIT ( \
            (0 << PL_WC_GPIO_LED_STATUS))


/******************************************************************************
 * Icehouse2
 */
#define PL_IH2_GPIO_BMC_AUTH         0   // in: BMC Authentication
#define PL_IH2_GPIO_PHY_LASI         1   // in: LASI
#define PL_IH2_GPIO_PCIE_RESET       2   // in: PCIE reset input
#define PL_IH2_GPIO_MAINPOWER        3   // in: Main Power On
#define PL_IH2_GPIO_PCS1_LINKUP      4   // out: pcs linkup signal to BRCM phy1 in 40G mode
#define PL_IH2_GPIO_PCS0_LINKUP      5   // out: pcs linkup signal to BRCM phy0 in 40G mode
#define PL_IH2_GPIO_BOARDTYPE_1      6   // in: board type 1 (pu)
#define PL_IH2_GPIO_BOARDTYPE_2      7   // in: board type 2 (pd)
#define PL_IH2_GPIO_RESET_BPHY       8   // out: BMC PHY reset (low = reset)
#define PL_IH2_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_IH2_GPIO_I2C0_SCL         10  // in: I2C Bus 0 Clock
#define PL_IH2_GPIO_MDIO_MUX_SEL     11  // out: icehouse2 MDIO mux select option
#define PL_IH2_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_IH2_GPIO_BOARDTYPE_0      13  // in: board type (pu)
#define PL_IH2_GPIO_LED_STATUS       14  // out: Sereno Heartbeat LED
#define PL_IH2_GPIO_PHY_RESET        15  // out: BRCM PHY reset

/*
 * Boot block initialization
 * -------------------------
 * Leave the GPIOs alone.
 */
#define PL_IH2_GPIO_DDR_BOOT_INIT    0
#define PL_IH2_GPIO_VAL_BOOT_INIT    0

/*
 * Main firmware initialization
 * ----------------------------
 * Leave RESET_PCIE alone (pulled up on the board).
 * Configure the LED_STATUS pin as output and turn off.
 */
#define PL_IH2_GPIO_DDR_INIT ( \
            (1 << PL_IH2_GPIO_LED_STATUS))
#define PL_IH2_GPIO_VAL_INIT ( \
            (0 << PL_IH2_GPIO_LED_STATUS))

/******************************************************************************
 * Susanville
 */
#define PL_SVL_GPIO_BMC_AUTH         0   // in: BMC Authentication
#define PL_SVL_GPIO_BMCPHYINT        1   // in: BMC GE PHY Interrupt (active low)
#define PL_SVL_GPIO_PCIE_RESET       2   // in: SFP+ PCIE reset
#define PL_SVL_GPIO_MAINPOWER        3   // in: Main Power On ('1' = ON)
#define PL_SVL_GPIO_SFP_MODPRES      4   // in: SFP+ module present (low = pres)
#define PL_SVL_GPIO_SFP_SEL0         5   // out: SFP+ module select [0]
#define PL_SVL_GPIO_BOARDTYPE_1      6   // in: board type 1
#define PL_SVL_GPIO_BOARDTYPE_2      7   // in: board type 2
#define PL_SVL_GPIO_RESET_BPHY       8   // out: BMC PHY reset (low = reset)
#define PL_SVL_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_SVL_GPIO_I2C0_SCL         10  // in: I2C Bus 0 Clock
#define PL_SVL_GPIO_I2C_RESET        11  // out: I2C Reset
#define PL_SVL_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_SVL_GPIO_BOARDTYPE_0      13  // in: board type (pd)
#define PL_SVL_GPIO_LED_STATUS       14  // out: status LED
#define PL_SVL_GPIO_SFP_SEL1         15  // out: SFP+ module select [1]

/*
 * Boot block initialization
 * -------------------------
 * Leave the GPIOs alone.
 */
#define PL_SVL_GPIO_DDR_BOOT_INIT    0
#define PL_SVL_GPIO_VAL_BOOT_INIT    0

/*
 * Main firmware initialization
 * ----------------------------
 * Leave RESET_PCIE alone (pulled up on the board).
 * Configure the LED_STATUS pin as output and turn off.
 */

#define PL_SVL_GPIO_DDR_INIT ( \
            (1 << PL_SVL_GPIO_SFP_SEL0) | \
            (1 << PL_SVL_GPIO_SFP_SEL1) | \
            (1 << PL_SVL_GPIO_LED_STATUS))

#define PL_SVL_GPIO_VAL_INIT ( \
            (0 << PL_SVL_GPIO_SFP_SEL0) | \
            (0 << PL_SVL_GPIO_SFP_SEL1) | \
            (0 << PL_SVL_GPIO_LED_STATUS))

/******************************************************************************
 * Torrance
 */
#define PL_TC_GPIO_BMC_AUTH         0   // in: BMC Authentication
#define PL_TC_GPIO_BMCPHYINT        1   // in: interrupt from BMC PHY
#define PL_TC_GPIO_PCIE_RESET       2   // in: PCIE reset input
#define PL_TC_GPIO_MAINPOWER        3   // in: Main Power On
#define PL_TC_GPIO_AQ_PORT0_IRQ     4   // in: AQ Port0 IRQ
#define PL_TC_GPIO_AQ_PORT1_IRQ     5   // in: AQ Port1 IRQ
#define PL_TC_GPIO_BOARDTYPE_1      6   // in: board type 1 (pu)
#define PL_TC_GPIO_BOARDTYPE_2      7   // in: board type 2 (pd)
#define PL_TC_GPIO_RESET_BPHY       8   // out: BMC PHY reset (low = reset)
#define PL_TC_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_TC_GPIO_I2C0_SCL         10  // in: I2C Bus 0 Clock
#define PL_TC_GPIO_MDIO_MUX_SEL     11  // out: Torrance MDIO mux select option
#define PL_TC_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_TC_GPIO_BOARDTYPE_0      13  // in: board type (pu)
#define PL_TC_GPIO_LED_STATUS       14  // out: status LED
#define PL_TC_GPIO_NC15             15  // nc

/*
 * Boot block initialization
 * -------------------------
 * Leave the GPIOs alone.
 */
#define PL_TC_GPIO_DDR_BOOT_INIT    0
#define PL_TC_GPIO_VAL_BOOT_INIT    0

/*
 * Main firmware initialization
 * ----------------------------
 * Leave RESET_PCIE alone (pulled up on the board).
 * Configure the LED_STATUS pin as output and turn off.
 */

#define PL_TC_GPIO_DDR_INIT ( \
        (1 << PL_TC_GPIO_LED_STATUS))

#define PL_TC_GPIO_VAL_INIT ( \
        (0 << PL_TC_GPIO_LED_STATUS))

/******************************************************************************
 **                             Cruz Boards                                  **
 ******************************************************************************/

/*
 *  GPIO    Calimesa            CalimesaB           Montclair
 *  ---------------------------------------------------------------------------
 *  0       b:I2C0_SCL          b:I2C0_SCL          i:QSFP_IRQ(pu)
 *  1       o:SFP_SEL0          o:SFP_SEL0          i:QSFP1_PRSNT(pu)
 *  2       o:SFP_SEL1          o:SFP_SEL1          i:QSFP0_PRSNT(pu)
 *  3       o:SFP_SEL2          o:SFP_SEL2          i:PCIESW_FATAL_ERR
 *  4       i:SFP_MODPRES       i:SFP_MODPRES       o:POST_STATUS_0(pu)
 *  5       i:SFP_LOS           i:SFP_LOS           o:POST_STATUS_1(pu)
 *  6       i:BOARDTYPE_1(pd)   i:QSFP8_PRSNT       o:QSFP1_RESET(pu)
 *  7       i:BOARDTYPE_2(pd)   i:QSFP7_PRSNT       o:QSFP0_RESET(pu)
 *  8       o:FLASH_RY_BY(pu)   i:FLASH_RY_BY(pu)   b:GPIO8_ARB(pu)
 *  9       i:CARD_ADDRESS(pu)  i:CARD_ADDRESS(pu)  o:QSFP1_LPMODE
 *  10      i:THERM_ALERT(pu)   n:GPIO_10           o:QSFP0_LPMODE
 *  11      o:ARB_RESET         o:ARB_RESET         o:ARB_RESET(pu)
 *  12      i:DIAG_MODE(pu)     i:MFGDIAG(pu)       i:MFGDIAG(pu)
 *  13      i:BOARDTYPE_0(pd)   o:PCH_RST_BUF       o:QSFP_I2C_SEL
 *  14      o:LED_STATUS        o:LED_STATUS        i:LED_STATUS
 *  15      i:GPHY_INT(pu)      i:GPHY_INT(pu)      b:GPIO15_ARB(pu)
 *
 *  GPIO    Calistoga           MountainView        MountTian
 *  ---------------------------------------------------------------------------
 *  0       o:pd                o:pd                o:SFP_SEL0
 *  1       o:pd                o:pd                o:SFP_SEL1
 *  2       i:SN_BACKUP         i:SN_BACKUP         i:SN_BACKUP
 *  3       i:pd                i:pd                i:pd
 *  4       i:SEL_20_40G        i:SEL_20_40G        i:SEL_20_40G
 *  5       i:pd                i:pd                i:SFP_LOS
 *  6       i:pd                i:pd                i:SFP_MODPRES
 *  7       i:pd                i:pd                i:pd
 *  8       i:pd                i:pd                i:pd
 *  9       i:CARD_ADDRESS      i:CARD_ADDRESS      i:CARD_ADDRESS
 *  10      i:pd                i:pd                i:pd
 *  11      o:ARB_RESET         o:ARB_RESET         o:ARB_RESET
 *  12      i:DIAG_MODE         i:DIAG_MODE         i:DIAG_MODE
 *  13      i:pd                i:pd                i:pd
 *  14      o:LED_STATUS        o:LED_STATUS        o:LED_STATUS
 *  15      i:LANE_CONFIG       i:LANE_CONFIG       i:LANE_CONFIG

 *  GPIO    MountTian2
 *  ---------------------------------------------------------------------------
 *  0       i:pd
 *  1       i:pd
 *  2       i:pd
 *  3       i:MAINPOWER
 *  4       i:pd
 *  5       i:SFP0_PRSNT
 *  6       i:SFP1_PRSNT
 *  7       i:pd
 *  8       i:I2C1_RST
 *  9       i:CARD_ADDRESS
 *  10      i:pd
 *  11      o:ARB_RESET
 *  12      i:DIAG_MODE
 *  13      i:pd
 *  14      o:LED_STATUS
 *  15      i:pd
 *
 * input(i:), output(o:)
 * no-connect(nc), floating(fl), pull-up(pu), pull-down(pd)
 */

/******************************************************************************
 * Calimesa
 */
#define PL_CM_GPIO_I2C0_SCL         0   // bi: I2C Bus 0 Clock
#define PL_CM_GPIO_SFP_SEL0         1   // out: SFP+ module select [0]
#define PL_CM_GPIO_SFP_SEL1         2   // out: SFP+ module select [1]
#define PL_CM_GPIO_SFP_SEL2         3   // out: SFP+ module select [2]
#define PL_CM_GPIO_SFP_MODPRES      4   // in: SFP+ module present (low = pres)
#define PL_CM_GPIO_SFP_LOS          5   // in: SFP+ Loss of Signal
#define PL_CM_GPIO_BOARDTYPE_1      6   // in: board type 1 (pd)
#define PL_CM_GPIO_BOARDTYPE_2      7   // in: board type 2 (pd)
#define PL_CM_GPIO_FLASH_RY_BY      8   // in: flash ready (pu)
#define PL_CM_GPIO_CARD_ADDRESS     9   // in: mezz slot id (pu)
#define PL_CM_GPIO_THERM_ALERT      10  // in: therman monitor alert (pu)
#define PL_CM_GPIO_ARB_RESET        11  // out: 9541 reset
#define PL_CM_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0) (pu)
#define PL_CM_GPIO_BOARDTYPE_0      13  // in: board type (pd)
#define PL_CM_GPIO_LED_STATUS       14  // out: status LED
#define PL_CM_GPIO_GPHY_INT         15  // in: BMC PHY Interrupt (pu)

/*
 * Boot block initialization
 * -------------------------
 * Configure the ARB_RESET pin as output and drive it high.
 */
#define PL_CM_GPIO_DDR_BOOT_INIT ( \
            (1 << PL_CM_GPIO_ARB_RESET))
#define PL_CM_GPIO_VAL_BOOT_INIT ( \
            (1 << PL_CM_GPIO_ARB_RESET))

/*
 * Main firmware initialization
 * ----------------------------
 * Configure the Status LED as output and turn it off.
 * Configure the ARB_RESET pin as output and drive it high.
 */
#define PL_CM_GPIO_DDR_INIT ( \
            (1 << PL_CM_GPIO_LED_STATUS) | \
            (1 << PL_CM_GPIO_ARB_RESET))

#define PL_CM_GPIO_VAL_INIT ( \
            (0 << PL_CM_GPIO_LED_STATUS) | \
            (1 << PL_CM_GPIO_ARB_RESET))

/******************************************************************************
 * CalimesaB
 */
#define PL_CB_GPIO_I2C0_SCL         0   // bi: I2C Bus 0 Clock
#define PL_CB_GPIO_SFP_SEL0         1   // out: SFP+ module select [0]
#define PL_CB_GPIO_SFP_SEL1         2   // out: SFP+ module select [1]
#define PL_CB_GPIO_SFP_SEL2         3   // out: SFP+ module select [2]
#define PL_CB_GPIO_SFP_MODPRES      4   // in: SFP+ module present (low = pres)
#define PL_CB_GPIO_SFP_LOS          5   // in: SFP+ Loss of Signal (LOS=0)
#define PL_CB_GPIO_QSFP8_PRSNT      6   // in: QSFP8 present (present=0)
#define PL_CB_GPIO_QSFP7_PRSNT      7   // in: QSFP7 present (present=0)
#define PL_CB_GPIO_FLASH_RY_BY      8   // in: flash ready (pu)
#define PL_CB_GPIO_CARD_ADDRESS     9   // in: mezz slot id (pu)
#define PL_CB_GPIO_10               10  // nc
#define PL_CB_GPIO_ARB_RESET        11  // out: 9541 reset (assert=0)
#define PL_CB_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0) (pu)
#define PL_CB_GPIO_PCH_RST_BUF      13  // out: ena pch reset buffer (on=0) (pu)
#define PL_CB_GPIO_LED_STATUS       14  // out: status LED
#define PL_CB_GPIO_GPHY_INT         15  // in: BMC PHY Interrupt (pu)

/*
 * Boot block initialization
 * -------------------------
 * Configure the ARB_RESET pin as output and drive it high.
 */
#define PL_CB_GPIO_DDR_BOOT_INIT ( \
            (1 << PL_CB_GPIO_ARB_RESET))
#define PL_CB_GPIO_VAL_BOOT_INIT ( \
            (1 << PL_CB_GPIO_ARB_RESET))

/*
 * Main firmware initialization
 * ----------------------------
 * Configure the Status LED as output and turn it off.
 * Configure the ARB_RESET pin as output and drive it high.
 */
#define PL_CB_GPIO_DDR_INIT ( \
            (1 << PL_CB_GPIO_LED_STATUS) | \
            (1 << PL_CB_GPIO_ARB_RESET) | \
            (1 << PL_CB_GPIO_SFP_SEL0) | \
            (1 << PL_CB_GPIO_SFP_SEL1) | \
            (1 << PL_CB_GPIO_SFP_SEL2))

#define PL_CB_GPIO_VAL_INIT ( \
            (0 << PL_CB_GPIO_LED_STATUS) | \
            (1 << PL_CB_GPIO_ARB_RESET) | \
            (0 << PL_CB_GPIO_SFP_SEL0) | \
            (0 << PL_CB_GPIO_SFP_SEL1) | \
            (0 << PL_CB_GPIO_SFP_SEL2))

/******************************************************************************
 * Montclair
 */
#define PL_MC_GPIO_QSFP_IRQ         0   // in: wired-AND QSFP IRQ (int=0) (pu)
#define PL_MC_GPIO_QSFP1_PRSNT      1   // in: QSFP1 present (pres=0) (pu)
#define PL_MC_GPIO_QSFP0_PRSNT      2   // in: QSFP0 present (pres=0) (pu)
#define PL_MC_GPIO_PCIESW_FATAL_ERR 3   // in: PCIE switch error (err=0)
#define PL_MC_GPIO_POST_STATUS_0    4   // out: POST status 0 (pu)
#define PL_MC_GPIO_POST_STATUS_1    5   // out: POST status 1 (pu)
#define PL_MC_GPIO_QSFP1_RESET      6   // out: QSFP1 reset (assert=0) (pu)
#define PL_MC_GPIO_QSFP0_RESET      7   // out: QSFP0 reset (assert=0) (pu)
#define PL_MC_GPIO_GPIO8_ARB        8   // bi: I2C arbiter option to PSoC (pu)
#define PL_MC_GPIO_QSFP1_LPMODE     9   // out: QSFP1 low-power mode (pu)
#define PL_MC_GPIO_QSFP0_LPMODE     10  // out: QSFP0 low-power mode (pu)
#define PL_MC_GPIO_ARB_RESET        11  // out: 9541 reset (shared w/ CMC) (pu)
#define PL_MC_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0) (pu)
#define PL_MC_GPIO_QSFP_I2C_SEL     13  // out: QSFP I2C select (0=QSFP0)
#define PL_MC_GPIO_LED_STATUS       14  // out: status LED
#define PL_MC_GPIO_GPIO15_ARB       15  // bi: I2C arbiter option to PSoC (pu)

/*
 * Boot block initialization
 * -------------------------
 *  POST_STATUS_[1:0]
 *     1   1 - POST never started (e.g. corrupt flash)
 *     1   0 - POST started (in progress)
 *     0   1 - POST or Firmware failure - Firmware halted.
 *     0   0 - POST success and firmware running
 *
 * Set the POST_STATUS signals to output and drive {1, 0}
 * Configure the Status LED as output and turn it on.
 * Select QSFP 1.  Drive outputs inactive.
 */
#define PL_MC_GPIO_DDR_BOOT_INIT    ( \
            (1 << PL_MC_GPIO_POST_STATUS_0) | \
            (1 << PL_MC_GPIO_POST_STATUS_1) | \
            (1 << PL_MC_GPIO_QSFP1_RESET) | \
            (1 << PL_MC_GPIO_QSFP0_RESET) | \
            (1 << PL_MC_GPIO_QSFP1_LPMODE) | \
            (1 << PL_MC_GPIO_QSFP0_LPMODE) | \
            (1 << PL_MC_GPIO_ARB_RESET) | \
            (1 << PL_MC_GPIO_QSFP_I2C_SEL) | \
            (1 << PL_MC_GPIO_LED_STATUS))

#define PL_MC_GPIO_VAL_BOOT_INIT    ( \
            (0 << PL_MC_GPIO_POST_STATUS_0) | \
            (1 << PL_MC_GPIO_POST_STATUS_1) | \
            (1 << PL_MC_GPIO_QSFP1_RESET) | \
            (1 << PL_MC_GPIO_QSFP0_RESET) | \
            (1 << PL_MC_GPIO_QSFP1_LPMODE) | \
            (1 << PL_MC_GPIO_QSFP0_LPMODE) | \
            (1 << PL_MC_GPIO_ARB_RESET) | \
            (1 << PL_MC_GPIO_QSFP_I2C_SEL) | \
            (1 << PL_MC_GPIO_LED_STATUS))

/*
 * Main firmware initialization
 * ----------------------------
 * Configure the Status LED as output and turn it off.
 * Leave the POST_STATUS signals to output and drive {1, 0} - they will
 * be driven to {0, 0} upon successful Linux POST.
 * Select QSFP 1.  Drive outputs inactive.
 */
#define PL_MC_GPIO_DDR_INIT ( \
            (1 << PL_MC_GPIO_POST_STATUS_0) | \
            (1 << PL_MC_GPIO_POST_STATUS_1) | \
            (1 << PL_MC_GPIO_QSFP1_RESET) | \
            (1 << PL_MC_GPIO_QSFP0_RESET) | \
            (1 << PL_MC_GPIO_QSFP1_LPMODE) | \
            (1 << PL_MC_GPIO_QSFP0_LPMODE) | \
            (1 << PL_MC_GPIO_ARB_RESET) | \
            (1 << PL_MC_GPIO_QSFP_I2C_SEL) | \
            (1 << PL_MC_GPIO_LED_STATUS))

#define PL_MC_GPIO_VAL_INIT ( \
            (0 << PL_MC_GPIO_POST_STATUS_0) | \
            (1 << PL_MC_GPIO_POST_STATUS_1) | \
            (1 << PL_MC_GPIO_QSFP1_RESET) | \
            (1 << PL_MC_GPIO_QSFP0_RESET) | \
            (1 << PL_MC_GPIO_QSFP1_LPMODE) | \
            (1 << PL_MC_GPIO_QSFP0_LPMODE) | \
            (1 << PL_MC_GPIO_ARB_RESET) | \
            (1 << PL_MC_GPIO_QSFP_I2C_SEL) | \
            (0 << PL_MC_GPIO_LED_STATUS))

/******************************************************************************
 * Calistoga
 */
#define PL_CS_GPIO_0                0   // out: not used (pd)
#define PL_CS_GPIO_1                1   // out: not used (pd)
#define PL_CS_GPIO_2                2   // in: not used (sn_backup)
#define PL_CS_GPIO_3                3   // in: not used (pd)
#define PL_CS_GPIO_SEL_20G_40G      4   // in: BMC sel 20G/40G (0=20G/1=40G)
#define PL_CS_GPIO_5                5   // in: not used (pd)
#define PL_CS_GPIO_6                6   // in: not used (pd)
#define PL_CS_GPIO_7                7   // in: not used (pd)
#define PL_CS_GPIO_8                8   // in: not used (pd)
#define PL_CS_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_CS_GPIO_10               10  // in: not used (pd)
#define PL_CS_GPIO_ARB_RESET        11  // out: 9541 reset
#define PL_CS_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_CS_GPIO_13               13  // in: not used (pd)
#define PL_CS_GPIO_LED_STATUS       14  // out: status LED
#define PL_CS_GPIO_MLOM_ON_BLADE    15  // in: mLOM lane config (0=20G/1=40G)

/*
 * Boot block initialization
 * -------------------------
 * Configure the ARB_RESET pin as output and drive it high.
 */
#define PL_CS_GPIO_DDR_BOOT_INIT ( \
            (1 << PL_CS_GPIO_ARB_RESET))
#define PL_CS_GPIO_VAL_BOOT_INIT ( \
            (1 << PL_CS_GPIO_ARB_RESET))

/*
 * Main firmware initialization
 * ----------------------------
 * Configure the Status LED as output and turn it off.
 * Configure the ARB_RESET pin as output and drive it high.
 */
#define PL_CS_GPIO_DDR_INIT ( \
            (1 << PL_CS_GPIO_LED_STATUS) | \
            (1 << PL_CS_GPIO_ARB_RESET))

#define PL_CS_GPIO_VAL_INIT ( \
            (0 << PL_CS_GPIO_LED_STATUS) | \
            (1 << PL_CS_GPIO_ARB_RESET))


/*
 * Clearlake
 */
#define PL_CL_GPIO_BMC_AUTH         0   // in: BMC Authentication(Phy interrupt)
#define PL_CL_GPIO_EXPANDER         1   // in: IO Expander I/O
#define PL_CL_GPIO_PCIE_RESET       2   // in: PCIE reset input
#define PL_CL_GPIO_MAINPOWER        3   // in: Main Power On ('1'=ON)
#define PL_CL_GPIO_QSFP1_PRSNT      4   // in: QSFP1 CRUZ MODULE PRESENT
#define PL_CL_GPIO_QSFP0_PRSNT      5   // in: QSFP0 CRUZ MODULE PRESENT
#define PL_CL_GPIO_6                6   // in: not used (pd)
#define PL_CL_GPIO_7                7   // in: not used (pd)
#define PL_CL_GPIO_BMC_PHY_RESET    8   // out: broadcom PHY reset signal
#define PL_CL_GPIO_9                9   // in: not used (pd)
#define PL_CL_GPIO_10               10  // in: not used (pd)
#define PL_CL_GPIO_ARB_RESET        11  // out: I2C arbiter reset
#define PL_CL_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_CL_GPIO_13               13  // in: not used (pd)
#define PL_CL_GPIO_LED_STATUS       14  // out: status LED
#define PL_CL_GPIO_I2C_MUX_RESET    15  // out: i2c mux reset

/*
 * Boot block initialization
 * -------------------------
 * Configure the ARB_RESET pin as output and drive it high.
 */
#define PL_CL_GPIO_DDR_BOOT_INIT ( \
            (1 << PL_CL_GPIO_ARB_RESET))

#define PL_CL_GPIO_VAL_BOOT_INIT ( \
            (1 << PL_CL_GPIO_ARB_RESET))

/*
 * Main firmware initialization
 * ----------------------------
 * Configure the Status LED as output and turn it off.
 * Configure the ARB_RESET pin as output and drive it high.
 */
#define PL_CL_GPIO_DDR_INIT ( \
            (1 << PL_CL_GPIO_LED_STATUS) | \
            (1 << PL_CL_GPIO_ARB_RESET)  | \
            (1<< PL_CL_GPIO_I2C_MUX_RESET))

#define PL_CL_GPIO_VAL_INIT ( \
            (0 << PL_CL_GPIO_LED_STATUS) | \
            (1 << PL_CL_GPIO_ARB_RESET)  | \
            (1 << PL_CL_GPIO_I2C_MUX_RESET))

/******************************************************************************
 * Mountain View
 */
#define PL_MV_GPIO_0                0   // out: not used (pd)
#define PL_MV_GPIO_1                1   // out: not used (pd)
#define PL_MV_GPIO_2                2   // in: not used (sn_backup)
#define PL_MV_GPIO_3                3   // in: not used (pd)
#define PL_MV_GPIO_SEL_20G_40G      4   // in: BMC sel 20G/40G (0=20G/1=40G)
#define PL_MV_GPIO_5                5   // in: not used (pd)
#define PL_MV_GPIO_6                6   // in: not used (pd)
#define PL_MV_GPIO_7                7   // in: not used (pd)
#define PL_MV_GPIO_8                8   // in: not used (pd)
#define PL_MV_GPIO_CARD_ADDRESS     9   // in: mezz slot id
#define PL_MV_GPIO_10               10  // in: not used (pd)
#define PL_MV_GPIO_ARB_RESET        11  // out: 9541 reset
#define PL_MV_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_MV_GPIO_13               13  // in: not used (pd)
#define PL_MV_GPIO_LED_STATUS       14  // out: status LED
#define PL_MV_GPIO_MLOM_ON_BLADE    15  // in: mLOM lane config (0=20G/1=40G)


/*
 * Boot block initialization
 * -------------------------
 * Configure the ARB_RESET pin as output and drive it high.
 */
#define PL_MV_GPIO_DDR_BOOT_INIT ( \
            (1 << PL_MV_GPIO_ARB_RESET))
#define PL_MV_GPIO_VAL_BOOT_INIT ( \
            (1 << PL_MV_GPIO_ARB_RESET))

/*
 * Main firmware initialization
 * ----------------------------
 * Configure the Status LED as output and turn it off.
 * Configure the ARB_RESET pin as output and drive it high.
 */
#define PL_MV_GPIO_DDR_INIT ( \
            (1 << PL_MV_GPIO_LED_STATUS) | \
            (1 << PL_MV_GPIO_ARB_RESET))

#define PL_MV_GPIO_VAL_INIT ( \
            (0 << PL_MV_GPIO_LED_STATUS) | \
            (1 << PL_MV_GPIO_ARB_RESET))

/******************************************************************************
 * Mount Tian (Colusa: Cruz-based SIOC MLOM)
 */
#define PL_MT_GPIO_SFP_SEL0       0   // out: SFP+ module select [0]
#define PL_MT_GPIO_SFP_SEL1       1   // out: SFP+ module select [1]
#define PL_MT_GPIO_2              2   // in: not used (SN_BACKUP)
#define PL_MT_GPIO_3              3   // in: not used
#define PL_MT_GPIO_SEL_20G_40G    4   // in: BMC sel 20G/40G (0=20G/1=40G)
#define PL_MT_GPIO_SFP_LOS        5   // in: SFP+ Loss of Signal
#define PL_MT_GPIO_SFP_MODPRES    6   // in: SFP+ module present (low = pres)
#define PL_MT_GPIO_7              7   // in: not used
#define PL_MT_GPIO_8              8   // in: not used
#define PL_MT_GPIO_CARD_ADDRESS   9   // in: SIOC slot id
#define PL_MT_GPIO_10             10  // in: not used
#define PL_MT_GPIO_ARB_RESET      11  // out: PCA9541 reset
#define PL_MT_GPIO_MFGDIAG        12  // in: mfg diag mode (on=0)
#define PL_MT_GPIO_13             13  // in: not used
#define PL_MT_GPIO_LED_STATUS     14  // out: status LED
#define PL_MT_GPIO_MLOM_ON_BLADE  15  // in: mLOM lane config (0=20G/1=40G)

/*
 * Boot block initialization
 * -------------------------
 * Configure the ARB_RESET pin as output and drive it high.
 */
#define PL_MT_GPIO_DDR_BOOT_INIT ( \
                (1 << PL_MT_GPIO_ARB_RESET) )
#define PL_MT_GPIO_VAL_BOOT_INIT ( \
                (1 << PL_MT_GPIO_ARB_RESET) )

/*
 * Main firmware initialization
 * ----------------------------
 * Configure the Status LED as output and turn it off.
 * Configure the ARB_RESET pin as output and drive it high.
 * Configure the SFP select pins as output and drive them low.
 */
#define PL_MT_GPIO_DDR_INIT ( \
            (1 << PL_MT_GPIO_LED_STATUS) | \
            (1 << PL_MT_GPIO_ARB_RESET)  | \
            (1 << PL_MT_GPIO_SFP_SEL0)   | \
            (1 << PL_MT_GPIO_SFP_SEL1) )

#define PL_MT_GPIO_VAL_INIT ( \
            (0 << PL_MT_GPIO_LED_STATUS) | \
            (1 << PL_MT_GPIO_ARB_RESET)  | \
            (0 << PL_MT_GPIO_SFP_SEL0)   | \
            (0 << PL_MT_GPIO_SFP_SEL1) )

/******************************************************************************
 * Mount Tian2 (Colusa2: Cruz-based SIOC)
 */
#define PL_MT2_GPIO_0                   0   // in: not used
#define PL_MT2_GPIO_1                   1   // in: not used
#define PL_MT2_GPIO_2                   2   // in: not used
#define PL_MT2_GPIO_MAIN_POWER          3   // in: main power on
#define PL_MT2_GPIO_4                   4   // in: not used
#define PL_MT2_GPIO_SFP0_PRSNT          5   // in: QSFP0 present (low=present)
#define PL_MT2_GPIO_SFP1_PRSNT          6   // in: QSFP1 present (low=present)
#define PL_MT2_GPIO_7                   7   // in: not used
#define PL_MT2_GPIO_I2C1_RST            8   // out: I2C bus1 reset
#define PL_MT2_GPIO_CARD_ADDRESS        9   // in: SIOC slot id
#define PL_MT2_GPIO_10                  10  // in: not used
#define PL_MT2_GPIO_ARB_RESET           11  // out: Arb PCA9541 reset
#define PL_MT2_GPIO_MFGDIAG             12  // in: mfg diag mode (on=0)
#define PL_MT2_GPIO_13                  13  // in: not used
#define PL_MT2_GPIO_LED_STATUS          14  // out: Heartbeat LED
#define PL_MT2_GPIO_15                  15  // in: not used

/*
 * Boot block initialization
 * -------------------------
 * Configure the ARB_RESET pin as output and drive it high.
 */
#define PL_MT2_GPIO_DDR_BOOT_INIT ( \
                (1 << PL_MT2_GPIO_ARB_RESET) )
#define PL_MT2_GPIO_VAL_BOOT_INIT ( \
                (1 << PL_MT2_GPIO_ARB_RESET) )
/*
 * Main firmware initialization
 * ----------------------------
 * Configure the Status LED as output and turn it off.
 * Configure the ARB_RESET pin as output and drive it high.
 * Configure the QSFP select pins as output and drive them low.
 */
#define PL_MT2_GPIO_DDR_INIT ( \
            (1 << PL_MT2_GPIO_LED_STATUS) | \
            (1 << PL_MT2_GPIO_ARB_RESET)  | \
            (1 << PL_MT2_GPIO_SFP_SEL0)   | \
            (1 << PL_MT2_GPIO_SFP_SEL1) )

#define PL_MT2_GPIO_VAL_INIT ( \
            (0 << PL_MT2_GPIO_LED_STATUS) | \
            (1 << PL_MT2_GPIO_ARB_RESET)  | \
            (0 << PL_MT2_GPIO_SFP_SEL0)   | \
            (0 << PL_MT2_GPIO_SFP_SEL1) )

/*
 * Claremont
 */
#define PL_CLM_GPIO_GPHY_INT         0   // in: BMC PHY 88e1112 Interrupt
#define PL_CLM_GPIO_EXPANDER_INT     1   // in: IO Expander Interrupt
#define PL_CLM_GPIO_PCIE_RESET       2   // in: PCIE reset input
#define PL_CLM_GPIO_MAINPOWER        3   // in: Main Power On ('1'=ON)
#define PL_CLM_GPIO_QSFP1_PRSNT      4   // in: QSFP1 CRUZ MODULE PRESENT
#define PL_CLM_GPIO_QSFP0_PRSNT      5   // in: QSFP0 CRUZ MODULE PRESENT
#define PL_CLM_GPIO_BMC_AUTH         6   // in: BMC_Auth
#define PL_CLM_GPIO_7                7   // in: not used (pd)
#define PL_CLM_GPIO_BMC_PHY_RESET    8   // out: BMC PHY reset
#define PL_CLM_GPIO_9                9   // in: not used (pd)
#define PL_CLM_GPIO_10               10  // in: not used (pd)
#define PL_CLM_GPIO_ARB_RESET        11  // out: I2C arbiter reset (i2c0)
#define PL_CLM_GPIO_MFGDIAG          12  // in: mfg diag mode (on=0)
#define PL_CLM_GPIO_13               13  // in: not used (pd)
#define PL_CLM_GPIO_LED_STATUS       14  // out: status LED
#define PL_CLM_GPIO_I2C_MUX_RESET    15  // out: i2c mux reset (i2c1)

/*
 * Boot block initialization
 * -------------------------
 * Configure the ARB_RESET pin as output and drive it high.
 */
#define PL_CLM_GPIO_DDR_BOOT_INIT ( \
            (1 << PL_CLM_GPIO_ARB_RESET))

#define PL_CLM_GPIO_VAL_BOOT_INIT ( \
            (1 << PL_CLM_GPIO_ARB_RESET))

/*
 * Main firmware initialization
 * ----------------------------
 * Configure the Status LED as output and turn it off.
 * Configure the ARB_RESET pin as output and drive it high.
 */
#define PL_CLM_GPIO_DDR_INIT ( \
            (1 << PL_CLM_GPIO_LED_STATUS) | \
            (1 << PL_CLM_GPIO_ARB_RESET)  | \
            (1<< PL_CLM_GPIO_I2C_MUX_RESET))

#define PL_CLM_GPIO_VAL_INIT ( \
            (0 << PL_CLM_GPIO_LED_STATUS) | \
            (1 << PL_CLM_GPIO_ARB_RESET)  | \
            (1 << PL_CLM_GPIO_I2C_MUX_RESET))

#endif
