/*
 * Copyright 2007 Nuova Systems, Inc. All Rights Reserved
 */
#ident "$Id: pcie_regs_defs.h 43 2016-12-03 22:59:19Z buildsa $"

#ifndef _PCIE_REGS_DEFS_H_
#define _PCIE_REGS_DEFS_H_

/*
 * interrupt cause.
 */
typedef enum pcie_cause_e {
    PCIE_CAUSE_RESERVED = 0x00,
    PCIE_CAUSE_TARGET_DISABLED = 0x01,
    PCIE_CAUSE_ECRC_ERROR = 0x02,
    PCIE_CAUSE_UNSUPPORTED = 0x03,
    PCIE_CAUSE_PMV = 0x4,
    PCIE_CAUSE_POISONED = 0x05,
    PCIE_CAUSE_MESSAGE = 0x06,

    PCIE_CAUSE_CMT_DISABLED = 0x07,
    PCIE_CAUSE_CMT_MISS = 0x08,
    PCIE_CAUSE_CMT_ECC_ERROR = 0x09,
    PCIE_CAUSE_CMT_OOR = 0x0a,

    PCIE_CAUSE_CRT_DISABLED = 0x0b,
    PCIE_CAUSE_CRT_ECC_ERROR = 0x0c,
    PCIE_CAUSE_CRT_MISS = 0x0d,
    PCIE_CAUSE_CRT_IND_WR = 0x0e,
    PCIE_CAUSE_CRT_IND_RD = 0x0f,

    PCIE_CAUSE_BMT_DISABLED = 0x10,
    PCIE_CAUSE_BMT_MISS = 0x11,
    PCIE_CAUSE_BMT_ECC_ERROR = 0x12,
    PCIE_CAUSE_BMT_OOR = 0x13,

    PCIE_CAUSE_BRT_DISABLED = 0x14,
    PCIE_CAUSE_BRT_ECC_ERROR = 0x15,
    PCIE_CAUSE_BRT_MISS = 0x16,
    PCIE_CAUSE_BRT_OOR = 0x17,
    PCIE_CAUSE_BRT_IND_WR = 0x18,
    PCIE_CAUSE_BRT_IND_RD = 0x19,

    PCIE_CAUSE_COUNT
} pcie_cause_t;


/*
 * Completion actions for cfg_tar_ind_rsp (Target Indirect Response).
 *
 * These actions control what the Palo PCIe block does with the current
 * indirect request and with the pipeline of requests that are pending.
 *
 *     PCIE_RSP_ACTION_COMPLETE_CONTINUE - Complete the current indirect
 *         request and allow the pipeline to continue.  The common case.
 *
 *     PCIE_RSP_ACTION_COMPLETE_RETRY - Complete the current indirect request
 *         and restart all the pipelined requests from the top.  Use this
 *         when the current request changed some PCIe block structures, such
 *         as when a config write to a BAR causes an update to the
 *         Bar Match Table.
 *
 *     PCIE_RSP_ACTION_RETRY - Retry the current request from the top,
 *         and also retry all pipelined requests from the top.
 */
typedef enum pcie_rsp_action_e {
    PCIE_RSP_ACTION_COMPLETE_CONTINUE = 0x0,
    PCIE_RSP_ACTION_COMPLETE_RETRY = 0x1,
    PCIE_RSP_ACTION_RETRY = 0x2,
    PCIE_RSP_ACTION_DEFERRED,     /* software action */
} pcie_rsp_action_t;


/*
 * Completion status types for cfg_tar_ind_rsp (Target Indirect Response).
 */
typedef enum pcie_rsp_status_e {
    PCIE_RSP_STATUS_SUCCESS = 0,
    PCIE_RSP_STATUS_UNSUPPORTED = 1,
    PCIE_RSP_STATUS_CONFIG_RETRY = 2,
    PCIE_RSP_STATUS_ABORT = 4,
    PCIE_RSP_STATUS_DROP = 7, /* Cruz */
} pcie_rsp_status_t;


/*
 * BMT bar_type register.
 */
typedef enum pcie_bar_types_e {
    PCIE_BAR_TYPE_MEM = 0,
    PCIE_BAR_TYPE_MEM64 = 1,
    PCIE_BAR_TYPE_IO = 2,
} pcie_bar_type_t;


struct pcie_ind_req_info {
    u_int32_t pcie_data[4];
    u_int8_t tlp_data[32];
};

//
// CFG_IND_REQ_INFO_ADDR alignment and size
//
#define REQ_INFO_BUF_ALIGN       16
#define REQ_INFO_BUF_SIZE        sizeof (struct pcie_ind_req_info)


static inline void
pcie_ind_req_data_enc(u_int32_t pcie_data[],
                      const u_int8_t cause,
                      const u_int8_t cmt_hit,
                      const u_int32_t cmti,
                      const u_int8_t bmt_hit,
                      const u_int32_t bmti,
                      const u_int8_t crt_hit,
                      const u_int32_t crti,
                      const u_int8_t brt_hit,
                      const u_int32_t brti,
                      const u_int64_t addr
#ifdef CONFIG_BASE_CRUZ
                      ,
                      const u_int8_t tid,
                      const u_int8_t arq,
                      const u_int8_t shid,
                      const u_int8_t hid
#endif // CONFIG_BASE_CRUZ
                      )
{
#if defined CONFIG_BASE_PALO
    pcie_data[0] = ((cause & 0x1f) |
                    ((cmt_hit & 0x1) << 5) |
                    ((cmti & 0xff) << 6) |
                    ((bmt_hit & 0x1) << 14) |
                    ((bmti & 0xff) << 15) |
                    ((crt_hit & 0x1) << 23) |
                    ((crti & 0xff) << 24));
    pcie_data[1] = (((crti >> 8) & 0x7) |
                    ((brt_hit & 0x1) << 3) |
                    ((brti & 0x7ff) << 4) |
                    ((addr & 0x1ffff) << 15));
    pcie_data[2] = (addr >> 17) & 0x7fff;
    pcie_data[3] = 0; // reserved for future use
#elif defined CONFIG_BASE_SERENO
    pcie_data[0] = ((cause & 0x1f) |
                    ((cmt_hit & 0x1) << 5) |
                    ((cmti & 0x1ff) << 6) |
                    ((bmt_hit & 0x1) << 15) |
                    ((bmti & 0x1ff) << 16) |
                    ((crt_hit & 0x1) << 25) |
                    ((crti & 0x3f) << 26));
    pcie_data[1] = (((crti >> 6) & 0x3f) |
                    ((brt_hit & 0x1) << 6) |
                    ((brti & 0xfff) << 7) |
                    ((addr & 0x1fff) << 19));
    pcie_data[2] = (addr >> 13) & 0x7ffff;
    pcie_data[3] = 0; // reserved for future use
#elif defined CONFIG_BASE_CRUZ
//            1         2         3          4         5         6
//  01234567890123456789012345678901 23456789012345678901234567890123
// +--------------------------------+--------------------------------+
// |cccccCBmmmmmmmmmmmmmCBrrrrrrrrrr|rrrrrrraaaaaaaaaaaaaaaaaaaaaaaaa|
// +--------------------------------+--------------------------------+
//        7         8         9          10        11        12
//  45678901234567890123456789012345 67890123456789012345678901234567
// +--------------------------------+--------------------------------+
// |aaaaaaaa........................|..................tttqqqqqsshhhh|
// +--------------------------------+--------------------------------+
    const u_int32_t umti = (cmt_hit ? cmti : bmt_hit ? bmti : 0);
    const u_int32_t urti = (crt_hit ? crti : brt_hit ? brti : 0);

    pcie_data[0] = ((cause & 0x1f) |
                    ((cmt_hit & 0x1) << 5) |
                    ((bmt_hit & 0x1) << 6) |
                    ((umti & 0x1fff) << 7) |
                    ((crt_hit & 0x1) << 20) |
                    ((brt_hit & 0x1) << 21) |
                    ((urti & 0x3ff) << 22));
    pcie_data[1] = (((urti >> 10) & 0x7f) |
                    ((addr & 0x1ffffff) << 7));
    pcie_data[2] = (((addr >> 25) & 0xff));
    pcie_data[3] = (((tid & 0x7) << 18) |
                    ((arq & 0x1f) << 21) |
                    ((shid & 0x3) << 26) |
                    ((hid & 0xf) << 28));
#endif /* CONFIG_BASE */
}

static inline void
pcie_ind_req_data_dec(const u_int32_t pcie_data[],
                      u_int8_t *cause,
                      u_int8_t *cmt_hit,
                      u_int32_t *cmti,
                      u_int8_t *bmt_hit,
                      u_int32_t *bmti,
                      u_int8_t *crt_hit,
                      u_int32_t *crti,
                      u_int8_t *brt_hit,
                      u_int32_t *brti,
                      u_int64_t *addr
#ifdef CONFIG_BASE_CRUZ
                      ,
                      u_int8_t *tid,
                      u_int8_t *arq,
                      u_int8_t *shid,
                      u_int8_t *hid
#endif // CONFIG_BASE_CRUZ
                      )
{
#if defined CONFIG_BASE_PALO
    *cause = pcie_data[0] & 0x1f;
    *cmt_hit = (pcie_data[0] >> 5) & 0x1;
    *cmti = (pcie_data[0] >> 6) & 0xff;
    *bmt_hit = (pcie_data[0] >> 14) & 0x1;
    *bmti = (pcie_data[0] >> 15) & 0xff;
    *crt_hit = (pcie_data[0] >> 23) & 0x1;
    *crti = (((pcie_data[0] >> 24) & 0xff) |
             ((pcie_data[1] & 0x7) << 8));
    *brt_hit = (pcie_data[1] >> 3) & 0x1;
    *brti = (pcie_data[1] >> 4) & 0x7ff;
    *addr = (((pcie_data[1] >> 15) & 0x1ffff) |
             ((pcie_data[2] & 0x7fff) << 17));
#elif defined CONFIG_BASE_SERENO
    *cause = pcie_data[0] & 0x1f;
    *cmt_hit = (pcie_data[0] >> 5) & 0x1;
    *cmti = (pcie_data[0] >> 6) & 0x1ff;
    *bmt_hit = (pcie_data[0] >> 15) & 0x1;
    *bmti = (pcie_data[0] >> 16) & 0x1ff;
    *crt_hit = (pcie_data[0] >> 25) & 0x1;
    *crti = (((pcie_data[0] >> 26) & 0x3f) |
             ((pcie_data[1] & 0x3f) << 6));
    *brt_hit = (pcie_data[1] >> 6) & 0x1;
    *brti = (pcie_data[1] >> 7) & 0xfff;
    *addr = (((pcie_data[1] >> 19) & 0x1fff) |
             ((pcie_data[2] & 0x7ffff) << 13));
#elif defined CONFIG_BASE_CRUZ
//            1         2         3          4         5         6
//  01234567890123456789012345678901 23456789012345678901234567890123
// +--------------------------------+--------------------------------+
// |cccccCBmmmmmmmmmmmmmCBrrrrrrrrrr|rrrrrrraaaaaaaaaaaaaaaaaaaaaaaaa|
// +--------------------------------+--------------------------------+
//        7         8         9          10        11        12
//  45678901234567890123456789012345 67890123456789012345678901234567
// +--------------------------------+--------------------------------+
// |aaaaaaaa........................|..................tttqqqqqsshhhh|
// +--------------------------------+--------------------------------+
    u_int32_t umti, urti;

    *cause = pcie_data[0] & 0x1f;
    *cmt_hit = (pcie_data[0] >> 5) & 0x1;
    *bmt_hit = (pcie_data[0] >> 6) & 0x1;
    umti = (pcie_data[0] >> 7) & 0x1fff;
    *cmti = (*cmt_hit ? umti : 0);
    *bmti = (*bmt_hit ? umti : 0);
    *crt_hit = (pcie_data[0] >> 20) & 0x1;
    *brt_hit = (pcie_data[0] >> 21) & 0x1;
    urti = (((pcie_data[0] >> 22) & 0x3ff) |
            ((pcie_data[1] & 0x7f) << 10));
    *crti = (*crt_hit ? urti : 0);
    *brti = (*brt_hit ? urti : 0);
    *addr = (((u_int64_t)(pcie_data[1] >> 7) & 0x1ffffff) |
             ((u_int64_t)(pcie_data[2] & 0xff) << 25));
    *tid = (pcie_data[3] >> 18 & 0x7);
    *arq = (pcie_data[3] >> 21 & 0x1f);
    *shid = (pcie_data[3] >> 26 & 0x3);
    *hid = (pcie_data[3] >> 28 & 0xf);
#endif /* CONFIG_BASE */
}

#endif /* _PCIE_REGS_DEFS_H_ */
